Package: NetSimR
Type: Package
Title: Actuarial Functions for Non-Life Insurance Modelling
Version: 0.1.6
Author: Yiannis Parizas [aut, cre]
Authors@R: person("Yiannis", "Parizas", email = "yiannis.parizas@gmail.com", role = c("aut", "cre"))
Maintainer: Yiannis Parizas <yiannis.parizas@gmail.com>
Description: Assists actuaries and other insurance modellers in pricing,
    reserving and capital modelling for non-life insurance and
    reinsurance modelling. Provides functions that help model
    excess levels, capping and pure Incurred but not reported
    claims (pure IBNR).
    Includes capped mean, exposure curves and increased limit
    factor curves (ILFs) for LogNormal, Gamma, Pareto, Sliced
    LogNormal-Pareto and Sliced Gamma-Pareto distributions.
    Includes mean, probability density function (pdf), cumulative
    probability function (cdf) and inverse cumulative probability
    function for Sliced LogNormal-Pareto and Sliced Gamma-Pareto
    distributions.
    Includes calculating pure IBNR exposure with LogNormal and
    Gamma distribution for reporting delay.
    Includes three shiny tools, one to simulate insurance claims applying
    reinsurance structures, fit generalised linear models and fit claims
    frequency or severity distributions.
    Methods used in the package refer to
    Free for All by Yiannis Parizas (2023) <https://www.theactuary.com/2023/03/02/free-all>;
    Escaping the triangle by Yiannis Parizas (2019) <https://www.theactuary.com/features/2019/06/2019/06/05/escaping-triangle>;
    Take to excess by Yiannis Parizas (2019) <https://www.theactuary.com/features/2019/03/2019/03/06/taken-excess>.
License: GPL-3
Encoding: UTF-8
Imports: rmarkdown, shiny, shinybusy, future.apply, scales, future,
        methods, DBI, RMySQL, RODBC, RPostgreSQL, RSQLite, plotly,
        shinyjs, MASS, fitdistrplus, Pareto, data.table, reactable,
        bslib
Suggests: knitr, crch, testthat
VignetteBuilder: knitr
RoxygenNote: 8.0.0
NeedsCompilation: no
Packaged: 2026-07-12 15:20:44 UTC; yiann
Repository: CRAN
Date/Publication: 2026-07-12 15:50:15 UTC
Built: R 4.5.2; ; 2026-07-12 18:50:39 UTC; unix

# Generated by roxygen2: do not edit by hand

export(ExposureCurveGamma)
export(ExposureCurveLNorm)
export(ExposureCurvePareto)
export(ExposureCurveSlicedGammaPareto)
export(ExposureCurveSlicedLNormPareto)
export(GammaCappedMean)
export(IGamma)
export(ILFGamma)
export(ILFLNorm)
export(ILFPareto)
export(ILFSlicedGammaPareto)
export(ILFSlicedLNormPareto)
export(LNormCappedMean)
export(ParetoCappedMean)
export(ParetoCappedMeanCalc)
export(PureIBNRGamma)
export(PureIBNRLNorm)
export(SlicedGammaParetoCappedMean)
export(SlicedGammaParetoMean)
export(SlicedLNormParetoCappedMean)
export(SlicedLNormParetoMean)
export(apply_deductible_limit)
export(dSlicedGammaPareto)
export(dSlicedLNormPareto)
export(erf)
export(pSlicedGammaPareto)
export(pSlicedLNormPareto)
export(qSlicedGammaPareto)
export(qSlicedLNormPareto)
export(run_shiny_distribution_fitting_tool)
export(run_shiny_glm_fitting_tool)
export(run_shiny_simulator)
export(simulate_function)
import(DBI)
import(MASS)
import(Pareto)
import(RODBC)
import(RPostgreSQL)
import(data.table)
import(fitdistrplus)
import(future.apply)
import(methods)
import(reactable)
import(rmarkdown)
import(scales)
import(shiny)
import(shinybusy)
import(stats)
import(utils)
importFrom(RMySQL,MySQL)
importFrom(RSQLite,SQLite)
importFrom(bslib,bs_theme)
importFrom(bslib,card)
importFrom(bslib,card_body)
importFrom(bslib,card_header)
importFrom(bslib,font_google)
importFrom(bslib,layout_columns)
importFrom(bslib,nav_panel)
importFrom(bslib,navbar_options)
importFrom(bslib,page_navbar)
importFrom(future,multisession)
importFrom(future,plan)
importFrom(future,sequential)
importFrom(plotly,add_bars)
importFrom(plotly,add_lines)
importFrom(plotly,layout)
importFrom(plotly,plot_ly)
importFrom(plotly,plotlyOutput)
importFrom(plotly,renderPlotly)
importFrom(shinyjs,useShinyjs)

# NetSimR

NetSimR is an R package providing functions which are usually used in insurance
pricing, capital modelling and claims reserving, including:

- Capped mean
- Exposure curves from severity curves
- Increased limit factor curves from severity curves
- Sliced distribution functions
- Pure IBNR estimation from reporting delay distributions

Three shiny tools are also included in this package:
- Distribution fitting tool allows fitting claims distributions for frequency or severity.
- GLM fitting tool allows fitting different glm distributions.
- Claims simulation tool allows simulating fitted claims applying various reinsurance structures.

## ----setup, include=FALSE-----------------------------------------------------
knitr::opts_chunk$set(echo = TRUE)

## ----pure IBNR example--------------------------------------------------------
library("NetSimR")
#set the seed
set.seed(0)

#Set the reporting delay distribution parameters
mu <- 3
sigma <- 1

#Generate reporting delays and preview the distribution
x<-rlnorm(1000,mu,sigma)
summary(x)
hist(x, breaks = 100)

## ----pure IBNR example continued----------------------------------------------
#Generate dates data
PoliciesPerDay<-100
PeriodLength<-365*2
InceptionDateStart<-as.Date("2011/1/1")
InceptionDate<-seq(InceptionDateStart, by = "day", length.out = PeriodLength)
InceptionDate<-rep(InceptionDate,PoliciesPerDay)
#assume all policies are yearly
DayPolicyDuration<-365
ExpiryDate<-InceptionDate+DayPolicyDuration
ValuationDate<-max(InceptionDate)
Data<-data.frame(InceptionDate,ExpiryDate)
summary(Data)
ValuationDate

#Generate Claims Counts Data and reporting delays
#Assume only 1 claim per policy possible with probability of claiming 10%
ClaimFreq<-0.1
Data$ClaimCount<-ifelse(runif(nrow(Data))<ClaimFreq,1,0)
#generate accident date assuming uniformly happening throughout the year
Data$AccidentDate<-Data$InceptionDate+runif(nrow(Data))*(Data$ExpiryDate-Data$InceptionDate)
#generate reporting delays
Data$ReportingDelay<-round(Data$ClaimCount*rlnorm(nrow(Data),mu,sigma),1)
#generate reporting days
Data$NotificationDate<-Data$AccidentDate+Data$ReportingDelay
#remove claims reported beyond valuation day
Data$ClaimReportedBeforeValuation<-ifelse(Data$NotificationDate>ValuationDate,0,Data$ClaimCount)
#Maximum reporting delays
Data$MaxReportingDelay<-as.numeric(ValuationDate-Data$AccidentDate)+1

#Earned Duration
Data$EarnedDuration<-ifelse(Data$ExpiryDate>ValuationDate,as.numeric(ValuationDate-Data$InceptionDate),as.numeric(Data$ExpiryDate-Data$InceptionDate))/365

#Filter claims only data
ModelData<-Data[Data$ClaimReportedBeforeValuation==1,]

#model reporting delays with a linear model
lm<-lm(log(ModelData$ReportingDelay)~1)
#summary(lm)
cbind(coefficients(lm),summary(lm)$sigma)

#model reporting delays with a truncated model (splitting to monthly periods would be more accurate in terms of maximum reporting delay)
library("crch")
rTLm<-crch(log(ModelData$ReportingDelay)~1, truncated = TRUE, right = log(ModelData$MaxReportingDelay), dist = "gaussian", link.scale = "identity")
#summary(rTLm)
coefficients(rTLm)

#Generate and predict pure IBNR and UPR Claims
Data$PureIBNRClaims<-ifelse(Data$AccidentDate>ValuationDate,0,Data$ClaimCount-Data$ClaimReportedBeforeValuation)
Data$UPRClaims<-Data$ClaimCount-Data$ClaimReportedBeforeValuation-Data$PureIBNR
PureIBNRPrediction<-PureIBNRLNorm(Data$InceptionDate,Data$ExpiryDate,ValuationDate,coefficients(rTLm)[1],coefficients(rTLm)[2])
Data$PredictedPureIBNRYears<-PureIBNRPrediction$PureIBNRDuration/DayPolicyDuration
Data$PredictedUPRYears<-PureIBNRPrediction$UnearnedDuration/DayPolicyDuration
Data$NetEarnedDuration<-1-Data$PredictedUPRYears-Data$PredictedPureIBNRYears+0.00001

#Predict Claim Frequency with reduced exposure to adjust for pure IBNR
AdjFreqGLM<-glm(Data$ClaimReportedBeforeValuation~1, offset = log(Data$NetEarnedDuration), family = "poisson")
#summary(AdjFreqGLM)
PredictedFrequency<-exp(AdjFreqGLM$coefficients)

#Compare as if not adjusting for pure IBNR
UnAdjFreqGLM<-glm(Data$ClaimReportedBeforeValuation~1, offset = log(Data$EarnedDuration+0.00001), family = "poisson")
#summary(UnAdjFreqGLM)

cbind(AdjustedModel=round(exp(AdjFreqGLM$coefficients),3),UnAdjustedModel=round(exp(UnAdjFreqGLM$coefficients),3),Actual=ClaimFreq)


#Compare UPR Claim predictions
cbind(Actual=sum(Data$UPRClaims),Predicted=round(sum(Data$PredictedUPRYears)*PredictedFrequency,0))

#Compare Pure IBNR Claim predictions
cbind(Actual=sum(Data$PureIBNRClaims),Predicted=round(sum(Data$PredictedPureIBNRYears)*PredictedFrequency,0), Theoretical=327)

#Theoretical comes from re-running with the theoretical parameter values

#Compare Pure IBNR predictions with Chain Ladder
ClaimsAY1RY1<-sum(Data[Data$AccidentDate<as.Date("2012/1/1") & Data$NotificationDate<as.Date("2012/1/1"), ]$ClaimReportedBeforeValuation)

ClaimsAY1RY2<-sum(Data[Data$AccidentDate<as.Date("2012/1/1") & Data$NotificationDate<as.Date("2013/1/1"), ]$ClaimReportedBeforeValuation)

ClaimsAY2RY1<-sum(Data$ClaimReportedBeforeValuation)-ClaimsAY1RY1-ClaimsAY1RY2

PredictedPureIBNRClaimsChainLadder<-(ClaimsAY1RY2/ClaimsAY1RY1-1)*ClaimsAY2RY1

cbind(Actual=sum(Data$PureIBNRClaims),PredictedGLM=round(sum(Data$PredictedPureIBNRYears)*PredictedFrequency,0),PredictedCL=round(PredictedPureIBNRClaimsChainLadder,0))

#Note: Chain ladder can predict better if periods brake down to months or quarters

#Sliced LogNormal-Pareto claim Severity assumption
mu=5.6
sigma=1.65
s=10000
alpha=1.5

#Simulate the reserve distribution
numberOfSimulations=10000
SimulatedYears <- numeric(length = numberOfSimulations)
ExpectedFrequency <- sum(Data$PredictedPureIBNRYears)*PredictedFrequency

for (i in 1 :numberOfSimulations){
  SimulatedYears[i]=round(sum(qSlicedLNormPareto(runif(rpois(1,ExpectedFrequency)),mu,sigma,s,alpha)),0)
}

#Visualising pure IBNR reserve distribution
head(SimulatedYears)
hist(SimulatedYears, breaks = 1000, xlim = c(0,1.5e6))
summary(SimulatedYears)

#Note: the strength of the method illustrated is that it can have GLM factors for reporting delay, frequency and severity, and run the simulations per risk

## ----setup, include=FALSE-----------------------------------------------------
knitr::opts_chunk$set(echo = TRUE)

## ----capped mean example------------------------------------------------------
#required packages
library("NetSimR")
library("crch")

#Set parameters
n<-10000
mu<-6
sigma<-1.7
Cap <- 10000

#Set seed to keep simulations constant
set.seed(10)

#Simulate data
x<-round(rlnorm(n,mu,sigma),0)
head(x)
summary(x)
plot(x)
hist(x, breaks = 400, xlim = c(0,10000))


#cap data
z<-ifelse(x>Cap,Cap,x)
head(z)
summary(z)
plot(z)
hist(z, breaks = 200)


#fit linear regression
lmLinear<-lm(log(z)~1)
summary(lmLinear)


#fit right censored regression
lmCensored<-crch(log(z)~1, right = log(Cap), link.scale="identity", dist = "gaussian")
summary(lmCensored)


#Compare regression outputs
cbind(coefficients(lmLinear),summary(lmLinear)$sigma)
coefficients(lmCensored)


#Compare regressions' attritional capped cost to empirical
round(cbind(
  Empirical=mean(z)
  ,LinearModel=exp(coefficients(lmLinear)+0.5*summary(lmLinear)$sigma*summary(lmLinear)$sigma)
  ,CensoredModel=LNormCappedMean(Cap,coefficients(lmCensored)[1],coefficients(lmCensored)[2])
),0)

## ----Exposure curve LogNormal-------------------------------------------------
ExposureCurveLNorm(1000,5,1.6)

## ----ILF curve LogNormal------------------------------------------------------
ILFLNorm(1000,1500,5,1.6)

## ----setup, include=FALSE-----------------------------------------------------
knitr::opts_chunk$set(echo = TRUE)

## ----sliced mean--------------------------------------------------------------
library("NetSimR")
round(SlicedLNormParetoMean(6,1.6,10000,1.2),0)

## ----sliced distribution simulation-------------------------------------------
#set seed
set.seed(1)

#set parameters
numberOfSimulations=20000
freq=15
mu=6
sigma=1.6
s=1000
alpha=1.2
Deductible=10000
limit=100000
XoLCededClaims <- numeric(length = numberOfSimulations)

#loop simulates frequency then severity and applies layer deductible and limit
for (i in 1 :numberOfSimulations){
  x=qSlicedLNormPareto(runif(rpois(1,freq)),mu,sigma,s,alpha)
  x=ifelse(x<Deductible,0,ifelse(x-Deductible>limit,limit,x-Deductible))
  XoLCededClaims[i]=round(sum(x),0)
}

#Visualising Simulation results
head(XoLCededClaims)
hist(XoLCededClaims, breaks = 100, xlim = c(0,100000))
summary(XoLCededClaims)
  

#set seed and number of simulations
set.seed(10)
n=10*1000*1000

#set parameters
cap=10*1000
gshape=1.5
rate=0.0004
SlicePoint=5000
shape=1.5

#Capped mean
x<-rgamma(n,gshape,rate)
x<-ifelse(x<SlicePoint,x,SlicePoint/(runif(n)^(1/shape)))
hist(x, breaks = 200000, xlim = c(0,3e4))
mean(x)
SlicedGammaParetoMean(gshape,rate,SlicePoint,shape)
y<-ifelse(x<cap,x,cap)
hist(y)
mean(y)
SlicedGammaParetoCappedMean(cap,gshape,rate,SlicePoint,shape)


#exposure curve
mean(y)/mean(x)
ExposureCurveSlicedGammaPareto(cap,gshape,rate,SlicePoint,shape)


#ILF
z<-ifelse(y>cap/2,cap/2,y)
mean(y)/mean(z)
ILFSlicedGammaPareto(cap/2,cap,gshape,rate,SlicePoint,shape)
ILFSlicedGammaPareto(cap/2,c(cap,cap*1.1),gshape,rate,SlicePoint,shape)

##################################
#Sliced LogNormal-Pareto Functions
##################################

#' Sliced LogNormal Pareto mean
#'
#' @param mu A real number -  the first parameter of the attritional Claim Severity's LogNormal distribution.
#' @param sigma A positive real number -  the second parameter of the attritional Claim Severity's LogNormal distribution.
#' @param SlicePoint A positive real number - the slice point and the scale parameter of the tail Claim Severity's Pareto distribution.
#' @param shape A positive real number - the shape parameter of the tail Claim Severity's Pareto distribution.
#' @return The mean of the claim severity with an attritional claim LogNormal distribution with parameters \code{mu} and \code{sigma} and a large claim Pareto distribution with parameters \code{SlicePoint} and \code{shape}.
#' @export
#' @examples
#' SlicedLNormParetoMean(6,1.5,1000,1.2)
#' SlicedLNormParetoMean(6.5,1.4,2000,1.6)
#' SlicedLNormParetoMean(7,1.6,3000,1.4)
SlicedLNormParetoMean<-function(mu, sigma, SlicePoint, shape){
  df<-data.frame(mu, sigma, SlicePoint, shape)
  ifelse(df$shape>1
         ,LNormCappedMean(SlicePoint, mu, sigma)+plnorm(SlicePoint, mu, sigma, FALSE)*((shape*SlicePoint)/(shape-1)-SlicePoint)
         ,Inf
  )
}



#' Sliced LogNormal Pareto capped mean
#'
#' @param cap A positive real number -  the claim severity cap.
#' @param mu A real number -  the first parameter of the attritional Claim Severity's LogNormal distribution.
#' @param sigma A positive real number -  the second parameter of the attritional Claim Severity's LogNormal distribution.
#' @param SlicePoint A positive real number - the slice point and the scale parameter of the tail Claim Severity's Pareto distribution.
#' @param shape A positive real number - the shape parameter of the tail Claim Severity's Pareto distribution.
#' @return The mean of the claim severity capped at \code{cap} with an attritional claim LogNormal distribution with parameters \code{mu} and \code{sigma} and a large claim Pareto distribution with parameters \code{SlicePoint} and \code{shape}.
#' @export
#' @examples
#' SlicedLNormParetoCappedMean(1200,6,1.5,1000,1.2)
#' SlicedLNormParetoCappedMean(2500,6.5,1.4,2000,1.6)
#' SlicedLNormParetoCappedMean(4000,7,1.6,3000,1.4)
SlicedLNormParetoCappedMean<-function(cap,mu, sigma, SlicePoint, shape){
  df<-data.frame(cap,mu, sigma, SlicePoint, shape)
  ifelse(df$cap<=df$SlicePoint
         ,LNormCappedMean(cap, mu, sigma)
         ,LNormCappedMean(SlicePoint, mu, sigma)+plnorm(SlicePoint,mu,sigma,lower.tail = FALSE)*(ParetoCappedMean(cap, SlicePoint, shape)-SlicePoint)
  )
}



#' Exposure Curve from a Sliced LogNormal Pareto severity distribution
#'
#' @param x A positive real number -  the claim amount where the exposure curve will be evaluated.
#' @param mu A real number -  the first parameter of the attritional Claim Severity's LogNormal distribution.
#' @param sigma A positive real number -  the second parameter of the attritional Claim Severity's LogNormal distribution.
#' @param SlicePoint A positive real number - the slice point and the scale parameter of the tail Claim Severity's Pareto distribution.
#' @param shape A positive real number - the shape parameter of the tail Claim Severity's Pareto distribution.
#' @return The value of the Exposure curve at \code{x} with an attritional claim LogNormal distribution with parameters \code{mu} and \code{sigma} and a large claim Pareto distribution with parameters \code{SlicePoint} and \code{shape}.
#' @export
#' @examples
#' ExposureCurveSlicedLNormPareto(1200,6,1.5,1000,1.2)
#' ExposureCurveSlicedLNormPareto(4000,7,1.6,3000,1.4)
ExposureCurveSlicedLNormPareto<-function(x, mu, sigma, SlicePoint, shape){
  df<-data.frame(x, mu, sigma, SlicePoint, shape)
  ifelse(df$shape>1
         ,SlicedLNormParetoCappedMean(x, mu, sigma, SlicePoint, shape)/SlicedLNormParetoMean(mu, sigma, SlicePoint, shape)
         ,0
  )
}



#' Increased Limit Factor Curve from a Sliced LogNormal Pareto severity distribution
#'
#' @param xLow A positive real number -  the claim amount where the Limit Factor Curve will be evaluated from.
#' @param xHigh A positive real number -  the claim amount where the Limit Factor Curve will be evaluated to.
#' @param mu A real number -  the first parameter of the attritional Claim Severity's LogNormal distribution.
#' @param sigma A positive real number -  the second parameter of the attritional Claim Severity's LogNormal distribution.
#' @param SlicePoint A positive real number - the slice point and the scale parameter of the tail Claim Severity's Pareto distribution.
#' @param shape A positive real number - the shape parameter of the tail Claim Severity's Pareto distribution.
#' @return The value of the Increased Limit Factor curve from \code{xLow} to \code{xHigh} with an attritional claim LogNormal distribution with parameters \code{mu} and \code{sigma} and a large claim Pareto distribution with parameters \code{SlicePoint} and \code{shape}.
#' @export
#' @examples
#' ILFSlicedLNormPareto(800,1200,6,1.5,1000,1.2)
#' ILFSlicedLNormPareto(2000,4000,7,1.6,3000,1.4)
ILFSlicedLNormPareto<-function(xLow,xHigh, mu, sigma, SlicePoint, shape){
  SlicedLNormParetoCappedMean(xHigh, mu, sigma, SlicePoint, shape)/SlicedLNormParetoCappedMean(xLow, mu, sigma, SlicePoint, shape)
}



#' The cumulative density function (cdf) of a Sliced LogNormal Pareto severity distribution
#'
#' @param x A positive real number -  the claim amount where the cumulative density function (cdf) will be evaluated.
#' @param mu A real number -  the first parameter of the attritional Claim Severity's LogNormal distribution.
#' @param sigma A positive real number -  the second parameter of the attritional Claim Severity's LogNormal distribution.
#' @param SlicePoint A positive real number - the slice point and the scale parameter of the tail Claim Severity's Pareto distribution.
#' @param shape A positive real number - the shape parameter of the tail Claim Severity's Pareto distribution.
#' @return The value of the cumulative density function (cdf) at \code{x} with an attritional claim LogNormal distribution with parameters \code{mu} and \code{sigma} and a large claim Pareto distribution with parameters \code{SlicePoint} and \code{shape}.
#' @export
#' @examples
#' pSlicedLNormPareto(1200,6,1.5,1000,1.2)
#' pSlicedLNormPareto(4000,7,1.6,3000,1.4)
pSlicedLNormPareto<-function(x, mu, sigma, SlicePoint, shape){
  ifelse(x>SlicePoint
         ,plnorm(SlicePoint, mu, sigma)+plnorm(SlicePoint, mu, sigma, FALSE)*(1-(SlicePoint/x)^shape)
         ,plnorm(x, mu, sigma)
  )
}



#' The inverse cumulative density function of a Sliced LogNormal Pareto severity distribution
#'
#' @param q A real number between 0 and 1 -  the probability where the inverse cumulative density function will be evaluated.
#' @param mu A real number -  the first parameter of the attritional Claim Severity's LogNormal distribution.
#' @param sigma A positive real number -  the second parameter of the attritional Claim Severity's LogNormal distribution.
#' @param SlicePoint A positive real number - the slice point and the scale parameter of the tail Claim Severity's Pareto distribution.
#' @param shape A positive real number - the shape parameter of the tail Claim Severity's Pareto distribution.
#' @return The value of the inverse cumulative density function at \code{q} with an attritional claim LogNormal distribution with parameters \code{mu} and \code{sigma} and a large claim Pareto distribution with parameters \code{SlicePoint} and \code{shape}.
#' @export
#' @examples
#' qSlicedLNormPareto(0.5,6,1.5,1000,1.2)
#' qSlicedLNormPareto(0.7,7,1.6,3000,1.4)
qSlicedLNormPareto<-function(q, mu, sigma, SlicePoint, shape){
  lp<-plnorm(SlicePoint, mu, sigma)
  up<-1-lp
  ifelse(q>lp
         ,SlicePoint/((1-((q-lp)/up))^(1/shape))
         ,qlnorm(q,mu, sigma)
  )
}



#' The probability density function (pdf) of a Sliced LogNormal Pareto severity distribution
#'
#' @param x A positive real number -  the claim amount where the probability density function (pdf) will be evaluated.
#' @param mu A real number -  the first parameter of the attritional Claim Severity's LogNormal distribution.
#' @param sigma A positive real number -  the second parameter of the attritional Claim Severity's LogNormal distribution.
#' @param SlicePoint A positive real number - the slice point and the scale parameter of the Claim Severity's Pareto distribution.
#' @param shape A positive real number - the shape parameter of the Claim Severity's Pareto distribution.
#' @return The value of the probability density function (pdf) at \code{x} with an attritional claim LogNormal distribution with parameters \code{mu} and \code{sigma} and a large claim Pareto distribution with parameters \code{SlicePoint} and \code{shape}.
#' @export
#' @examples
#' dSlicedLNormPareto(1200,6,1.5,1000,1.2)
#' dSlicedLNormPareto(4000,7,1.6,3000,1.4)
dSlicedLNormPareto<-function(x, mu, sigma, SlicePoint, shape){
  ifelse(x>SlicePoint
         ,plnorm(SlicePoint, mu, sigma, FALSE)*(shape*SlicePoint^shape)/(x^(shape+1))
         ,dlnorm(x, mu, sigma)
  )
}

#set seed and number of simulations
set.seed(1)
n=10*1000*1000

#set parameters
cap=5000
mu=6.7
sigma=1.65
SlicePoint=3000
shape=1.5

#Capped mean
x<-rlnorm(n,mu,sigma)
x<-ifelse(x<SlicePoint,x,SlicePoint/(runif(n)^(1/shape)))
hist(x, breaks = 20000, xlim = c(0,1e4))
mean(x)
SlicedLNormParetoMean(mu,sigma,SlicePoint,shape)
y<-ifelse(x<cap,x,cap)
hist(y)
mean(y)
SlicedLNormParetoCappedMean(cap,mu,sigma,SlicePoint,shape)


#exposure
mean(y)/mean(x)
ExposureCurveSlicedLNormPareto(cap,mu,sigma,SlicePoint,shape)


#ILF
z<-ifelse(y>cap/2,cap/2,y)
mean(y)/mean(z)
ILFSlicedLNormPareto(cap/2,cap,mu,sigma,SlicePoint,shape)
ILFSlicedLNormPareto(cap/2,c(cap,cap*1.1),mu,sigma,SlicePoint,shape)

## ----setup, include=FALSE-----------------------------------------------------
knitr::opts_chunk$set(echo = TRUE)

## ----capped mean example, eval=requireNamespace("crch", quietly = TRUE)-------
library("NetSimR")

#Set parameters
n<-10000
mu<-6
sigma<-1.7
Cap <- 10000

#Set seed to keep simulations constant
set.seed(10)

#Simulate data
x<-round(rlnorm(n,mu,sigma),0)
head(x)
summary(x)
plot(x)
hist(x, breaks = 400, xlim = c(0,10000))


#cap data
z<-ifelse(x>Cap,Cap,x)
head(z)
summary(z)
plot(z)
hist(z, breaks = 200)


#fit linear regression
lmLinear<-lm(log(z)~1)
summary(lmLinear)


#fit right censored regression
lmCensored<-crch::crch(log(z)~1, right = log(Cap), link.scale="identity", dist = "gaussian")
summary(lmCensored)


#Compare regression outputs
cbind(coefficients(lmLinear),summary(lmLinear)$sigma)
coefficients(lmCensored)


#Compare regressions' attritional capped cost to empirical
round(cbind(
  Empirical=mean(z)
  ,LinearModel=exp(coefficients(lmLinear)+0.5*summary(lmLinear)$sigma*summary(lmLinear)$sigma)
  ,CensoredModel=LNormCappedMean(Cap,coefficients(lmCensored)[1],coefficients(lmCensored)[2])
),0)

## ----Exposure curve LogNormal-------------------------------------------------
ExposureCurveLNorm(1000,5,1.6)

## ----ILF curve LogNormal------------------------------------------------------
ILFLNorm(1000,1500,5,1.6)

#' UI file for the Shiny glm fitting tool
#'
#' @return Returns the UI code for the shiny application.

distribution_fitting_tool_UI = fluidPage(



  headerPanel(div("NetDisFit by NetSimR"), windowTitle = "NetSim Distribution Fitting Tool"),

  tabsetPanel(
    tabPanel(
      "Welcome",
      h3("Welcome to the NetDisFit Tool"),
      p("This tool allows you to analyse and fit probability distributions to your data."),
      p("Created by Yiannis Parizas. For more information, please visit my ", a("LinkedIn profile", href = "https://www.linkedin.com/in/yiannisparizas/"), "."),
      p("You can also reach out to me via email at ", a("yiannis.parizas@gmail.com", href = "mailto:yiannis.parizas@gmail.com"), "."),
      p("Please reach out if you have any feedback or encounter any bugs.")
    ),

    tabPanel(
      "Data Upload",
      sidebarLayout(
        sidebarPanel(
          fileInput("file1", "Choose CSV/text File", accept = c(
            'text/csv',
            'text/comma-separated-values,text/plain',
            '.csv'
          )),
          checkboxInput("data_includes_header", "CSV Has Header", value = TRUE),
          radioButtons('sep', 'Separator', c(Comma=',', Semicolon=';', Tab='\t'), ','),
          radioButtons('quote', 'Quote', c('Double Quote'='"', 'Single Quote'="'", None=''), '"'),
        ),
        mainPanel(
          br(),
          reactable::reactableOutput("data_table")
        )
      )
    ),


    tabPanel(
      "Frequency Analysis",
      sidebarLayout(
        sidebarPanel(
          selectInput(inputId='counts_var', label='Claim Counts variable',choices=""),
          checkboxInput("counts_weighted_var", "Weigthed counts fit", value = F),
          conditionalPanel(
            condition = "input.counts_weighted_var",
            selectInput(inputId='counts_weights_var', label='Weigths variable',choices="")
          ),
          actionButton("execute_freq_analysis", "Execute Analysis"),
          br(),
          br(),
          p('Note: the above will remove any negative or non numeric values'),
          br(),
          sliderInput(inputId = "count_hist_bins",
                      label = "Histogram number of bins:",
                      min = 1,
                      max = 100,
                      value = 20),
          selectInput("FreqDistri", "Selected Frequency Distribution",
                      c("Poisson" = "Poisson",
                        "Negative Binomial" = "NegativeBinomial")),
          tabPanel(title="Claim Count Summary", strong("Frequency parameters fitted"), verbatimTextOutput("selected_freq_params"))
        ),
        mainPanel(
          br(),
          tabPanel(title="Claim Count Histogram",plotly::plotlyOutput('count_hist')),
          tabPanel(title="Claim Count CDF vs model fit",plotly::plotlyOutput('freq_fit_plot')),
          tabPanel(title="Claim Count Summary", h5(strong("Frequency data summary"), align="center"), verbatimTextOutput("freq_summary")),
          tabPanel(title="Distribution Proposal", h5(strong("Distribution Proposal"), align="center"), verbatimTextOutput("distribution_proposal")),
          tabPanel(title="Selected distribution Summary", h5(strong("Selected distribution Summary"), align="center"), verbatimTextOutput("selected_distribution_summary"))
        )
      )
    ),



    tabPanel(
      "Severity Analysis",
      sidebarLayout(
        sidebarPanel(
          selectInput(inputId='severity_var', label='Claim severity variable',choices=""),
          actionButton("execute_sev_analysis", "Execute Analysis"),
          br(),
          br(),
          p('Note: the above will remove any negative, zero or non numeric values'),
          br(),
          sliderInput(inputId = "severity_hist_bins",
                      label = "Histogram number of bins:",
                      min = 1,
                      max = 200,
                      value = 20),
          checkboxInput("sev_fit_log_scale", "Log Scale for Severity cdf", value = F),
          tabPanel(title="Severity parameters summary",
                   strong("Severity parameters fitted"),
                   verbatimTextOutput("sev_param_summary"))
        ),
        mainPanel(
          tabPanel(title="Claim Severity Histogram and cdf fit", plotly::plotlyOutput('sev_hist')),
          tabPanel(title="Claim Severity Summary", h5(strong("Severity data summary"), align="center"), verbatimTextOutput("sev_summary")),
          tabPanel(title="Claim Severity fit log scale", plotly::plotlyOutput('sev_fit_plot'))
        )
      )
    ),



    tabPanel(
      "Sliced Severity Analysis",
      sidebarLayout(
        sidebarPanel(
          selectInput(inputId='sliced_sev_var', label='Sliced Claim severity variable',choices=""),
          actionButton("execute_sliced_sev_analysis", "Execute Analysis"),
          br(),
          br(),
          p('Note: the above will remove any negative, zero or non numeric values'),
          br(),
          sliderInput(
            inputId = "slicing_point_left",
            label = "Slicing Point",
            min = 0,
            max = 10,
            value = 5
          ),
          sliderInput(
            inputId = "slicing_point_right",
            label = "Second Slicing Point",
            min = 0,
            max = 20,
            value = 10
          ),
          checkboxInput("sev_cens_fit_log_scale", "Log Scale for Severity cdf", value = F),
          tabPanel(
            title="Sliced Severity parameters summary",
            strong("Sliced Severity parameters fitted"),
            verbatimTextOutput("slc_sev_fitted_param_summary")
          )
        ),
        mainPanel(
          tabPanel(title="Mean Excess Function of Claim Severity",plotly::plotlyOutput('mean_excess_func_plot')),
          tabPanel(title="Claim Severity Cumulative Density Function",plotly::plotlyOutput('sliced_sev_cdf_plot'))
        )
      )
    ),


    tabPanel(
      "Piecewise Pareto fit",
      sidebarLayout(
        sidebarPanel(
          selectInput(inputId='piecewise_pareto_var', label='Piecewise Pareto severity variable',choices=""),
          actionButton("execute_piecewise_sev_analysis", "Execute Analysis"),
          sliderInput(inputId = "num_pareto_slices", label = "Number of Inputs (1-6):", min = 1, max = 6, value = 3),
          uiOutput("pareto_slider_inputs"),
          br(),
          br(),
          p('Note: the above will remove any negative, zero or non numeric values'),
          br(),
          checkboxInput("piecewise_pareto_fit_log_scale", "Log Scale for Severity cdf", value = F)
        ),
        mainPanel(
          tableOutput("fitted_sliced_pareto"),
          verbatimTextOutput("piecewise_pareto_ks_test"),
          tabPanel(title="Claim Severity Cumulative Density Function", plotly::plotlyOutput('piewcewise_pareto_cdf_plot'))
        )
      )
    )
  )
)

#set seed and number of simulations
set.seed(1)
n=10*1000*1000

#set gamma parameters
mu=6.7
sigma=1.65
SlicePoint=300
shape=1.5

#test pdf
x<-rlnorm(n,mu,sigma)
x<-ifelse(x<SlicePoint,x,SlicePoint/(runif(n)^(1/shape)))
hist(x, breaks = 200000, xlim = c(0,3e3), probability = T)
lines(0:1e4,dSlicedLNormPareto(0:1e4,mu,sigma,SlicePoint,shape), col="red")

#test cdf
step<-0.001
plot(log(quantile(x,seq(0+step, 1-step, step)))
     ,pSlicedLNormPareto(quantile(x,seq(0+step, 1-step, step)),mu,sigma,SlicePoint,shape)
     ,type = "l"
  )
lines(log(quantile(x,seq(0+step, 1-step, step))),seq(0+step, 1-step, step), type = "l", col="red")


#test inverese cdf
plot(log(qSlicedLNormPareto(seq(0+step, 1-step, step),mu,sigma,SlicePoint,shape))
     ,seq(0+step, 1-step, step)
     ,type = "l"
)
lines(log(quantile(x,seq(0+step, 1-step, step))),seq(0+step, 1-step, step), type = "l", col="red")

#' NetSimR: A non-life insurance package for computating various statistics.
#'
#' The NetSimR package provides three categories of functions:
#' \enumerate{
#' \item Capped means, Exposure and ILF curve from various severity distributions
#' \item Pure IBNR and UPR earned periods
#' \item Sliced distributions
#' }
#'
#' @section NetSimR mean functions:
#' \code{\link{SlicedGammaParetoMean}}
#' \code{\link{SlicedLNormParetoMean}}
#'
#' @section NetSimR capped mean functions:
#' \code{\link{GammaCappedMean}}
#' \code{\link{LNormCappedMean}}
#' \code{\link{ParetoCappedMean}}
#' \code{\link{SlicedGammaParetoCappedMean}}
#' \code{\link{SlicedLNormParetoCappedMean}}
#'
#' @section NetSimR exposure curve functions:
#' \code{\link{ExposureCurveGamma}}
#' \code{\link{ExposureCurveLNorm}}
#' \code{\link{ExposureCurvePareto}}
#' \code{\link{ExposureCurveSlicedGammaPareto}}
#' \code{\link{ExposureCurveSlicedLNormPareto}}
#'
#' @section NetSimR ILF curve functions:
#' \code{\link{ILFGamma}}
#' \code{\link{ILFLNorm}}
#' \code{\link{ILFPareto}}
#' \code{\link{ILFSlicedGammaPareto}}
#' \code{\link{ILFSlicedLNormPareto}}
#'
#' @section NetSimR pure IBNR functions:
#' \code{\link{PureIBNRGamma}}
#' \code{\link{PureIBNRLNorm}}
#'
#' @section NetSimR Sliced distribution functions:
#' \code{\link{dSlicedGammaPareto}}
#' \code{\link{dSlicedLNormPareto}}
#' \code{\link{pSlicedGammaPareto}}
#' \code{\link{pSlicedLNormPareto}}
#' \code{\link{qSlicedGammaPareto}}
#' \code{\link{qSlicedLNormPareto}}
#'
#' @keywords internal
"_PACKAGE"

##############################
#Sliced Gamma-Pareto Functions
##############################

#' Sliced Gamma Pareto mean
#'
#' @param GShape A positive real number -  the shape parameter of the attritional Claim Severity's Gamma distribution.
#' @param GRate A positive real number -  the rate parameter of the attritional Claim Severity's Gamma distribution.
#' @param SlicePoint A positive real number - the slice point and the scale parameter of the tail Claim Severity's Pareto distribution.
#' @param PShape A positive real number - the Shape parameter of the tail Claim Severity's Pareto distribution.
#' @return The mean of the claim severity with an attritional claim Gamma distribution with parameters \code{GShape} and \code{GRate} and a large claim Pareto distribution with parameters \code{SlicePoint} and \code{PShape}.
#' @export
#' @examples
#' SlicedGammaParetoMean(1,0.0005,1000,1.2)
#' SlicedGammaParetoMean(1.1,0.0006,2000,1.6)
#' SlicedGammaParetoMean(1.2,0.0004,3000,1.4)
SlicedGammaParetoMean<-function(GShape, GRate, SlicePoint, PShape){
  df<-data.frame(GShape, GRate, SlicePoint, PShape)
  ifelse(df$PShape>1
         ,GammaCappedMean(SlicePoint,GShape,GRate)+pgamma(SlicePoint,GShape,GRate,lower.tail = FALSE)*((PShape*SlicePoint)/(PShape-1)-SlicePoint)
         ,Inf
  )
}



#' Sliced Gamma Pareto capped mean
#'
#' @param cap A positive real number -  the claim severity cap.
#' @param GShape A positive real number -  the shape parameter of the attritional Claim Severity's Gamma distribution.
#' @param GRate A positive real number -  the rate parameter of the attritional Claim Severity's Gamma distribution.
#' @param SlicePoint A positive real number - the slice point and the scale parameter of the tail Claim Severity's Pareto distribution.
#' @param PShape A positive real number - the shape parameter of the tail Claim Severity's Pareto distribution.
#' @return The mean of the claim severity capped at \code{cap} with an attritional claim Gamma distribution with parameters \code{GShape} and \code{GRate} and a large claim Pareto distribution with parameters \code{SlicePoint} and \code{PShape}.
#' @export
#' @examples
#' SlicedGammaParetoCappedMean(3000,1,0.0005,1000,1.2)
#' SlicedGammaParetoCappedMean(1000,1.1,0.0006,2000,1.6)
#' SlicedGammaParetoCappedMean(2000,1.2,0.0004,3000,1.4)
SlicedGammaParetoCappedMean<-function(cap, GShape, GRate, SlicePoint, PShape){
  ifelse(cap<=SlicePoint
         ,GammaCappedMean(cap,GShape,GRate)
         ,GammaCappedMean(SlicePoint,GShape,GRate)+pgamma(SlicePoint,GShape,GRate,lower.tail = FALSE)*(ParetoCappedMean(cap, SlicePoint, PShape)-SlicePoint)
  )
}



#' Exposure Curve from a Sliced Gamma Pareto severity distribution
#'
#' @param x A positive real number -  the claim amount where the exposure curve will be evaluated.
#' @param GShape A positive real number -  the shape parameter of the Claim Severity's Gamma distribution.
#' @param GRate A positive real number -  the rate parameter of the Claim Severity's Gamma distribution.
#' @param SlicePoint A positive real number - the slice point and the scale parameter of the Claim Severity's Pareto distribution.
#' @param PShape A positive real number - the shape parameter of the Claim Severity's Pareto distribution.
#' @return The value of the Exposure curve at \code{x} with an attritional claim Gamma distribution with parameters \code{GShape} and \code{GRate} and a large claim Pareto distribution with parameters \code{SlicePoint} and \code{PShape}.
#' @export
#' @examples
#' ExposureCurveSlicedGammaPareto(3000,1,0.0005,1000,1.2)
#' ExposureCurveSlicedGammaPareto(1000,1.1,0.0006,2000,1.6)
#' ExposureCurveSlicedGammaPareto(2000,1.2,0.0004,3000,1.4)
ExposureCurveSlicedGammaPareto<-function(x, GShape, GRate, SlicePoint, PShape){
  df<-data.frame(x, GShape, GRate, SlicePoint, PShape)
  ifelse(df$PShape>1
         ,SlicedGammaParetoCappedMean(x, GShape, GRate, SlicePoint, PShape)/SlicedGammaParetoMean(GShape, GRate, SlicePoint, PShape)
         ,0
  )
}



#' Increased Limit Factor Curve from a Sliced Gamma Pareto severity distribution
#'
#' @param xLow A positive real number -  the claim amount where the Limit Factor Curve will be evaluated from.
#' @param xHigh A positive real number -  the claim amount where the Limit Factor Curve will be evaluated to.
#' @param GShape A positive real number -  the shape parameter of the attritional Claim Severity's Gamma distribution.
#' @param GRate A positive real number -  the rate parameter of the attritional Claim Severity's Gamma distribution.
#' @param SlicePoint A positive real number - the slice point and the scale parameter of the tail Claim Severity's Pareto distribution.
#' @param PShape A positive real number - the shape parameter of the tail Claim Severity's Pareto distribution.
#' @return The value of the Increased Limit Factor curve from \code{xLow} to \code{xHigh} with an attritional claim Gamma distribution with parameters \code{GShape} and \code{GRate} and a large claim Pareto distribution with parameters \code{SlicePoint} and \code{PShape}.
#' @export
#' @examples
#' ILFSlicedGammaPareto(2000,3000,1,0.0005,1000,1.2)
#' ILFSlicedGammaPareto(800,1000,1.1,0.0006,2000,1.6)
#' ILFSlicedGammaPareto(1200,2000,1.2,0.0004,3000,1.4)
ILFSlicedGammaPareto<-function(xLow, xHigh, GShape, GRate, SlicePoint, PShape){
  SlicedGammaParetoCappedMean(xHigh, GShape, GRate, SlicePoint, PShape)/SlicedGammaParetoCappedMean(xLow, GShape, GRate, SlicePoint, PShape)
}



#' The cumulative density function (cdf) of a Sliced Gamma-Pareto severity distribution
#'
#' @param x A positive real number -  the claim amount where the cumulative density function (cdf) will be evaluated.
#' @param GShape A positive real number -  the shape parameter of the attritional Claim Severity's Gamma distribution.
#' @param GRate A positive real number -  the rate parameter of the attritional Claim Severity's Gamma distribution.
#' @param SlicePoint A positive real number - the slice point and the scale parameter of the tail Claim Severity's Pareto distribution.
#' @param PShape A positive real number - the shape parameter of the tail Claim Severity's Pareto distribution.
#' @return The value of the cumulative density function (cdf) at \code{x} with an attritional claim Gamma distribution with parameters \code{GShape} and \code{GRate} and a large claim Pareto distribution with parameters \code{SlicePoint} and \code{PShape}.
#' @export
#' @examples
#' pSlicedGammaPareto(3000,1,0.0005,1000,1.2)
#' pSlicedGammaPareto(1000,1.1,0.0006,2000,1.6)
#' pSlicedGammaPareto(2000,1.2,0.0004,3000,1.4)
pSlicedGammaPareto<-function(x, GShape, GRate, SlicePoint, PShape){
  ifelse(x>SlicePoint
         ,pgamma(SlicePoint, GShape, GRate)+pgamma(SlicePoint, GShape, GRate, lower.tail = FALSE)*(1-(SlicePoint/x)^PShape)
         ,pgamma(x, GShape, GRate)
  )
}



#' The inverse cumulative density function of a Sliced Gamma Pareto severity distribution
#'
#' @param q A real number between 0 and 1 -  the probability where the inverse cumulative density function will be evaluated.
#' @param GShape A positive real number -  the shape parameter of the attritional Claim Severity's Gamma distribution.
#' @param GRate A positive real number -  the rate parameter of the attritional Claim Severity's Gamma distribution.
#' @param SlicePoint A positive real number - the slice point and the scale parameter of the tail Claim Severity's Pareto distribution.
#' @param PShape A positive real number - the shape parameter of the tail Claim Severity's Pareto distribution.
#' @return The value of the inverse cumulative density function at \code{q} with an attritional claim Gamma distribution with parameters \code{GShape} and \code{GRate} and a large claim Pareto distribution with parameters \code{SlicePoint} and \code{PShape}.
#' @export
#' @examples
#' qSlicedGammaPareto(0.5,1,0.0005,1000,1.2)
#' qSlicedGammaPareto(0.2,1.1,0.0006,2000,1.6)
#' qSlicedGammaPareto(0.8,1.2,0.0004,3000,1.4)
qSlicedGammaPareto<-function(q, GShape, GRate, SlicePoint, PShape){
  lp<-pgamma(SlicePoint, GShape, GRate)
  up<-1-lp
  ifelse(q>lp
         ,SlicePoint/((1-((q-lp)/up))^(1/PShape))
         ,qgamma(q, GShape, GRate)
  )
}



#' The probability density function (pdf) of a Sliced Gamma Pareto severity distribution
#'
#' @param x A positive real number -  the claim amount where the probability density function (pdf) will be evaluated.
#' @param GShape A positive real number -  the shape parameter of the attritional Claim Severity's Gamma distribution.
#' @param GRate A positive real number -  the rate parameter of the attritional Claim Severity's Gamma distribution.
#' @param SlicePoint A positive real number - the slice point and the scale parameter of the tail Claim Severity's Pareto distribution.
#' @param PShape A positive real number - the shape parameter of the tail Claim Severity's Pareto distribution.
#' @return The value of the probability density function (pdf) at \code{x} with an attritional claim Gamma distribution with parameters \code{GShape} and \code{GRate} and a large claim Pareto distribution with parameters \code{SlicePoint} and \code{PShape}.
#' @export
#' @examples
#' dSlicedGammaPareto(3000,1,0.0005,1000,1.2)
#' dSlicedGammaPareto(1000,1.1,0.0006,2000,1.6)
#' dSlicedGammaPareto(2000,1.2,0.0004,3000,1.4)
dSlicedGammaPareto<-function(x, GShape, GRate, SlicePoint, PShape){
  ifelse(x>SlicePoint
         ,pgamma(SlicePoint, GShape, GRate, lower.tail = FALSE)*(PShape*SlicePoint^PShape)/(x^(PShape+1))
         ,dgamma(x, GShape, GRate)
  )
}

#settings
options(shiny.maxRequestSize=64*1024^3)

#' A function to run the glm fitting tool application
#'
#' @return Opens the glm fitting tool application
#' @export
run_shiny_distribution_fitting_tool = function(){
  shinyApp(ui = distribution_fitting_tool_UI, server = distribution_fitting_tool_Server)
}

#help functions

#' Parameter to set the maximum number of pareto slices
#'
#' @return The the maximum number of Pareto Slices.
max_number_of_pareto_slices = 5

#' Random Pareto generator
#'
#' @param n Number of values to generate.
#' @param alpha A positive real number. Alpha parameter of the Pareto distribution.
#' @param x_m A positive real number. The minimum value for the Pareto distribution.
#' @return A vector of \code{n} random Pareto variables with parameters \code{alpha} and \code{x_m}.
rpareto <- function(n, alpha, x_m) x_m / runif(n)^(1/alpha)

#' Apply severity cap function
#'
#' @param claims A vector of Claims.
#' @param severity_cap_boolean A variable that if true, the function will cap the claims, otherwise will just return them.
#' @param severity_cap_amount The claim cap value.
#' @return If \code{severity_cap_boolean} is true, then will return the minimum of \code{severity_cap_amount} or \code{claims} otherwise will return \code{claims}. The operation is vectorised.
apply_severity_cap <- function(claims, severity_cap_boolean, severity_cap_amount){
  if (!severity_cap_boolean) return(claims)
  pmin(claims, severity_cap_amount)
}

#' A vector with the reinsurance structure options
#'
#' @return The reinsurance structure options
reinsurance_structures_options <- c('No Reinsurance Structure', 'Unlimited Layer', 'Limited Layer', 'Exclude Layer')

#' Apply a deductible and limit to claims
#'
#' @param gross_claims_data A vector of Claims.
#' @param reinsurance_structure The chosen reinsurance structure. Options are: 'No Reinsurance Structure', 'Unlimited Layer', 'Limited Layer', 'Exclude Layer'.
#' @param deductible The deductible of the reinsurance structure.
#' @param limit The limit of the reinsurance structure.
#' @return The ceded claims for the structure, with the chosen deductible and limit.
#' @export
#' @examples
#' apply_deductible_limit(c(100, 50, 20), 'Limited Layer', 40, 20)
#' apply_deductible_limit(c(100, 50, 20), 'Limited Layer', 10, 30)
apply_deductible_limit <- function(gross_claims_data, reinsurance_structure, deductible, limit){
  if (reinsurance_structure == 'No Reinsurance Structure') {return(gross_claims_data)}

  layer_claims <- pmax(gross_claims_data - deductible, 0)

  if (reinsurance_structure == 'Unlimited Layer') {return(layer_claims)}

  limited_layer_claims <- pmin(layer_claims, limit)

  if (reinsurance_structure == 'Limited Layer') {return(limited_layer_claims)}

  if (reinsurance_structure == 'Exclude Layer') {return(gross_claims_data - limited_layer_claims)}

  stop("Unknown reinsurance structure: ", reinsurance_structure)
}

#' The class of the distribution objects
#'
distributionClass <- setClass("distributionClass", slots = c(
  distrID="character"
  ,distr_label="character"
  ,paramIDs="character"
  ,param_labels="character"
  ,param_min_values="numeric"
  ,param_max_values="numeric"
  ,simulate_func = "function"
))

#' A vector with the frequency distribution objects
#'
#' @return The frequency distribution objects.
freq_dist_options <- c(
  Poisson=distributionClass(
    distrID='Poisson'
    ,distr_label='Poisson'
    ,paramIDs=c("lamda")
    ,param_labels=c("lamda")
    ,param_min_values=c(0)
    ,simulate_func = function(number_of_simulations, parameters){return(
      rpois(n = number_of_simulations, lambda = parameters[1])
    )}
  )
  ,Negative_Binomial=distributionClass(
    distrID='Negative_Binomial'
    ,distr_label='Negative Binomial'
    ,paramIDs=c("r", "beta")
    ,param_labels=c("r", "beta")
    ,param_min_values=c(0,0)
    ,simulate_func = function(number_of_simulations, parameters){return(
      rpois(n = number_of_simulations, lambda = rgamma(n =number_of_simulations, shape = parameters[1], scale = parameters[2]))
    )}
  )
  ,Binomial=distributionClass(
    distrID='Binomial'
    ,distr_label='Binomial'
    ,paramIDs=c("n", "p")
    ,param_labels=c("n", "p")
    ,param_min_values=c(0,0)
    ,param_max_values=c(NA,1)
    ,simulate_func = function(number_of_simulations, parameters){return(
      rbinom(n = number_of_simulations, size = parameters[1], prob = parameters[2])
    )}
  )
  ,Fixed_number_of_Counts=distributionClass(
    distrID='Fixed_number_of_Counts'
    ,distr_label='Fixed number of Counts'
    ,paramIDs=c("FixedNumberOfCounts")
    ,param_labels=c("FixedNumberOfCounts")
    ,param_min_values=c(0)
    ,simulate_func = function(number_of_simulations, parameters){return(
      rep(parameters[1], number_of_simulations)
    )}
  )
)

#' A data frame with the frequency distribution parameter placeholders
#'
#' @return The frequency distribution parameter placeholders.
freq_dist_parameter_placeholders <- data.frame(
  param_number = 1:max(sapply(freq_dist_options, function(x) length(x@paramIDs)))
  ,param_id = paste0("freq_param_", 1:max(sapply(freq_dist_options, function(x) length(x@paramIDs))))
)

#' A vector with the severity distribution objects
#'
#' @return The severity distribution objects.
sev_dist_options <- c(
  Normal=distributionClass(
    distrID='Normal'
    ,distr_label='Normal'
    ,paramIDs=c("mu", "sigma")
    ,param_labels=c("mu", "sigma")
    ,param_min_values=c(0, 0)
    ,simulate_func = function(number_of_simulations, parameters){return(
      rnorm(n = number_of_simulations, mean = parameters[1], sd = parameters[2])
    )}
  )
  ,LogNormal=distributionClass(
    distrID='LogNormal'
    ,distr_label='Log-Normal'
    ,paramIDs=c("mu", "sigma")
    ,param_labels=c("mu", "sigma")
    ,param_min_values=c(0,0)
    ,param_max_values=c(NA,1)
    ,simulate_func = function(number_of_simulations, parameters){return(
      rlnorm(n = number_of_simulations, meanlog = parameters[1], sdlog = parameters[2])
    )}
  )
  ,Gamma=distributionClass(
    distrID='Gamma'
    ,distr_label='Gamma'
    ,paramIDs=c("shape", "scale")
    ,param_labels=c("shape", "scale")
    ,param_min_values=c(0,0)
    ,param_max_values=c(NA,1)
    ,simulate_func = function(number_of_simulations, parameters){return(
      rgamma(n = number_of_simulations, shape = parameters[1], scale = parameters[2])
    )}
  )
  ,Exponential=distributionClass(
    distrID='Exponential'
    ,distr_label='Exponential'
    ,paramIDs=c("rate")
    ,param_labels=c("rate")
    ,param_min_values=c(0)
    ,simulate_func = function(number_of_simulations, parameters){return(
      rexp(n = number_of_simulations, rate = parameters[1])
    )}
  )
  ,Pareto=distributionClass(
    distrID='Pareto'
    ,distr_label='Pareto'
    ,paramIDs=c("alpha", "x_m")
    ,param_labels=c("alpha", "x_m")
    ,param_min_values=c(0, 0)
    ,simulate_func = function(number_of_simulations, parameters){return(
      rpareto(n = number_of_simulations, alpha = parameters[1], x_m = parameters[2])
    )}
  )
  ,Fixed_Severity=distributionClass(
    distrID='Fixed_Severity'
    ,distr_label='Fixed Severity'
    ,paramIDs=c("Fixed_sev_amount")
    ,param_labels=c("Fixed Severity Amount")
    ,param_min_values=c(0)
    ,simulate_func = function(number_of_simulations, parameters){return(
      rep(parameters[1], number_of_simulations)
    )}
  )
)

#' A data frame with the severity distribution parameter placeholders
#'
#' @return The severity distribution parameter placeholders.
sev_dist_parameter_placeholders <- data.frame(
  param_number = 1:max(sapply(sev_dist_options, function(x) length(x@paramIDs)))
  ,param_id = paste0("sev_param_", 1:max(sapply(sev_dist_options, function(x) length(x@paramIDs))))
)

#' A function to simulate frequency - severity of insurance claims using chunked vectorisation.
#' The function applies severity cap, reinsurance structure for each and every loss claim,
#' reinsurance structure for aggregate claims, and allows for piecewise pareto slices.
#'
#' @param numOfSimulations The number of simulations to run.
#' @param freq_params A vector of the frequency distribution parameters.
#' @param sev_params A vector of the severity distribution parameters.
#' @param seedSetBinary True if there is a fixed seed, otherwise false.
#' @param seedValue The seed value.
#' @param freqDistr The frequency distribution. Options are as per the freq_dist_options.
#' @param sevDistr The severity distribution. Options are as per the sev_dist_options.
#' @param paretoSlice True if there is Pareto slicing.
#' @param pareto_slice_times The number of Pareto slices.
#' @param slice_pareto_alphas A vector of Pareto slices' alpha parameters.
#' @param slice_pareto_x_ms A vector of Pareto slices' x_m parameters.
#' @param sevCapBinary True if there is a severity cap.
#' @param sev_cap_amount The severity cap amount.
#' @param reinsuranceStructureEEL The chosen reinsurance structure for each and every loss claim.
#' @param reinsurance_structure_eel_dedctible_amount The deductible for each and every loss reinsurance structure.
#' @param reinsurance_structure_eel_limit_amount The limit for each and every loss reinsurance structure.
#' @param reinsuranceStructureAL The chosen reinsurance structure for aggregate claims.
#' @param reinsurance_structure_al_dedctible_amount The deductible for aggregate reinsurance structure.
#' @param reinsurance_structure_al_limit_amount The limit for aggregate reinsurance structure.
#' @param reinsuranceStructureLimitedReinstatements True if there is a limit in reinstatements, otherwise false.
#' @param reinsuranceStructureReinstatementLimit The reinstatement limit.
#' @param multiprocessing True if multiprocessing is used, otherwise false.
#' @param chunk_size The number of simulations processed per vectorised batch. Defaults to 10000.
#' @return A data frame with claims counts, ceded claims and the number of reinstatements used.
#' @export
#' @import data.table
simulate_function <- function(
    numOfSimulations,
    freq_params,
    sev_params,
    seedSetBinary,
    seedValue,
    freqDistr,
    sevDistr,
    paretoSlice,
    pareto_slice_times,
    slice_pareto_alphas,
    slice_pareto_x_ms,
    sevCapBinary,
    sev_cap_amount,
    reinsuranceStructureEEL,
    reinsurance_structure_eel_dedctible_amount,
    reinsurance_structure_eel_limit_amount,
    reinsuranceStructureAL,
    reinsurance_structure_al_dedctible_amount,
    reinsurance_structure_al_limit_amount,
    reinsuranceStructureLimitedReinstatements,
    reinsuranceStructureReinstatementLimit,
    multiprocessing,
    chunk_size = 10000
){
  #set custom seed
  if(seedSetBinary){set.seed(seedValue)}

  n_chunks <- ceiling(numOfSimulations / chunk_size)
  chunk_sizes <- rep(chunk_size, n_chunks)
  remainder <- numOfSimulations - chunk_size * (n_chunks - 1)
  chunk_sizes[n_chunks] <- remainder

  #vectorised worker for a single chunk of simulations
  simulate_chunk <- function(this_n){

    #simulate claim counts for this chunk
    counts <- freq_dist_options[[freqDistr]]@simulate_func(
      number_of_simulations = this_n
      ,parameters = freq_params
    )

    total_claims_needed <- sum(counts)

    #edge case: chunk has zero claims across all simulations
    if(total_claims_needed == 0){
      return(list(claim_counts = counts, total_claims = rep(0, this_n)))
    }

    sim_id <- rep.int(seq_len(this_n), counts)

    #simulate all individual severities for the chunk in one vectorised call
    claims <- sev_dist_options[[sevDistr]]@simulate_func(
      number_of_simulations = total_claims_needed
      ,parameters = sev_params
    )

    #apply pareto slices using logical indexing (only redraws claims above threshold)
    if(paretoSlice){
      for(j in 1:pareto_slice_times){
        idx <- claims > slice_pareto_x_ms[j]
        if(any(idx)){
          claims[idx] <- rpareto(n = sum(idx), alpha = slice_pareto_alphas[j], x_m = slice_pareto_x_ms[j])
        }
      }
    }

    #apply severity cap
    claims <- apply_severity_cap(
      claims
      ,severity_cap_boolean = sevCapBinary
      ,severity_cap_amount = sev_cap_amount
    )

    #apply EEL deductible/limit per individual claim
    claims <- apply_deductible_limit(
      claims
      ,reinsurance_structure = reinsuranceStructureEEL
      ,deductible = reinsurance_structure_eel_dedctible_amount
      ,limit = reinsurance_structure_eel_limit_amount
    )

    #aggregate individual claims back to simulation-level totals via data.table
    dt <- data.table::data.table(sim_id = sim_id, claim = claims)
    agg <- stats::aggregate(claim ~ sim_id, data = dt, FUN = sum)
    names(agg)[names(agg) == "claim"] <- "total_claims"
    totals <- numeric(this_n)
    totals[agg[["sim_id"]]] <- agg[["total_claims"]]

    return(list(claim_counts = counts, total_claims = totals))
  }

  #run chunks, optionally in parallel across chunks (not per-simulation)
  #run chunks, optionally in parallel across chunks (not per-simulation)
  if (multiprocessing) {
    future::plan(future::multisession)
    on.exit(future::plan(future::sequential), add = TRUE)

    shiny::showNotification(
      "Running in parallel. Live progress is not available in multiprocessing mode.",
      type = "message",
      duration = NULL,
      id = "parallel_sim_notice"
    )
    on.exit(shiny::removeNotification("parallel_sim_notice"), add = TRUE)

    chunk_results <- future.apply::future_lapply(
      chunk_sizes,
      simulate_chunk,
      future.seed = TRUE
    )

  } else {
    chunk_results <- shiny::withProgress(
      message = "Running simulations",
      detail = paste("Processing", n_chunks, "chunks"),
      value = 0,
      {
        res <- vector("list", length(chunk_sizes))

        for (i in seq_along(chunk_sizes)) {
          res[[i]] <- simulate_chunk(chunk_sizes[i])
          shiny::incProgress(
            amount = 1 / length(chunk_sizes),
            detail = paste("Chunk", i, "of", length(chunk_sizes))
          )
        }

        res
      }
    )
  }

  data <- data.frame(
    claim_counts = unlist(lapply(chunk_results, `[[`, "claim_counts"), use.names = FALSE)
    ,total_claims = unlist(lapply(chunk_results, `[[`, "total_claims"), use.names = FALSE)
  )
  rm(chunk_results); gc(FALSE)

  #apply reinstatements
  if(reinsuranceStructureEEL %in% c('Limited Layer')){
    if(reinsuranceStructureLimitedReinstatements){
      data$total_claims <- apply_deductible_limit(
        data$total_claims
        ,'Limited Layer'
        ,0
        ,(reinsuranceStructureReinstatementLimit + 1) * reinsurance_structure_eel_limit_amount
      )
      data$number_of_reinstatements_used <- (data$total_claims / reinsurance_structure_eel_limit_amount)
      data$number_of_reinstatements_used <- ifelse(
        data$number_of_reinstatements_used > reinsuranceStructureReinstatementLimit
        ,reinsuranceStructureReinstatementLimit
        ,data$number_of_reinstatements_used
      )
      data$number_of_reinstatements_used <- round(data$number_of_reinstatements_used, 2)
    }
  }

  #apply reinsurance structure AL
  data$total_claims <- apply_deductible_limit(
    data$total_claims
    ,reinsuranceStructureAL
    ,reinsurance_structure_al_dedctible_amount
    ,reinsurance_structure_al_limit_amount
  )
  data$total_claims <- round(data$total_claims, 2)
  return(data)
}

#' A function to run the shiny simulator application
#'
#' @return Opens the shiny simulator application
#' @export
run_shiny_simulator = function(){shinyApp(ui = shiny_simulator_ui, server = shiny_simulator_server)}

#' UI file for the Shiny GLM fitting tool
#'
#' @return Returns the UI code for the shiny application.

GLMFittingToolUI <- bslib::page_navbar(
  title = div(
    style = "display:flex; align-items:center; gap:12px;",
    div(
      style = paste(
        "width:44px; height:44px; border-radius:12px;",
        "background: linear-gradient(135deg, #14b8a6, #0f766e);",
        "display:flex; align-items:center; justify-content:center;",
        "color:white; font-weight:800; font-size:16px;",
        "box-shadow: 0 6px 18px rgba(0,0,0,0.12);"
      ),
      "GLM"
    ),
    div(
      style = "display:flex; flex-direction:column; line-height:1.05;",
      tags$span(
        "GLM Fitting Tool",
        style = "font-weight:800; font-size:1.05rem; color:white; letter-spacing:-0.02em;"
      ),
      tags$span(
        "Model fitting and diagnostics",
        style = "font-size:12px; color:rgba(255,255,255,0.72); margin-top:2px;"
      )
    )
  ),
  window_title = "GLM Fitting Tool",
  id = "main_navbar",
  selected = "Welcome",

  navbar_options = bslib::navbar_options(
    bg = "#0f766e",
    theme = "dark",
    underline = FALSE
  ),

  theme = bslib::bs_theme(
    version = 5,
    bootswatch = "flatly",
    primary = "#0f766e",
    secondary = "#64748b",
    success = "#15803d",
    info = "#0891b2",
    warning = "#d97706",
    danger = "#dc2626",
    bg = "#f8fafc",
    fg = "#0f172a",
    base_font = bslib::font_google("Inter"),
    heading_font = bslib::font_google("Inter"),
    code_font = bslib::font_google("JetBrains Mono")
  ),

  header = tagList(
    shinybusy::use_busy_spinner(spin = "fading-circle", position = "full-page"),
    tags$head(
      tags$style(HTML("
:root {
  --app-radius: 18px;
  --app-shadow: 0 10px 30px rgba(15, 23, 42, 0.08);
  --app-border: 1px solid rgba(148, 163, 184, 0.22);
}

body {
  background: linear-gradient(180deg, #f8fafc 0%, #f1f5f9 100%);
}

.navbar {
  box-shadow: 0 2px 16px rgba(15, 23, 42, 0.08);
  padding-top: 0.55rem;
  padding-bottom: 0.55rem;
}

.navbar-brand {
  margin-right: 2.8rem !important;
  padding-top: 0.35rem;
  padding-bottom: 0.35rem;
  min-width: 280px;
}

.navbar .container-fluid {
  align-items: center;
}

.navbar .navbar-nav {
  margin-left: 1.2rem !important;
  gap: 0.35rem;
  align-items: center;
}

.navbar .nav-item {
  display: flex;
  align-items: center;
}

.navbar .nav-link {
  color: rgba(255,255,255,0.88) !important;
  font-weight: 700;
  font-size: 0.98rem;
  border-radius: 12px;
  padding: 0.7rem 1rem !important;
  transition: all 0.18s ease-in-out;
}

.navbar .nav-link:hover,
.navbar .nav-link:focus {
  color: #ffffff !important;
  background-color: rgba(255,255,255,0.12);
}

.navbar .nav-link.active,
.navbar .nav-item.show .nav-link {
  color: #0f172a !important;
  background: #f8fafc !important;
  box-shadow: 0 4px 12px rgba(15, 23, 42, 0.10);
}

.tab-content {
  padding-top: 0.75rem;
}

.glm-hero {
  padding: 8px 0 4px 0;
}

.glm-hero-title {
  font-size: 2rem;
  font-weight: 800;
  letter-spacing: -0.03em;
  color: #0f172a;
  margin-bottom: 0.5rem;
}

.glm-hero-subtitle {
  font-size: 1rem;
  color: #475569;
  max-width: 760px;
  margin-bottom: 0;
}

.glm-stat {
  background: white;
  border-radius: var(--app-radius);
  box-shadow: var(--app-shadow);
  border: var(--app-border);
  padding: 1rem 1.1rem;
  height: 100%;
}

.glm-stat-label {
  font-size: 0.78rem;
  font-weight: 700;
  text-transform: uppercase;
  letter-spacing: 0.06em;
  color: #64748b;
  margin-bottom: 0.35rem;
}

.glm-stat-value {
  font-size: 1.2rem;
  font-weight: 800;
  color: #0f172a;
}

.card {
  border: var(--app-border) !important;
  border-radius: var(--app-radius) !important;
  box-shadow: var(--app-shadow);
  overflow: hidden;
}

.card-header {
  background: linear-gradient(180deg, #ffffff 0%, #f8fafc 100%);
  font-weight: 700;
  border-bottom: 1px solid rgba(148, 163, 184, 0.16);
}

.card-body {
  overflow: visible !important;
}

.scroll-card {
  height: 760px;
}

.scroll-card .card-body,
.scroll-card-body {
  overflow-y: auto !important;
  overflow-x: hidden !important;
  max-height: 680px;
  padding-right: 0.85rem;
}

.import-scroll-card {
  height: 700px;
}

.import-scroll-card .card-body,
.import-scroll-card-body {
  overflow-y: auto !important;
  overflow-x: hidden !important;
  max-height: 620px;
  padding-right: 0.85rem;
}

.form-label, .control-label {
  display: block;
  font-weight: 700;
  color: #334155;
  margin-bottom: 0.45rem;
  line-height: 1.3;
}

.form-control, .form-select, .selectize-input {
  border-radius: 12px !important;
  border: 1px solid #cbd5e1 !important;
  min-height: 46px;
  box-shadow: none !important;
  margin-bottom: 0.25rem;
}

.form-control:focus, .form-select:focus, .selectize-input.focus {
  border-color: #0f766e !important;
  box-shadow: 0 0 0 0.2rem rgba(15, 118, 110, 0.15) !important;
}

textarea.form-control {
  min-height: 140px;
}

.btn {
  border-radius: 12px;
  font-weight: 700;
  min-height: 42px;
}

.btn-primary {
  background: linear-gradient(135deg, #0f766e, #0ea5a4);
  border: none;
}

.btn-primary:hover,
.btn-primary:focus {
  background: linear-gradient(135deg, #0d6b63, #0c948f);
  border: none;
}

.btn-default, .btn-secondary {
  border: 1px solid #cbd5e1;
  background: white;
  color: #334155;
}

.well {
  border-radius: 14px;
  border: var(--app-border);
  background: #ffffff;
  box-shadow: none;
}

pre, .shiny-text-output {
  background: #0f172a;
  color: #e2e8f0;
  border-radius: 14px;
  padding: 1rem;
  border: none;
}

.form-section-title {
  font-size: 0.8rem;
  font-weight: 800;
  letter-spacing: 0.08em;
  text-transform: uppercase;
  color: #64748b;
  margin: 0 0 1rem 0;
}

.form-block {
  margin-bottom: 1.15rem;
}

.model-controls-body {
  padding-top: 1.25rem;
}

.import-controls-body {
  padding-top: 1rem;
}

.model-controls-body .shiny-input-container,
.import-controls-body .shiny-input-container {
  width: 100%;
  margin-bottom: 1rem;
}

.subgrid-2 {
  display: grid;
  grid-template-columns: 1fr 1fr;
  gap: 1rem 1rem;
  margin-bottom: 0.25rem;
}

.stored-formula-card {
  border: 1px solid rgba(148, 163, 184, 0.22);
  border-radius: 14px;
  padding: 1rem;
  background: #f8fafc;
  height: 100%;
}

.stored-formula-title {
  font-weight: 800;
  color: #334155;
  margin-bottom: 0.75rem;
}

.muted-note {
  color: #64748b;
  font-size: 0.95rem;
  margin-bottom: 0;
}

.contact-link a {
  text-decoration: none;
  font-weight: 600;
}

.shiny-download-link {
  width: 100%;
  margin-bottom: 0.5rem;
}

.html-fill-container,
.html-fill-item {
  min-height: 0;
}

.reactable {
  font-size: 0.95rem;
}

@media (max-width: 991.98px) {
  .navbar-brand {
    min-width: auto;
    margin-right: 1rem !important;
  }

  .navbar .navbar-nav {
    margin-left: 0 !important;
    gap: 0.15rem;
    padding-top: 0.5rem;
  }

  .navbar .nav-link {
    padding: 0.6rem 0.8rem !important;
  }

  .subgrid-2 {
    grid-template-columns: 1fr;
  }

  .scroll-card,
  .import-scroll-card {
    height: auto;
  }

  .scroll-card .card-body,
  .scroll-card-body,
  .import-scroll-card .card-body,
  .import-scroll-card-body {
    max-height: none;
    overflow: visible !important;
    padding-right: 0;
  }
}
"))
    )
  ),

  bslib::nav_panel(
    "Welcome",
    bslib::layout_columns(
      col_widths = c(8, 4),

      bslib::card(
        bslib::card_body(
          class = "glm-hero",
          div(class = "glm-hero-title", "Welcome to the GLM Fitting Tool"),
          p(
            class = "glm-hero-subtitle",
            "A clean workspace for importing data, fitting generalized linear models, comparing formulas, and visualising actual versus predicted results."
          )
        )
      ),

      bslib::card(
        bslib::card_header("About"),
        bslib::card_body(
          p("Designed for fast actuarial and analytical model experimentation."),
          div(
            class = "contact-link",
            p(
              "Created by Yiannis Parizas. Visit ",
              tags$a(
                "LinkedIn profile",
                href = "https://www.linkedin.com/in/yiannisparizas/",
                target = "_blank",
                rel = "noopener noreferrer"
              ),
              "."
            ),
            p(
              "Contact: ",
              tags$a(
                "yiannis.parizas@gmail.com",
                href = "mailto:yiannis.parizas@gmail.com"
              )
            )
          )
        )
      ),

      div(
        class = "glm-stat",
        div(class = "glm-stat-label", "Workflow"),
        div(class = "glm-stat-value", "Import -> Fit -> Compare -> Export")
      ),
      div(
        class = "glm-stat",
        div(class = "glm-stat-label", "Data sources"),
        div(class = "glm-stat-value", "CSV and databases")
      ),
      div(
        class = "glm-stat",
        div(class = "glm-stat-label", "Model families"),
        div(class = "glm-stat-value", "Gaussian, Poisson, Binomial, Gamma")
      )
    )
  ),

  bslib::nav_panel(
    "Save - Load inputs",
    bslib::layout_columns(
      col_widths = c(4, 8),

      bslib::card(
        bslib::card_header("Configuration"),
        bslib::card_body(
          fileInput("load_config", "Load configuration backup"),
          uiOutput("downloadConfButton"),
          tags$hr(),
          p(
            class = "muted-note",
            "Use this section to save your current app state or reload a previous setup."
          )
        )
      ),

      bslib::card(
        bslib::card_header("Instructions"),
        bslib::card_body(
          h4("Recover previous work"),
          p("To fully restore your previous work:"),
          tags$ol(
            tags$li("Load the saved configuration file."),
            tags$li("Reload the original dataset from CSV or database."),
            tags$li("Load the configuration again if some dynamic inputs depend on the imported data.")
          ),
          p(
            class = "muted-note",
            "This two-step restore flow is useful because several model controls depend on the available data columns."
          )
        )
      )
    )
  ),

  bslib::nav_panel(
    "Data import",
    bslib::layout_columns(
      col_widths = c(4, 8),

      bslib::card(
        full_screen = TRUE,
        height = "700px",
        class = "import-scroll-card",
        bslib::card_header("Import settings"),
        bslib::card_body(
          class = "import-controls-body import-scroll-card-body",

          div(class = "form-section-title", "Source"),
          selectInput(
            "data_source",
            "Select Data Source",
            choices = c("CSV File", "Database")
          ),

          conditionalPanel(
            condition = "input.data_source == 'Database'",
            div(
              div(class = "form-section-title", "Connection"),
              selectInput(
                "db_type",
                "Select Database Type",
                choices = c("MySQL", "SQLite", "SQL Server", "PostgreSQL")
              ),
              textInput("db_host", "Database Host/Path", "localhost"),
              textInput("db_name", "Database Name", "public"),

              conditionalPanel(
                condition = "((input.db_type == 'MySQL') || (input.db_type == 'PostgreSQL'))",
                textInput("db_port", "Database Port (optional)", "")
              ),

              conditionalPanel(
                condition = "input.db_type == 'SQL Server'",
                checkboxInput("windows_auth", "Windows Authentication", value = FALSE)
              ),

              conditionalPanel(
                condition = "(input.db_type != 'SQL Server') || ((input.db_type == 'SQL Server') && !input.windows_auth)",
                div(
                  textInput("db_user", "Database User", "root"),
                  passwordInput("db_password", "Database Password", "")
                )
              ),

              div(class = "form-section-title", "Query"),
              textAreaInput(
                "sql_query",
                "SQL Query",
                "SELECT * FROM your_table",
                rows = 6,
                width = "100%"
              )
            )
          ),

          conditionalPanel(
            condition = "input.data_source == 'CSV File'",
            div(
              div(class = "form-section-title", "File"),
              fileInput("csv_file", "Upload CSV File")
            )
          ),

          tags$br(),
          actionButton("submit", "Submit Query / Upload", class = "btn-primary")
        )
      ),

      bslib::card(
        full_screen = TRUE,
        height = "700px",
        bslib::card_header("Selected input data"),
        bslib::card_body(
          reactable::reactableOutput("selected_input_data_table")
        )
      )
    )
  ),

  bslib::nav_panel(
    "GLM Model Fitting",
    bslib::layout_columns(
      col_widths = c(4, 8),

      bslib::card(
        full_screen = TRUE,
        height = "760px",
        class = "scroll-card",
        bslib::card_header("Model controls"),
        bslib::card_body(
          class = "model-controls-body scroll-card-body",

          div(class = "form-section-title", "Core setup"),

          div(
            class = "form-block",
            selectInput("response_variable", "Select Response Variable", choices = NULL)
          ),

          div(
            class = "subgrid-2",
            div(
              class = "form-block",
              selectInput(
                "glm_distribution",
                "GLM Distribution",
                choices = c("gaussian", "poisson", "binomial", "Gamma", "inverse.gaussian"),
                selected = "gaussian"
              )
            ),
            div(
              class = "form-block",
              selectInput(
                "link_function",
                "Link Function",
                choices = c("identity", "logit", "probit", "log", "inverse"),
                selected = "identity"
              )
            )
          ),

          div(
            class = "subgrid-2",
            div(
              class = "form-block",
              selectInput("offset", "Offset (optional)", choices = NULL)
            ),
            div(
              class = "form-block",
              selectInput("weights", "Weights (optional)", choices = NULL)
            )
          ),

          div(
            class = "form-block",
            textAreaInput("formula", "Enter Formula", rows = 7, width = "100%")
          ),

          actionButton("fit_model", "Fit GLM Model", class = "btn-primary"),
          tags$hr(),

          div(class = "form-section-title", "Stored formulas"),

          div(
            class = "subgrid-2",
            div(
              class = "stored-formula-card",
              div(class = "stored-formula-title", "Placeholder 1"),
              actionButton("save_formula_1", "Save", class = "btn-secondary"),
              tags$span(style = "display:inline-block; width:8px;"),
              actionButton("load_formula_1", "Load", class = "btn-secondary"),
              tags$br(), tags$br(),
              verbatimTextOutput("aic_output_1")
            ),
            div(
              class = "stored-formula-card",
              div(class = "stored-formula-title", "Placeholder 2"),
              actionButton("save_formula_2", "Save", class = "btn-secondary"),
              tags$span(style = "display:inline-block; width:8px;"),
              actionButton("load_formula_2", "Load", class = "btn-secondary"),
              tags$br(), tags$br(),
              verbatimTextOutput("aic_output_2")
            )
          ),

          tags$hr(),
          div(class = "form-section-title", "Downloads"),
          downloadButton("download_model", "Download Model (RDS)"),
          downloadButton("download_summary", "Download Model Summary"),
          downloadButton("download_data_with_predictions", "Download Predictions CSV")
        )
      ),

      bslib::card(
        full_screen = TRUE,
        height = "760px",
        bslib::card_header("GLM model summary"),
        bslib::card_body(
          verbatimTextOutput("model_summary")
        )
      )
    )
  ),

  bslib::nav_panel(
    "Predictions visualisation",
    bslib::layout_columns(
      col_widths = c(4, 8),

      bslib::card(
        bslib::card_header("Visualisation controls"),
        bslib::card_body(
          selectInput(
            "visualize_variable",
            "Select Explanatory Variable",
            choices = c("None")
          ),
          sliderInput(
            "number_of_bands_input",
            "Select number of bands:",
            min = 10,
            max = 100,
            value = 10
          ),
          tags$br(),
          actionButton("execute_visualization", "Execute Visualization", class = "btn-primary"),
          tags$hr(),
          p(
            class = "muted-note",
            "Compare actual and predicted values across grouped bands, with exposure shown on the secondary axis."
          )
        )
      ),

      bslib::card(
        full_screen = TRUE,
        height = "720px",
        bslib::card_header("Actual vs predicted"),
        bslib::card_body(
          plotly::plotlyOutput("fitness_plot", height = "620px")
        )
      )
    )
  )
)

#################
#Pareto Functions
#################

#' Pareto capped mean intermediary calculation
#'
#' @param cap A positive real number -  the claim severity cap.
#' @param scale A positive real number - the scale parameter of the Claim Severity's Pareto distribution.
#' @param shape A positive real number - the shape parameter of the Claim Severity's Pareto distribution.
#' @return An interim calculation for the mean of the claim severity capped at \code{cap} with a Pareto distribution with parameters \code{scale} and \code{shape}.
#' @export
#' @examples
#' ParetoCappedMeanCalc(800,100,1.1)
#' ParetoCappedMeanCalc(1000,500,0.9)
ParetoCappedMeanCalc<-function(cap,scale,shape){
  (shape * scale - cap * (scale/cap)^shape)/(shape-1)
}



#' Pareto capped mean
#'
#' @param cap A positive real number -  the claim severity cap.
#' @param scale A positive real number - the scale parameter of the Claim Severity's Pareto distribution.
#' @param shape A positive real number - the shape parameter of the Claim Severity's Pareto distribution.
#' @return The mean of the claim severity capped at \code{cap} with a Pareto distribution with parameters \code{scale} and \code{shape}.
#' @export
#' @examples
#' ParetoCappedMean(600,200,1.2)
#' ParetoCappedMean(800,100,1)
#' ParetoCappedMean(1000,500,0.8)
ParetoCappedMean<-function(cap,scale,shape){
  df<-data.frame(cap,scale,shape)
  ifelse(df$shape==1
         ,(ParetoCappedMeanCalc(cap,scale,shape+0.0001)+ParetoCappedMeanCalc(cap,scale,shape-0.0001))/2
         ,ParetoCappedMeanCalc(cap,scale,shape)
  )
}



#' Exposure Curve from a Pareto severity distribution
#'
#' @param x A positive real number -  the claim amount where the exposure curve will be evaluated.
#' @param scale A positive real number - the scale parameter of the Claim Severity's Pareto distribution.
#' @param shape A positive real number - the shape parameter of the Claim Severity's Pareto distribution.
#' @return The value of the Exposure curve at \code{x} with Claim Severity from a Pareto distribution with parameters \code{scale} and \code{shape}.
#' @export
#' @examples
#' ExposureCurvePareto(700,500,1.2)
#' ExposureCurvePareto(20000,200,1.1)
ExposureCurvePareto<-function(x,scale,shape){
  df<-data.frame(x,scale,shape)
  ifelse(df$shape>1
         ,ParetoCappedMean(x,scale,shape)*(shape-1)/shape/scale
         ,0
  )
}



#' Increased Limit Factor Curve from a Pareto severity distribution
#'
#' @param xLow A positive real number -  the claim amount where the Increased Limit Factor Curve will be evaluated from.
#' @param xHigh A positive real number -  the claim amount where the Increased Limit Factor Curve will be evaluated to.
#' @param scale A positive real number - the scale parameter of the Claim Severity's Pareto distribution.
#' @param shape A positive real number - the shape parameter of the Claim Severity's Pareto distribution.
#' @return The value of the Increased Limit Factor curve from \code{xLow} to \code{xHigh} with Claim Severity from a Pareto distribution with parameters \code{scale} and \code{shape}.
#' @export
#' @examples
#' ILFPareto(700,1200,500,1.2)
#' ILFPareto(1200,20000,200,1.1)
ILFPareto<-function(xLow,xHigh,scale,shape){
  ParetoCappedMean(xHigh,scale,shape)/ParetoCappedMean(xLow,scale,shape)
}

## ----setup, include=FALSE-----------------------------------------------------
knitr::opts_chunk$set(echo = TRUE)

## ----pure IBNR example--------------------------------------------------------
library("NetSimR")
#set the seed
set.seed(0)

#Set the reporting delay distribution parameters
mu <- 3
sigma <- 1

#Generate reporting delays and preview the distribution
x<-rlnorm(1000,mu,sigma)
summary(x)
hist(x, breaks = 100)

## ----pure IBNR example continued, eval=requireNamespace("crch", quietly = TRUE)----
#Generate dates data
PoliciesPerDay<-100
PeriodLength<-365*2
InceptionDateStart<-as.Date("2011/1/1")
InceptionDate<-seq(InceptionDateStart, by = "day", length.out = PeriodLength)
InceptionDate<-rep(InceptionDate,PoliciesPerDay)
#assume all policies are yearly
DayPolicyDuration<-365
ExpiryDate<-InceptionDate+DayPolicyDuration
ValuationDate<-max(InceptionDate)
Data<-data.frame(InceptionDate,ExpiryDate)
summary(Data)
ValuationDate

#Generate Claims Counts Data and reporting delays
#Assume only 1 claim per policy possible with probability of claiming 10%
ClaimFreq<-0.1
Data$ClaimCount<-ifelse(runif(nrow(Data))<ClaimFreq,1,0)
#generate accident date assuming uniformly happening throughout the year
Data$AccidentDate<-Data$InceptionDate+runif(nrow(Data))*(Data$ExpiryDate-Data$InceptionDate)
#generate reporting delays
Data$ReportingDelay<-round(Data$ClaimCount*rlnorm(nrow(Data),mu,sigma),1)
#generate reporting days
Data$NotificationDate<-Data$AccidentDate+Data$ReportingDelay
#remove claims reported beyond valuation day
Data$ClaimReportedBeforeValuation<-ifelse(Data$NotificationDate>ValuationDate,0,Data$ClaimCount)
#Maximum reporting delays
Data$MaxReportingDelay<-as.numeric(ValuationDate-Data$AccidentDate)+1

#Earned Duration
Data$EarnedDuration<-ifelse(Data$ExpiryDate>ValuationDate,as.numeric(ValuationDate-Data$InceptionDate),as.numeric(Data$ExpiryDate-Data$InceptionDate))/365

#Filter claims only data
ModelData<-Data[Data$ClaimReportedBeforeValuation==1,]

#model reporting delays with a linear model
lm<-lm(log(ModelData$ReportingDelay)~1)
#summary(lm)
cbind(coefficients(lm),summary(lm)$sigma)

#model reporting delays with a truncated model (splitting to monthly periods would be more accurate in terms of maximum reporting delay)
rTLm<-crch::crch(log(ModelData$ReportingDelay)~1, truncated = TRUE, right = log(ModelData$MaxReportingDelay), dist = "gaussian", link.scale = "identity")
#summary(rTLm)
coefficients(rTLm)

#Generate and predict pure IBNR and UPR Claims
Data$PureIBNRClaims<-ifelse(Data$AccidentDate>ValuationDate,0,Data$ClaimCount-Data$ClaimReportedBeforeValuation)
Data$UPRClaims<-Data$ClaimCount-Data$ClaimReportedBeforeValuation-Data$PureIBNR
PureIBNRPrediction<-PureIBNRLNorm(Data$InceptionDate,Data$ExpiryDate,ValuationDate,coefficients(rTLm)[1],coefficients(rTLm)[2])
Data$PredictedPureIBNRYears<-PureIBNRPrediction$PureIBNRDuration/DayPolicyDuration
Data$PredictedUPRYears<-PureIBNRPrediction$UnearnedDuration/DayPolicyDuration
Data$NetEarnedDuration<-1-Data$PredictedUPRYears-Data$PredictedPureIBNRYears+0.00001

#Predict Claim Frequency with reduced exposure to adjust for pure IBNR
AdjFreqGLM<-glm(Data$ClaimReportedBeforeValuation~1, offset = log(Data$NetEarnedDuration), family = "poisson")
#summary(AdjFreqGLM)
PredictedFrequency<-exp(AdjFreqGLM$coefficients)

#Compare as if not adjusting for pure IBNR
UnAdjFreqGLM<-glm(Data$ClaimReportedBeforeValuation~1, offset = log(Data$EarnedDuration+0.00001), family = "poisson")
#summary(UnAdjFreqGLM)

cbind(AdjustedModel=round(exp(AdjFreqGLM$coefficients),3),UnAdjustedModel=round(exp(UnAdjFreqGLM$coefficients),3),Actual=ClaimFreq)


#Compare UPR Claim predictions
cbind(Actual=sum(Data$UPRClaims),Predicted=round(sum(Data$PredictedUPRYears)*PredictedFrequency,0))

#Compare Pure IBNR Claim predictions
cbind(Actual=sum(Data$PureIBNRClaims),Predicted=round(sum(Data$PredictedPureIBNRYears)*PredictedFrequency,0), Theoretical=327)

#Theoretical comes from re-running with the theoretical parameter values

#Compare Pure IBNR predictions with Chain Ladder
ClaimsAY1RY1<-sum(Data[Data$AccidentDate<as.Date("2012/1/1") & Data$NotificationDate<as.Date("2012/1/1"), ]$ClaimReportedBeforeValuation)

ClaimsAY1RY2<-sum(Data[Data$AccidentDate<as.Date("2012/1/1") & Data$NotificationDate<as.Date("2013/1/1"), ]$ClaimReportedBeforeValuation)

ClaimsAY2RY1<-sum(Data$ClaimReportedBeforeValuation)-ClaimsAY1RY1-ClaimsAY1RY2

PredictedPureIBNRClaimsChainLadder<-(ClaimsAY1RY2/ClaimsAY1RY1-1)*ClaimsAY2RY1

cbind(Actual=sum(Data$PureIBNRClaims),PredictedGLM=round(sum(Data$PredictedPureIBNRYears)*PredictedFrequency,0),PredictedCL=round(PredictedPureIBNRClaimsChainLadder,0))

#Note: Chain ladder can predict better if periods brake down to months or quarters

#Sliced LogNormal-Pareto claim Severity assumption
mu=5.6
sigma=1.65
s=10000
alpha=1.5

#Simulate the reserve distribution
numberOfSimulations=10000
SimulatedYears <- numeric(length = numberOfSimulations)
ExpectedFrequency <- sum(Data$PredictedPureIBNRYears)*PredictedFrequency

for (i in 1 :numberOfSimulations){
  SimulatedYears[i]=round(sum(qSlicedLNormPareto(runif(rpois(1,ExpectedFrequency)),mu,sigma,s,alpha)),0)
}

#Visualising pure IBNR reserve distribution
head(SimulatedYears)
hist(SimulatedYears, breaks = 1000, xlim = c(0,1.5e6))
summary(SimulatedYears)

#Note: the strength of the method illustrated is that it can have GLM factors for reporting delay, frequency and severity, and run the simulations per risk

#set seed and number of simulations
set.seed(1)
n=1000*1000


#set parameters
cap=3000
mu=6.5
sigma=1.7


#Capped mean
x<-rlnorm(n, mu, sigma)
hist(x, breaks = 2000, xlim = c(0,1.5e4))
y<-ifelse(x<cap,x,cap)
hist(y)
mean(y)
LNormCappedMean(cap,mu,sigma)


#exposure curve
mean(y)/mean(x)
ExposureCurveLNorm(cap,mu,sigma)


#ILF
z<-ifelse(y>cap/2,cap/2,y)
mean(y)/mean(z)
ILFLNorm(cap/2,cap,mu,sigma)
ILFLNorm(cap/2,c(cap,cap*1.1),mu,sigma)

#set seed and number of simulations
set.seed(1)
n=10*1000*1000

#set gamma parameters
gshape=1.5
rate=0.0004
SlicePoint=400
shape=1.5

#test pdf
x<-rgamma(n,gshape,rate)
x<-ifelse(x<SlicePoint,x,SlicePoint/(runif(n)^(1/shape)))
hist(x, breaks = 200000, xlim = c(0,1.5e4), probability = T)
lines(0:1e4,dSlicedGammaPareto(0:1e4,gshape,rate,SlicePoint,shape), col="red")

#test cdf
step<-0.001
plot(log(quantile(x,seq(0+step, 1-step, step)))
     ,pSlicedGammaPareto(quantile(x,seq(0+step, 1-step, step)),gshape,rate,SlicePoint,shape)
     ,type = "l"
)
lines(log(quantile(x,seq(0+step, 1-step, step))),seq(0+step, 1-step, step), type = "l", col="red")


#test inverese cdf
plot(log(qSlicedGammaPareto(seq(0+step, 1-step, step),gshape,rate,SlicePoint,shape))
     ,seq(0+step, 1-step, step)
     ,type = "l"
)
lines(log(quantile(x,seq(0+step, 1-step, step))),seq(0+step, 1-step, step), type = "l", col="red")

#' Server function for the Shiny Simulator application
#'
#' @param input Input for the server function.
#' @param output Output for the server function.
#' @param session Session for the server function.
#' @return Returns server rendering for the shiny application.
#' @import rmarkdown
#' @import shiny
#' @import shinybusy
#' @import future.apply
#' @import data.table
#' @importFrom future plan
#' @importFrom future sequential
#' @importFrom future multisession
#' @import rmarkdown
#' @import methods
#' @import stats
#' @import scales
#' @import utils
#' @import reactable
shiny_simulator_server = function(input, output, session) {
  #set data.table threads once per session, leave one core free for the main process
  data.table::setDTthreads(max(1, parallel::detectCores() - 1))

  #ensure any future workers are cleaned up when the session ends
  session$onSessionEnded(function() {
    future::plan(future::sequential)
  })

  #seed input
  output$seed_value <- renderUI({
    if (input$seedSetBinary) {
      sliderInput('seedValue', 'Seed value', min=1, max=100, value=1, step=1)
    }
  })

  #help function to render freq/sev parameters
  render_UI_param <- function(param_id, object){
    return(if(!is.na(object@paramIDs[param_id])){numericInput(
      inputId = object@paramIDs[param_id]
      ,label = object@param_labels[param_id]
      ,min = object@param_min_values[param_id]
      ,max = object@param_max_values[param_id]
      ,value = NULL
    )
    })}

  #render frequency parameters
  lapply(freq_dist_parameter_placeholders$param_number, function(i) {
    output[[paste0("freq_param_", i)]] <- renderUI({
      render_UI_param(param_id=i, object=freq_dist_options[[input$freqDistr]])
    })
  })

  #render severity parameters
  lapply(sev_dist_parameter_placeholders$param_number, function(i) {
    output[[paste0("sev_param_", i)]] <- renderUI({
      render_UI_param(param_id=i, object=sev_dist_options[[input$sevDistr]])
    })
  })

  #render Pareto slice parameters
  output$pareto_slice_times <- renderUI({
    if (input$paretoSlice) {
      selectInput("pareto_slice_times","Number of Pareto Slices",1:max_number_of_pareto_slices)
    }
  })

  lapply(1:(2*max_number_of_pareto_slices), function(i) {
    output[[paste0("slice_pareto_param_", i)]] <- renderUI({
      req(input$pareto_slice_times)
      if(input$paretoSlice){if(as.numeric(input$pareto_slice_times)*2>=i){
        numericInput(
          inputId = paste0("slice_pareto_param_", i)
          ,label = ifelse(i %% 2, paste("Sliced alpha", (1+i)/2), paste('Sliced x_m', i/2))
          ,value = NULL
          ,min = 0
        )
      }}
    })
  })

  #render severity cap amount
  output$sev_cap_amount <- renderUI({
    if (input$sevCapBinary) {
      numericInput('sev_cap_amount', 'Severity Cap Amount', value = NULL, min = 0)
    }
  })

  #reinsurance structure EEL inputs
  output$reinsuranceStructureDeductibleEEL <- renderUI({
    if (input$reinsuranceStructureEEL %in% c('Unlimited Layer', 'Exclude Layer', 'Limited Layer')) {
      numericInput('reinsurance_structure_eel_dedctible_amount', 'EEL Deductible Amount', value = NULL, min = 0)
    }
  })

  output$reinsuranceStructureLimitEEL <- renderUI({
    if (input$reinsuranceStructureEEL %in% c('Limited Layer', 'Exclude Layer')) {
      numericInput('reinsurance_structure_eel_limit_amount', 'EEL Limit Amount', value = NULL, min = 0)
    }
  })

  output$reinsuranceStructureLimitedReinstatements <- renderUI({
    if (input$reinsuranceStructureEEL %in% c('Limited Layer')) {
      checkboxInput('reinsuranceStructureLimitedReinstatements', 'Limited Reinstatments', value = F)
    }
  })

  output$reinsuranceStructureReinstatementLimit <- renderUI({
    req(input$reinsuranceStructureLimitedReinstatements)
    if (input$reinsuranceStructureEEL %in% c('Limited Layer')) {
      if(input$reinsuranceStructureLimitedReinstatements) {
        numericInput('reinsuranceStructureReinstatementLimit', 'Number of Reinstatements', value = NULL, min = 0)
      }
    }
  })

  #reinsurance structure AL inputs
  output$reinsuranceStructureDeductibleAL <- renderUI({
    if (input$reinsuranceStructureAL %in% c('Unlimited Layer', 'Exclude Layer', 'Limited Layer')) {
      numericInput('reinsurance_structure_al_dedctible_amount', 'AL Deductible Amount', value = NULL, min = 0)
    }
  })

  output$reinsuranceStructureLimitAL <- renderUI({
    if (input$reinsuranceStructureAL %in% c('Limited Layer', 'Exclude Layer')) {
      numericInput('reinsurance_structure_al_limit_amount', 'AL Limit Amount', value = NULL, min = 0)
    }
  })

  #create simulation data dataFrame reactive to enable download buttons & simulation settings list
  simulated_data <- reactiveValues(data=NULL)
  simulation_settings <- list()
  is_running <- reactiveVal(FALSE)

  observe({
    if (is_running()) {
      updateActionButton(session, "RunSimulations", label = "Running...")
    } else {
      updateActionButton(session, "RunSimulations", label = "Run Simulations")
    }
  })

  #run simulation button
  observeEvent(input$RunSimulations,{
    if (is_running()) return(NULL)
    is_running(TRUE)

    on.exit({
      is_running(FALSE)
      hide_spinner()
      gc()
    }, add = TRUE)

    show_spinner()

    #set parameters
    simulation_settings <<- list(
      freq_params = unname(sapply(
        freq_dist_options[[input$freqDistr]]@paramIDs
        ,function(x) input[[x]]
      ))
      ,sev_params = unname(sapply(
        sev_dist_options[[input$sevDistr]]@paramIDs
        ,function(y) input[[y]]
      ))
      ,numOfSimulations = input$numberOfSimulations
      ,seedSetBinary = input$seedSetBinary
      ,seedValue = input$seedValue
      ,freqDistr = input$freqDistr
      ,sevDistr = input$sevDistr
      ,paretoSlice = input$paretoSlice
      ,pareto_slice_times = as.numeric(input$pareto_slice_times)
      ,slice_pareto_alphas = if(input$paretoSlice){unname(sapply(
        1:as.numeric(input$pareto_slice_times)
        ,function(y) input[[paste0("slice_pareto_param_", y*2-1)]]
      ))}
      ,slice_pareto_x_ms = if(input$paretoSlice){unname(sapply(
        1:as.numeric(input$pareto_slice_times)
        ,function(y) input[[paste0("slice_pareto_param_", y*2)]]
      ))}
      ,sevCapBinary = input$sevCapBinary
      ,sev_cap_amount = input$sev_cap_amount
      ,reinsuranceStructureEEL = input$reinsuranceStructureEEL
      ,reinsurance_structure_eel_dedctible_amount = input$reinsurance_structure_eel_dedctible_amount
      ,reinsurance_structure_eel_limit_amount = input$reinsurance_structure_eel_limit_amount
      ,reinsuranceStructureAL = input$reinsuranceStructureAL
      ,reinsurance_structure_al_dedctible_amount = input$reinsurance_structure_al_dedctible_amount
      ,reinsurance_structure_al_limit_amount = input$reinsurance_structure_al_limit_amount
      ,multiprocessing = input$multiprocessingBinary
      ,reinsuranceStructureLimitedReinstatements = input$reinsuranceStructureLimitedReinstatements
      ,reinsuranceStructureReinstatementLimit = input$reinsuranceStructureReinstatementLimit
    )
    #run simmulations
    simulated_data$data <- tryCatch(
      {
        do.call(simulate_function, simulation_settings)
      }, error = function(cond) {
        future::plan(future::sequential)
        showNotification(
          paste("Error:", conditionMessage(cond)),
          type = "error",
          duration = NULL
        )
        print(cond)
        NULL
      }
    )
  })

  is_downloading_report <- reactiveVal(FALSE)

  output$downloadReportButton <- renderUI({
    req(simulated_data$data)

    label <- if (is_downloading_report()) {
      "Preparing report..."
    } else {
      "Report"
    }

    downloadButton('downloadReportHandler', label)
  })

  #download data button
  output$DownloadDataHandler <- downloadHandler(
    filename = function() { paste0('Simulation_data', '.csv') },
    content = function(file) {
      showNotification(
        "Preparing data file, save dialog will appear shortly...",
        type = "message",
        duration = 2
      )
      write.csv(simulated_data$data, file, row.names = FALSE)
    }
  )

  output$downloadDataButton <- renderUI({
    req(simulated_data$data)
    downloadButton('DownloadDataHandler', 'Data')
  })

  #download report button
  output$downloadReportHandler <- downloadHandler(
    filename = "simulation_report.html",
    content = function(file) {
      is_downloading_report(TRUE)
      on.exit(is_downloading_report(FALSE), add = TRUE)

      showNotification(
        "Preparing report, save dialog will appear shortly...",
        type = "message",
        duration = 3
      )

      shiny::withProgress(message = "Preparing report", value = 0, {
        incProgress(0.2, detail = "Copying template")
        tempReport <- normalizePath(file.path(tempdir(), "ShinySimulatorReport.Rmd"), mustWork = FALSE)
        file.copy(
          normalizePath(system.file("rmd", "ShinySimulatorReport.Rmd", package = "NetSimR")),
          tempReport,
          overwrite = TRUE
        )

        incProgress(0.5, detail = "Rendering report")
        rmarkdown::render(
          tempReport,
          output_file = file,
          quiet = TRUE,
          params = append(simulation_settings, list(total_claims_data = simulated_data$data$total_claims)),
          envir = new.env(parent = globalenv())
        )

        incProgress(1, detail = "Done")
      })
    }
  )
}

#set seed and number of simulations
set.seed(1)
n=1000*1000


#set parameters
cap=5000
scale=2000
shape=1.7


#Capped mean
x<-scale/(runif(n)^(1/shape))
hist(x, breaks = 20000, xlim = c(0,2e4))
y<-ifelse(x<cap,x,cap)
hist(y)
mean(y)
ParetoCappedMean(cap,scale,shape)


#exposure curve
mean(y)/mean(x)
ExposureCurvePareto(cap,scale,shape)


#ILF
z<-ifelse(y>cap/2,cap/2,y)
mean(y)/mean(z)
ILFPareto(cap/2,cap,scale,shape)
ILFPareto(cap/2,cap,c(scale,scale*1.1),shape)

################
#Gamma Functions
################

#' Lower incomplete gamma function
#'
#' @param x A positive real number.
#' @param a A positive real number.
#' @return The value of the lower incomplete gamma function at \code{x} with shape parameter \code{a}.
#' @export
#' @examples
#' IGamma(1,1)
#' IGamma(0.1,2)
IGamma<-function(a,x){
  gamma(a) * pgamma(x, shape = a, scale = 1, lower.tail = FALSE)
}



#' Gamma capped mean
#'
#' @param cap A positive real number -  the claim severity cap.
#' @param shape A positive real number - the shape parameter of the Claim Severity's Gamma distribution.
#' @param rate A positive real number - the rate parameter of the Claim Severity's Gamma distribution.
#' @return The mean of the claim severity capped at \code{cap} with a Gamma distribution with parameters \code{shape} and \code{rate}.
#' @export
#' @examples
#' GammaCappedMean(700,1,0.0005)
#' GammaCappedMean(1000,1.5,0.0006)
GammaCappedMean<- function(cap,shape,rate){
  (shape * gamma(shape) + rate * cap * IGamma(shape, rate * cap) - IGamma(1 + shape, rate * cap))/(rate * gamma(shape))
}



#' Exposure Curve from a Gamma severity distribution
#'
#' @param x A positive real number -  the claim amount where the exposure curve will be evaluated.
#' @param shape A positive real number - the shape parameter of the Claim Severity's Gamma distribution.
#' @param rate A positive real number - the rate parameter of the Claim Severity's Gamma distribution.
#' @return The value of the Exposure curve at \code{x} with Claim Severity from a Gamma distribution with parameters \code{shape} and \code{rate}.
#' @export
#' @examples
#' ExposureCurveGamma(700,1,0.0005)
#' ExposureCurveGamma(1000,1.5,0.0006)
ExposureCurveGamma<-function(x,shape,rate){
  GammaCappedMean(x,shape,rate)*rate/shape
}



#' Increased Limit Factor Curve from a Gamma severity distribution
#'
#' @param xLow A positive real number -  the claim amount where the Increased Limit Factor Curve will be evaluated from.
#' @param xHigh A positive real number -  the claim amount where the Increased Limit Factor Curve will be evaluated to.
#' @param shape A positive real number - the shape parameter of the Claim Severity's Gamma distribution.
#' @param rate A positive real number - the rate parameter of the Claim Severity's Gamma distribution.
#' @return The value of the Increased Limit Factor curve from \code{xLow} to \code{xHigh} with Claim Severity from a Gamma distribution with parameters \code{shape} and \code{rate}.
#' @export
#' @examples
#' ILFGamma(1000,700,1,0.0005)
#' ILFGamma(1200,1000,1.5,0.0006)
ILFGamma<-function(xLow,xHigh,shape,rate){
  GammaCappedMean(xHigh,shape,rate)/GammaCappedMean(xLow,shape,rate)
}



#' Pure IBNR exposure from a Gamma reporting delay distribution
#'
#' @param IncDate A date -  the inception date of the period.
#' @param ExpDate A date -  the expiry date of the period. Must be greater than inception date.
#' @param ValDate A date -  the valuation date.
#' @param shape A positive real number - the shape parameter of the reporing delay's Gamma distribution.
#' @param rate A positive real number - the rate parameter of the reporing delay's Gamma distribution.
#' @return Unearned and Pure IBNR exposure in days and as a percentage of the period's duration, where the reporting delay has a Gamma distribution with parameters \code{shape} and \code{rate}.
#' @export
#' @examples
#' Dates = data.frame(
#'     inceptionDate = c("01/01/2006", "01/07/2006", "01/01/2007")
#'     ,expiryDate = c("31/12/2006", "30/06/2007", "31/12/2007")
#' )
#'
#' Dates$inceptionDate<-as.POSIXct(Dates$inceptionDate, format="%d/%m/%Y")
#'
#' Dates$expiryDate<-as.POSIXct(Dates$expiryDate, format="%d/%m/%Y")
#'
#' ValuationDate<-as.POSIXct("30/10/2007", format="%d/%m/%Y")
#'
#' PureIBNRGamma(Dates$inceptionDate,Dates$expiryDate,ValuationDate,7,0.15)
##Pure IBNR Gamma
PureIBNRGamma <- function(IncDate, ExpDate, ValDate, shape, rate){
  MinRepDelay<-ifelse(ValDate<ExpDate,0,difftime(ValDate, ExpDate, units="days"))
  MaxRepDelay<-ifelse(ValDate<IncDate,0,difftime(ValDate, IncDate, units="days"))
  Duration<-as.numeric(difftime(ExpDate,IncDate, units="days"))
  EarnedDuration<-MaxRepDelay-MinRepDelay
  UnearnedDuration<-Duration-EarnedDuration
  UnearnedDurationRatio<-ifelse(Duration==0,0,round(UnearnedDuration/Duration,5))
  PureIBNRDuration<-round(GammaCappedMean(MaxRepDelay,shape,rate)-GammaCappedMean(MinRepDelay,shape,rate),2)
  PureIBNRDurationRatio<-ifelse(Duration==0,0,round(PureIBNRDuration/Duration,5))
  data.frame(UnearnedDuration,PureIBNRDuration,UnearnedDurationRatio,PureIBNRDurationRatio)
}

#' Server function for the Distribution Fitting tool application
#'
#' @param input Input for the server function.
#' @param output Output for the server function.
#' @param session Session for the server function.
#' @return Returns server rendering for the shiny application.
#' @import shiny
#' @import MASS
#' @importFrom plotly plot_ly add_lines layout add_bars renderPlotly
#' @import fitdistrplus
#' @import Pareto
#' @import reactable

distribution_fitting_tool_Server = function(input, output, session) {

  ######################
  #help functions
  ######################

  clean_and_convert_to_numeric <- function(input_vector, greater_than_zero = F){
    # Remove non-numeric elements
    numeric_data <- as.numeric(input_vector[!is.na(as.numeric(input_vector))])
    # Remove negative values
    if (greater_than_zero) {
      numeric_data <- numeric_data[numeric_data > 0]
    } else {
      numeric_data <- numeric_data[numeric_data >= 0]
    }
    return(numeric_data)
  }

  generate_xaxis_points <- function(claim_vector, num_of_points) {
    claim_vector_unique <- unique(claim_vector)
    if (max(claim_vector_unique) < num_of_points) {
      return(0:max(claim_vector_unique))
    } else if (length(claim_vector_unique) <= num_of_points) {
      return(sort(unique(claim_vector_unique)))
    } else {
      num_of_individual_tail_points <- num_of_points * 0.2
      tail_points <- head(claim_vector_unique[order(claim_vector_unique, decreasing = TRUE)], n = num_of_individual_tail_points)
      step_sliced_sev <- round(max(claim_vector_unique)/num_of_points,0)+1
      xaxis <- seq(0, max(claim_vector_unique), step_sliced_sev)
      xaxis <- xaxis[xaxis<min(tail_points)]
      xaxis <- c(xaxis, tail_points)
      return(sort(xaxis))
    }
  }

  kstest <- function(claims, predictions){round(max(abs(rank(claims)/(length(claims)+1)-predictions))*sqrt(length(claims)),2)}

  existence_check_shiny <- function(df){
    tryCatch({
      df
      return(T)
    },
    shiny.silent.error = function(e) {return(F)}
    )
  }


  fit_slice_pareto <- function (sev_data, slic_pont_lft, slic_pont_rght) {
    z <- subset(sev_data, sev_data > slic_pont_lft)
    w <- subset(z, z <= slic_pont_rght)
    dat <- data.frame(severity = w, empirical = rank(w) / (length(z) + 1))
    SumOfSquares <- function(data, par) {
      sum((1 - (slic_pont_lft / data$severity)^par[1] - data$empirical)^2)
    }
    x=1
    result <- optim(par = x, SumOfSquares, data = dat, lower = 0.0001, method = "L-BFGS-B")
    return(result$par)
  }

  ######################
  #Data input
  ######################

  data <- reactive({
    req(input$file1)
    inFile <- input$file1
    header <- if (input$data_includes_header) TRUE else FALSE
    df <- read.csv(inFile$datapath, header = header, sep = input$sep, quote = input$quote)
    updateSelectInput(session, inputId = 'counts_var', label = 'Frequency Variable',
                      choices = names(df), selected = names(df))
    updateSelectInput(session, inputId = 'counts_weights_var', label = 'Weights Variable',
                      choices = names(df), selected = names(df))
    updateSelectInput(session, inputId = 'severity_var', label = 'Severity Variable',
                      choices = names(df), selected = names(df))
    updateSelectInput(session, inputId = 'sliced_sev_var', label = 'Sliced Severity Variable',
                      choices = names(df), selected = names(df))
    updateSelectInput(session, inputId = 'piecewise_pareto_var', label = 'Piecwise Pareto Severity Variable',
                      choices = names(df), selected = names(df))
    return(df)
  })

  output$data_table <- reactable::renderReactable({
    reactable::reactable(data())
  })

  ######################
  #Frequency analysis
  ######################

  counts_data <- eventReactive(input$execute_freq_analysis, {
    clean_and_convert_to_numeric(data()[, input$counts_var])
  })
  weights_data <- eventReactive(input$execute_freq_analysis, {
    if (input$counts_weighted_var) {
      clean_and_convert_to_numeric(data()[, input$counts_weights_var])
    } else { NULL }
  })

  freq_nb_fit <- reactive({
    if (input$counts_weighted_var) {
      suppressWarnings(fitdist(counts_data(), "nbinom", weights = weights_data()))
    } else {
      suppressWarnings(fitdist(counts_data(), "nbinom"))
    }
  })
  freq_po_fit <- reactive({
    if (input$counts_weighted_var) {
      suppressWarnings(fitdist(counts_data(), "pois", weights = weights_data()))
    } else {
      suppressWarnings(fitdist(counts_data(), "pois"))
    }
  })
  mean_counts <- reactive({mean(counts_data())})
  median_counts <- reactive({median(counts_data())})
  var_counts <- reactive({var(counts_data())})

  output$selected_freq_params <- renderPrint({
    param <- NULL
    if (input$FreqDistri == "Poisson") {
      param <- data.frame(Mean = format(freq_po_fit()$estimate[1], scientific = TRUE, digits = 4))
    } else {
      param <- data.frame(
        Beta = format(freq_nb_fit()$estimate[2]/freq_nb_fit()$estimate[1], scientific = TRUE, digits = 4), # also mu/size
        r = format(freq_nb_fit()$estimate[1], scientific = TRUE, digits = 4) # r is also called size
      )
    }
    rownames(param) <- "Value"
    noquote(format(param, justify = "left"))
  })

  output$freq_summary <- renderPrint({summary(counts_data())})

  output$distribution_proposal <- renderPrint({
    summary <- matrix(c(
      format(round(mean_counts(),2), big.mark=",")
      ,format(round(var_counts(),2), big.mark=",")
      ,if (var_counts()>mean_counts()) {"Negative Binomial"} else {"Poisson"}
    ), nrow = 1, ncol = 3, byrow = TRUE)
    colnames(summary)<-c("Mean  ","Variance  ","Recomended Freq. Distr.")
    rownames(summary)<-c("Value  ")
    noquote(summary)
  })

  output$selected_distribution_summary <- renderPrint({
    if (input$FreqDistri == "Poisson") {summary(freq_po_fit())} else {summary(freq_nb_fit())}
  })

  output$count_hist <- renderPlotly({
    p <- plot_ly(
      x = counts_data()
      ,type = "histogram"
      ,nbinsx  = input$count_hist_bins
      ,marker = list(line = list(color = "black", width = 2))
      ,name = "Empirical pdf"
    )
    p <- layout(p, xaxis = list(title = "Number of claims"), yaxis = list(title = "Counts"))
    p <- layout(p, title = "Histogram of Claim Counts")
    return(p)
  })

  output$freq_fit_plot <- renderPlotly({
    x_axis <- generate_xaxis_points(counts_data(), 1000)
    empirical_cdf <- unlist(lapply(x_axis, function(x) sum(counts_data() <= x)))/length(counts_data())
    p <- plot_ly(x = x_axis, y = empirical_cdf, type = "scatter", mode = "lines", name = "Empirical cdf")
    p <- layout(p, xaxis = list(title = "Claim frequency"), yaxis = list(title = "CDF"))
    p <- layout(p, title =  "Claim Frequency CDF and model fit")
    poisson_cdf <- ppois(x_axis, lambda = freq_po_fit()$estimate[1])
    p <- add_lines(p, x = x_axis, y = poisson_cdf, type = 'scatter', mode = 'lines', name = 'Poisson cdf')
    nb_cdf <- pnbinom(x_axis, size = freq_nb_fit()$estimate[1], mu = freq_nb_fit()$estimate[2])
    p <- add_lines(p, x = x_axis, y = nb_cdf, type = 'scatter', mode = 'lines', name = 'Negative Binomial cdf')
    return(p)
  })

  ######################
  #Severity analysis
  ######################

  severity_data <- eventReactive(input$execute_sev_analysis, {
    clean_and_convert_to_numeric(data()[, input$severity_var], T)
  })

  sev_norm_fit <- reactive({fitdist(severity_data(), "norm")})
  sev_lnorm_fit <- reactive({fitdist(severity_data(), "lnorm")})
  sev_gamma_fit <- reactive({fitdistr(severity_data(), 'gamma', method = "L-BFGS-B", lower = c(0, 0), start = list(scale = 1, shape = 1))})

  sev_pareto_xm <- reactive({ min(severity_data()) })
  sev_pareto_alpha <- reactive({ length(severity_data())/(sum(log(severity_data()))-length(severity_data())*log(sev_pareto_xm())) })


  output$sev_param_summary <- renderPrint({
    KSNorm <- kstest(severity_data(), pnorm(severity_data(),sev_norm_fit()$estimate[1],sev_norm_fit()$estimate[2]))
    KSlNorm <- kstest(severity_data(), plnorm(severity_data(), meanlog = sev_lnorm_fit()$estimate[1], sdlog = sev_lnorm_fit()$estimate[2]))
    KSPareto <- kstest(severity_data(), 1-(sev_pareto_xm()/severity_data())^sev_pareto_alpha())
    KSExpo <- kstest(severity_data(), pexp(severity_data(),1/sev_norm_fit()$estimate[1]))
    KSGamma <- kstest(severity_data(), pgamma(severity_data(),scale = sev_gamma_fit()$estimate[1],shape = sev_gamma_fit()$estimate[2]))

    param <- matrix(c(round(sev_norm_fit()$estimate[1],0),round(sev_norm_fit()$estimate[2],2),KSNorm
                      ,"","","","Mu","Sigma", "K-S Dist",round(sev_lnorm_fit()$estimate[1],4),round(sev_lnorm_fit()$estimate[2],4),KSlNorm
                      ,"","","","xm","alpha", "K-S Dist",round(sev_pareto_xm(),0),round(sev_pareto_alpha(),4),KSPareto
                      ,"","","","Mean","", "K-S Dist",round(sev_norm_fit()$estimate[1],0),"",KSExpo
                      ,"","","","Scale","Shape", "K-S Dist",round(sev_gamma_fit()$estimate[1],4),round(sev_gamma_fit()$estimate[2],4),KSGamma
    ),
    nrow = 13, ncol = 3, byrow = TRUE)
    colnames(param)<-c("Mean", "Std. Dev.", "K-S Dist")
    rownames(param)<-c("Normal","","","LogNormal","","","Pareto","","","Exponential","","","Gamma")
    noquote(param)
  })


  output$sev_hist <- renderPlotly({
    p <- plot_ly(
      x = severity_data()
      ,type = "histogram"
      ,nbinsx  = input$severity_hist_bins
      ,marker = list(line = list(color = "black", width = 2))
      ,name = "Empirical pdf"
    )
    p <- layout(p, xaxis = list(title = "Claim Severity"), yaxis = list(title = "Severity"))
    p <- layout(p, title = "Histogram of Claim Severity")
    return(p)
  })

  output$sev_summary <- renderPrint({summary(severity_data())})

  output$sev_fit_plot <- renderPlotly({
    x_axis <- generate_xaxis_points(severity_data(), 1000)
    x_axis_after_log_option <- if (input$sev_fit_log_scale) {log(x_axis)} else {x_axis}
    empirical_cdf <- unlist(lapply(x_axis, function(x) sum(severity_data() <= x)))/length(severity_data())
    p <- plot_ly(x = x_axis_after_log_option, y = empirical_cdf, type = "scatter", mode = "lines", name = "Empirical cdf")
    p <- layout(p, xaxis = list(title = paste("Claim Severity", ifelse(T,"Log Scale",""))), yaxis = list(title = "Severity log scale"))
    p <- layout(p, title =  paste("Claim Severity and severity model fit", ifelse(T,"Log Scale","")))
    normal_cdf <- pnorm(x_axis, mean=sev_norm_fit()$estimate[1], sd=sev_norm_fit()$estimate[2])
    p <- add_lines(p, x = x_axis_after_log_option, y = normal_cdf, type = 'scatter', mode = 'lines', name = 'Normal cdf')
    lnormal_cdf <- plnorm(x_axis, meanlog=sev_lnorm_fit()$estimate[1], sdlog=sev_lnorm_fit()$estimate[2])
    p <- add_lines(p, x = x_axis_after_log_option, y = lnormal_cdf, type = 'scatter', mode = 'lines', name = 'LogNormal cdf')
    exponential_cdf <- pexp(x_axis, 1/sev_norm_fit()$estimate[1])
    p <- add_lines(p, x = x_axis_after_log_option, y = exponential_cdf, type = 'scatter', mode = 'lines', name = 'Exponential cdf')
    gamma_cdf <- pgamma(x_axis, scale = sev_gamma_fit()$estimate[1], shape = sev_gamma_fit()$estimate[2])
    p <- add_lines(p, x = x_axis_after_log_option, y = gamma_cdf, type = 'scatter', mode = 'lines', name = 'Gamma cdf')
    pareto_cdf <- 1-(sev_pareto_xm()/x_axis)^sev_pareto_alpha()
    p <- add_lines(p, x = x_axis_after_log_option, y = pareto_cdf, type = 'scatter', mode = 'lines', name = 'Pareto cdf')
    return(p)
  })


  ######################
  #Sliced Sev analysis
  ######################

  sliced_sev_data <- eventReactive(input$execute_sliced_sev_analysis, {
    clean_and_convert_to_numeric(data()[, input$sliced_sev_var], T)
  })

  x_axis_sliced_sev <- eventReactive(input$execute_sliced_sev_analysis, {
    generate_xaxis_points(sliced_sev_data(), 1000)
  })

  output$mean_excess_func_plot <- renderPlotly({
    p <- plot_ly(
      y = unlist(lapply(x_axis_sliced_sev(), function(x) mean(x_axis_sliced_sev()[x_axis_sliced_sev() >= x])))
      ,x = x_axis_sliced_sev(), type = "scatter", mode = "lines"
    )
    p <- layout(
      p,
      xaxis = list(title = "Claim Severity"),
      yaxis = list(title = "Mean Excess Function"),
      title = "Mean Excess Function of Claim Severity"
    )
  })

  observeEvent(input$execute_sliced_sev_analysis,{
    updateSliderInput(
      session
      ,"slicing_point_left"
      ,min = round(min(sliced_sev_data()),0)
      ,max = round(max(sliced_sev_data()),0)
      ,value = (round(min(sliced_sev_data()),0)+round(max(sliced_sev_data()),0))/2
    )
  })

  observeEvent(input$execute_sliced_sev_analysis,{
    updateSliderInput(
      session
      ,"slicing_point_right"
      ,min = (round(min(sliced_sev_data()),0)+round(max(sliced_sev_data()),0))/2
      ,max = round(max(sliced_sev_data()),0)
      ,value = 3*(round(min(sliced_sev_data()),0)+round(max(sliced_sev_data()),0))/4
    )
  })

  observeEvent(input$slicing_point_left,{
    updateSliderInput(
      session
      ,"slicing_point_right"
      ,min = input$slicing_point_left
      ,value = (input$slicing_point_left+round(max(sliced_sev_data()),0))/2
    )
    update
  })

  slc_sev_lnorm_fit <- reactive({fitdist(sliced_sev_data(), "lnorm")})

  slc_sev_censored_lnorm_fit <- reactive({
    tryCatch({
      if(!existence_check_shiny(sliced_sev_data())){return(data.frame(par=c(1,2)))}
      w <- subset(sliced_sev_data(), sliced_sev_data() <= input$slicing_point_left)
      dat <- data.frame(severity = w, empirical = rank(w) / (length(sliced_sev_data()) + 1))
      SumOfSquares <- function(data, par) {
        sum((plnorm(data$severity, meanlog = par[1], sdlog = par[2]) - data$empirical)^2)
      }
      result <- optim(par = c(mean(log(w)), sd(log(w))), SumOfSquares, data = dat, lower=0.0001, method = "L-BFGS-B")
      if(result$convergence != 0) {return(NULL)}
      return(result)
    }, error = function(e) {
      cat("An error occurred:", conditionMessage(e), "\n")
      return(e)
    })
  })

  zGreaterThanXm1 <- reactive({ sliced_sev_data()[sliced_sev_data()>input$slicing_point_left] })
  zGreaterThanXm2 <- reactive({ sliced_sev_data()[sliced_sev_data()>input$slicing_point_right] })
  paretoX1Alpha <- reactive({ length(zGreaterThanXm1())/(sum(log(zGreaterThanXm1()))-length(zGreaterThanXm1())*log(input$slicing_point_left)) })
  paretoX2AlphaMod <- reactive({
    tryCatch({
      if(!existence_check_shiny(sliced_sev_data())){return(1)}
      return(fit_slice_pareto(sliced_sev_data(), input$slicing_point_left, input$slicing_point_right))
    }, error = function(e) {
      cat("An error occurred: ", conditionMessage(e), "\n")
      return(e)
    })
  })
  paretoX2Alpha <- reactive({ length(zGreaterThanXm2())/(sum(log(zGreaterThanXm2()))-length(zGreaterThanXm2())*log(input$slicing_point_right)) })
  PrXLessThanParetoX1 <- reactive({ plnorm(input$slicing_point_left, meanlog = slc_sev_censored_lnorm_fit()$par[1], sdlog = slc_sev_censored_lnorm_fit()$par[2]) })

  output$sliced_sev_cdf_plot <- renderPlotly({
    LogNormalParetoInv<- function(x){
      ifelse(
        x<=input$slicing_point_left
        ,plnorm(x, meanlog = slc_sev_censored_lnorm_fit()$par[1], sdlog = slc_sev_censored_lnorm_fit()$par[2])
        ,(1-(input$slicing_point_left/x)^paretoX1Alpha())*(1-PrXLessThanParetoX1())+PrXLessThanParetoX1()

      )
    }

    PrXLessThanParetoX2<-(1-(input$slicing_point_left/input$slicing_point_right)^paretoX2AlphaMod())*(1-PrXLessThanParetoX1())+PrXLessThanParetoX1()

    LogNormalParetoParetoInv<- function(x){
      ifelse(
        x<=input$slicing_point_right
        ,ifelse(
          x<=input$slicing_point_left
          ,plnorm(x, meanlog = slc_sev_censored_lnorm_fit()$par[1], sdlog = slc_sev_censored_lnorm_fit()$par[2])
          ,(1-(input$slicing_point_left/x)^paretoX2AlphaMod())*(1-PrXLessThanParetoX1())+PrXLessThanParetoX1()
        )
        ,(1-(input$slicing_point_right/x)^paretoX2Alpha())*(1-PrXLessThanParetoX2)+PrXLessThanParetoX2

      )
    }

    x_axis_after_log_option <- if (input$sev_cens_fit_log_scale) {log(x_axis_sliced_sev())} else {x_axis_sliced_sev()}
    empirical_cdf <- unlist(lapply(x_axis_sliced_sev(), function(x) sum(sliced_sev_data() <= x)))/length(sliced_sev_data())
    p <- plot_ly(x = x_axis_after_log_option, y = empirical_cdf, type = "scatter", mode = "lines", name = "Empirical cdf")
    p <- layout(p, xaxis = list(title = paste("Claim Severity", ifelse(T,"Log Scale",""))), yaxis = list(title = "Severity log scale"))
    p <- layout(p, title =  paste("Claim Severity and severity model fit", ifelse(T,"Log Scale","")))
    lnormal_cdf <- plnorm(x_axis_sliced_sev(), meanlog=slc_sev_lnorm_fit()$estimate[1], sdlog=slc_sev_lnorm_fit()$estimate[2])
    p <- add_lines(p, x = x_axis_after_log_option, y = lnormal_cdf, type = 'scatter', mode = 'lines', name = 'LogNormal cdf')
    LogNormalParetoInv <- LogNormalParetoInv(x_axis_sliced_sev())
    p <- add_lines(p, x = x_axis_after_log_option, y = LogNormalParetoInv, type = 'scatter', mode = 'lines', name = 'LogNormal - Pareto cdf')
    LogNormalParetoParetoInv <- LogNormalParetoParetoInv(x_axis_sliced_sev())
    p <- add_lines(p, x = x_axis_after_log_option, y = LogNormalParetoParetoInv, type = 'scatter', mode = 'lines', name = 'LogNormal - Pareto - Pareto cdf')
    return(p)
  })



  output$slc_sev_fitted_param_summary <- renderPrint({
    LogNormalParetoInv<- function(x){
      ifelse(
        x<=input$slicing_point_left
        ,plnorm(x, meanlog = slc_sev_censored_lnorm_fit()$par[1], sdlog = slc_sev_censored_lnorm_fit()$par[2])
        ,(1-(input$slicing_point_left/x)^paretoX1Alpha())*(1-PrXLessThanParetoX1())+PrXLessThanParetoX1()

      )
    }

    PrXLessThanParetoX2<-(1-(input$slicing_point_left/input$slicing_point_right)^paretoX2AlphaMod())*(1-PrXLessThanParetoX1())+PrXLessThanParetoX1()

    LogNormalParetoParetoInv<- function(x){
      ifelse(
        x<=input$slicing_point_right
        ,ifelse(
          x<=input$slicing_point_left
          ,plnorm(x, meanlog = slc_sev_censored_lnorm_fit()$par[1], sdlog = slc_sev_censored_lnorm_fit()$par[2])
          ,(1-(input$slicing_point_left/x)^paretoX2AlphaMod())*(1-PrXLessThanParetoX1())+PrXLessThanParetoX1()
        )
        ,(1-(input$slicing_point_right/x)^paretoX2Alpha())*(1-PrXLessThanParetoX2)+PrXLessThanParetoX2

      )
    }

    KSlNorm <- kstest(sliced_sev_data(), plnorm(sliced_sev_data(), meanlog = slc_sev_lnorm_fit()$estimate[1], sdlog = slc_sev_lnorm_fit()$estimate[2]))
    KSlNormPareto <- kstest(sliced_sev_data(), LogNormalParetoInv(sliced_sev_data()))
    KSlNormParetoPareto <- kstest(sliced_sev_data(), LogNormalParetoParetoInv(sliced_sev_data()))

    param <- matrix(c(round(slc_sev_lnorm_fit()$estimate[1],4),round(slc_sev_lnorm_fit()$estimate[2],4),KSlNorm
                      ,"","","","mu","sigma", "",round(slc_sev_censored_lnorm_fit()$par[1],4),round(slc_sev_censored_lnorm_fit()$par[2],4),""
                      ,"","","","xm","alpha", "K-S Dist",input$slicing_point_left,round(paretoX1Alpha(),4),KSlNormPareto
                      ,"","","","xm","alpha", "",input$slicing_point_left,round(paretoX2AlphaMod(),4),""
                      ,"","","","xm","alpha", "K-S Dist",input$slicing_point_right,round(paretoX2Alpha(),4),KSlNormParetoPareto
    ),
    nrow = 13, ncol = 3, byrow = TRUE)
    colnames(param)<-c("Mu", "Sigma", "K-S Dist")
    rownames(param)<-c("LogNormal","","","LogNormal Sliced","","","Pareto x1","","","Pareto x2 lower","","","Pareto x2 upper")
    noquote(param)
  })



  ######################
  #piecewise pareto
  ######################


  piecewise_sev_data <- eventReactive(input$execute_piecewise_sev_analysis, {
    sort(clean_and_convert_to_numeric(data()[, input$piecewise_pareto_var], T))
  })

  # Create a reactive to generate the dynamic sliders
  dynamic_sliders <- eventReactive(list(input$num_pareto_slices, input$execute_piecewise_sev_analysis), {
    num_sliders <- input$num_pareto_slices
    slider_list <- lapply(1:num_sliders, function(i) {
      min_val <- sort(piecewise_sev_data())[2]
      max_val <- sort(piecewise_sev_data())[length(piecewise_sev_data())-1]
      sliderInput(paste0("slider_", i), label = paste("Slider", i), min = min_val, max = max_val, value = quantile(piecewise_sev_data(), (i/(input$num_pareto_slices+1))^0.1))
    })
    do.call(fluidRow, slider_list)
  })

  # Render the dynamic sliders
  output$pareto_slider_inputs <- renderUI({
    dynamic_sliders()
  })

  piecwise_pareto_mu <- reactive({
    mu_values <- c(min(piecewise_sev_data()), input$slider_1)
    if (input$num_pareto_slices>1) {
      for (i in 2:input$num_pareto_slices) {
        mu_values <- append(mu_values, input[[paste0('slider_', i)]])
      }
    }
    mu_values <- sort(mu_values)
    return(mu_values)
  })

  piecwise_pareto_alpha <- reactive({
    Pareto::PiecewisePareto_ML_Estimator_Alpha(piecewise_sev_data(), piecwise_pareto_mu())
  })

  output$fitted_sliced_pareto <- renderTable({
    data.frame(
      mu = piecwise_pareto_mu()
      ,alpha = piecwise_pareto_alpha()
    )
  })


  empirical_piecwise_cdf <- reactive({1:length(piecewise_sev_data())/(length(piecewise_sev_data())+1)})
  predicted_piecwise_cdf <- reactive({Pareto::pPiecewisePareto(piecewise_sev_data(), piecwise_pareto_mu(), piecwise_pareto_alpha())})
  output$piecewise_pareto_ks_test <- renderText({paste(
    'k-s test:'
    ,round(max(abs(empirical_piecwise_cdf() - predicted_piecwise_cdf())),4)
  )})

  output$piewcewise_pareto_cdf_plot <- renderPlotly({
    x_axis <- piecewise_sev_data()
    if (input$piecewise_pareto_fit_log_scale) {x_axis <- log(x_axis)}
    p <- plot_ly()
    p <- add_lines(p, x = ~x_axis, y = ~empirical_piecwise_cdf(), name = "Actual", line = list(color = "blue"))
    p <- add_lines(p, x = ~x_axis, y = ~predicted_piecwise_cdf(), name = "Predicted", line = list(color = "red"))
    p <- layout(p, title = "CDF Empirical vs Fitted", xaxis = list(title = "Claims amount"), yaxis = list(title = "CDF"))
    return(p)
  })

}

#Set Parameters
mu = 4
sigma = 1.5

#test pure IBNR duration
inceptionDate<-as.POSIXct("01/01/2007", format="%d/%m/%Y")

expiryDate<-as.POSIXct("31/12/2007", format="%d/%m/%Y")

ValuationDate<-as.POSIXct("30/10/2007", format="%d/%m/%Y")

PureIBNRLNorm(inceptionDate,expiryDate,ValuationDate,mu,sigma)

MaxRepDelay <- 0:as.numeric(ValuationDate-inceptionDate)

pureIBNRDays<-sum(plnorm(MaxRepDelay,mu,sigma,lower.tail = F))
pureIBNRDays

duration=as.numeric(expiryDate-inceptionDate)

UPRDays<-as.numeric(expiryDate-ValuationDate)
UPRDays

UPRDays/duration

pureIBNRDays/duration

#settings
options(shiny.maxRequestSize=64*1024^3)

#' A function to run the glm fitting tool application
#'
#' @return Opens the glm fitting tool application
#' @export
run_shiny_glm_fitting_tool = function(){
  shinyjs::useShinyjs()
  shinyApp(ui = GLMFittingToolUI, server = GLMFittingToolServer)
}

#' UI file for the Shiny NetSimR Simulator Tool
#'
#' @return Returns the UI code for the shiny application.
shiny_simulator_ui = fluidPage(
  use_busy_spinner(spin = "fading-circle", position = "full-page"),
  headerPanel(div("NetSimR Claims & Reinsurance Simulator"), windowTitle = "NetSim Claim Simulation Tool"),

  tabsetPanel(
    tabPanel(
      "Welcome",
      icon = icon("house"),
      h3("Welcome to the NetSimR Simulator Tool"),
      p("This tool allows you to simulate insurance claims from various distributions and implement various reinsurance structures."),
      p("Created by Yiannis Parizas. For more information, please visit my ", a("LinkedIn profile", href = "https://www.linkedin.com/in/yiannisparizas/", target = "_blank"), "."),
      p("You can also reach out to me via email at ", a("yiannis.parizas@gmail.com", href = "mailto:yiannis.parizas@gmail.com"), "."),
      p("Please reach out if you have any feedback or encounter any bugs.")
    ),

    tabPanel(
      "Claims Simulator",
      icon = icon("dice"),
      br(),

      fluidRow(
        column(4,
               wellPanel(
                 h4(icon("gear"), " Simulation Settings"),
                 sliderInput('numberOfSimulations', 'Number of simulations',
                             min = 10*1000, max = 1000*1000, value = 10*1000, step = 10*1000),
                 checkboxInput('seedSetBinary', 'Custom Seed', value = FALSE),
                 uiOutput("seed_value"),
                 checkboxInput('multiprocessingBinary', 'Multiprocessing (faster for large runs)', value = FALSE)
               )
        ),
        column(4,
               wellPanel(
                 h4(icon("chart-bar"), " Frequency Distribution"),
                 helpText("Models how many claims occur per simulated period."),
                 radioButtons(
                   inputId = 'freqDistr',
                   label = NULL,
                   choiceNames = unname(sapply(freq_dist_options, function(x) x@distr_label)),
                   choiceValues = unname(sapply(freq_dist_options, function(x) x@distrID))
                 ),
                 lapply(1:max(sapply(freq_dist_options, function(x) length(x@paramIDs))),
                        function(i){ uiOutput(paste0('freq_param_', i)) })
               )
        ),
        column(4,
               wellPanel(
                 h4(icon("chart-line"), " Severity Distribution"),
                 helpText("Models the size of each individual claim."),
                 radioButtons(
                   inputId = 'sevDistr',
                   label = NULL,
                   choiceNames = unname(sapply(sev_dist_options, function(x) x@distr_label)),
                   choiceValues = unname(sapply(sev_dist_options, function(x) x@distrID))
                 ),
                 lapply(1:max(sapply(sev_dist_options, function(x) length(x@paramIDs))),
                        function(i){ uiOutput(paste0('sev_param_', i)) })
               )
        )
      ),

      fluidRow(
        column(4,
               wellPanel(
                 h4(icon("scissors"), " Tail Adjustments"),
                 checkboxInput('paretoSlice', 'Apply Pareto Slices', value = FALSE),
                 helpText("Splices a Pareto tail onto the severity distribution above chosen thresholds."),
                 uiOutput("pareto_slice_times"),
                 lapply(1:(max_number_of_pareto_slices*2),
                        function(i){ uiOutput(paste0('slice_pareto_param_', i)) }),
                 hr(),
                 checkboxInput('sevCapBinary', 'Severity Cap', value = FALSE),
                 helpText("Caps any single claim at a maximum amount."),
                 uiOutput("sev_cap_amount")
               )
        ),
        column(4,
               wellPanel(
                 h4(icon("shield-halved"), " Reinsurance - Each & Every Loss (EEL)"),
                 helpText("Applied to each individual claim before it is summed into the total."),
                 radioButtons('reinsuranceStructureEEL', NULL, choices = reinsurance_structures_options),
                 uiOutput("reinsuranceStructureDeductibleEEL"),
                 uiOutput("reinsuranceStructureLimitEEL"),
                 uiOutput("reinsuranceStructureLimitedReinstatements"),
                 uiOutput("reinsuranceStructureReinstatementLimit")
               )
        ),
        column(4,
               wellPanel(
                 h4(icon("umbrella"), " Reinsurance - Aggregate Layer (AL)"),
                 helpText("Applied to the total claims across all EEL-adjusted losses per simulation."),
                 radioButtons('reinsuranceStructureAL', NULL, choices = reinsurance_structures_options),
                 uiOutput("reinsuranceStructureDeductibleAL"),
                 uiOutput("reinsuranceStructureLimitAL")
               )
        )
      ),

      hr(),

      fluidRow(align = "center",
               div(style = "display:inline-block; margin: 5px;",
                   actionButton("RunSimulations", "Run Simulations", icon = icon("play"), class = "btn-primary")),
               div(style = "display:inline-block; margin: 5px;", uiOutput("downloadDataButton")),
               div(style = "display:inline-block; margin: 5px;", uiOutput("downloadReportButton"))
      )
    )
  )
)

#' Server function for the GLM Fitting tool application
#'
#' @param input Input for the server function.
#' @param output Output for the server function.
#' @param session Session for the server function.
#' @return Returns server rendering for the shiny application.
#' @import shiny
#' @import DBI
#' @importFrom RMySQL MySQL
#' @importFrom RSQLite SQLite
#' @import RODBC
#' @import RPostgreSQL
#' @importFrom shinyjs useShinyjs
#' @importFrom plotly plot_ly add_lines layout add_bars renderPlotly
#' @import shinybusy
#' @import reactable
#' @importFrom plotly plotlyOutput
#' @importFrom bslib page_navbar nav_panel layout_columns card card_body card_header navbar_options bs_theme font_google
GLMFittingToolServer = function(input, output, session) {

  #import data
  selected_data <- eventReactive(input$submit, {
    show_spinner()
    result <- tryCatch({
      if (input$data_source == "Database") {
        # need to test all connectors work
        if (input$db_type == "MySQL") {
          port <- if (input$db_port == "") 3306 else as.numeric(input$db_port)
          con <- dbConnect(RMySQL::MySQL(),
                           host = input$db_host,
                           user = input$db_user,
                           password = input$db_password,
                           port = port,
                           dbname = input$db_name)
          # Execute the query
          df_input <- dbGetQuery(con, input$sql_query)
          # Close the connection
          dbDisconnect(con)

        } else if (input$db_type == "SQLite") {
          con <- dbConnect(RSQLite::SQLite(),
                           dbname = input$db_name)
          # Execute the query
          df_input <- dbGetQuery(con, input$sql_query)
          # Close the connection
          dbDisconnect(con)

        } else if (input$db_type == "SQL Server") {
          if (isTRUE(input$windows_auth)) {
            # Use Windows authentication
            con <- odbcDriverConnect(
              connection = paste0(
                "Driver={SQL Server};Server=", input$db_host,
                ";Database=", input$db_name,
                ";Trusted_Connection=yes"
              )
            )
          } else {
            # Ask for username and password
            con <- odbcDriverConnect(
              connection = paste0(
                "Driver={SQL Server};Server=", input$db_host,
                ";Database=", input$db_name,
                ";Uid=", input$db_user,
                ";Pwd=", input$db_password
              )
            )
          }
          # Execute the query
          df_input <- sqlQuery(con, input$sql_query)
          # Close the connection
          odbcClose(con)

        } else if (input$db_type == "PostgreSQL") {
          port <- if (input$db_port == "") 5432 else as.numeric(input$db_port)
          con <- dbConnect(
            RPostgreSQL::PostgreSQL(),
            host = input$db_host,
            port = port,
            dbname = input$db_name,
            user = input$db_user,
            password = input$db_password
          )
          # Execute the query
          df_input <- dbGetQuery(con, input$sql_query)
          # Close the connection
          dbDisconnect(con)
        }

        #return data
        hide_spinner()
        return(df_input)

      } else if (input$data_source == "CSV File") {

        # Read the uploaded CSV file
        file <- input$csv_file
        if (is.null(file)) {
          hide_spinner()
          return(NULL)
        }
        df <- read.csv(file$datapath)
        hide_spinner()
        return(df)
      }
    }, error = function(e) {
      hide_spinner()
      showModal(modalDialog(
        title = "Error",
        "An error occurred while importing data. Please check your settings and try again.",
        footer = NULL
      ))
      return(NULL)
    })
    return(result)
  })

  # Display the query result or uploaded data in a data table
  output$selected_input_data_table <- reactable::renderReactable({
    reactable::reactable(selected_data())
  })

  #save configurtations
  output$DownloadDataHandlerConf <- downloadHandler(
    filename = function() {paste0('app_config','.rds')}
    ,content = function(file) {
      saveRDS(reactiveValuesToList(input), file)
    }
  )
  output$downloadConfButton <- renderUI({
    downloadButton('DownloadDataHandlerConf', 'Save data')
  })

  #load configurations
  observeEvent(input$load_config, {
    input_config <- readRDS(input$load_config$datapath)
    for (x in names(input_config)) {
      if (is.null(input_config[[x]])) {
      } else {
        updateSelectInput(session, inputId = x, selected = input_config[[x]])
      }
    }
  })


  #modelling
  observe({
    data_columns <- names(selected_data())
    updateSelectInput(session, "response_variable", choices = data_columns)
    updateSelectInput(session, "offset", choices = c(c("None"), data_columns))
    updateSelectInput(session, "weights", choices = c(c("None"), data_columns))
    updateSelectInput(session, "visualize_variable", choices = c(c("None"), data_columns))
  })

  fitted_model <- eventReactive(input$fit_model, {
    show_spinner()
    tryCatch({
      offset_formula <- if (input$offset != "None") paste("offset(", input$offset, ") +", sep = "")
      formula_str <- paste(input$response_variable, "~", offset_formula, input$formula)
      glm_model <- glm(
        as.formula(formula_str)
        ,data = selected_data()
        #not working need to fix
        ,weights = if (input$weights != "None") eval(parse(text = input$weights)) else NULL
        ,family = eval(parse(text = paste0(input$glm_distribution, '(link = ', input$link_function,')')))
      )
      hide_spinner()
      return(glm_model)
    }, error = function(e) {
      message("Model fitting failed:", e)
      hide_spinner()
      return(NULL)
    })
  })

  # Store the formula text
  formula_text_1 <- reactiveVal()
  formula_text_2 <- reactiveVal()

  # Save the formula when the "Save Formula" button is clicked
  observeEvent(input$save_formula_1, {
    formula_text_1(input$formula)
  })
  observeEvent(input$save_formula_2, {
    formula_text_2(input$formula)
  })

  # Load the saved formula when the "Load Formula" button is clicked
  observeEvent(input$load_formula_1, {
    if (!is.null(formula_text_1())) {
      updateTextAreaInput(session, "formula", value = formula_text_1())
    }
  })
  observeEvent(input$load_formula_2, {
    if (!is.null(formula_text_2())) {
      updateTextAreaInput(session, "formula", value = formula_text_2())
    }
  })

  # Calculate AIC and display it
  aic_output_1 <- eventReactive(input$save_formula_1, {
    if (!is.null(fitted_model())) {
      aic <- AIC(fitted_model())
      return(paste("AIC formula 1:", round(aic, 2)))
    }
  })

  output$aic_output_1 <- renderText({
    aic_output_1()
  })

  aic_output_2 <- eventReactive(input$save_formula_2, {
    if (!is.null(fitted_model())) {
      aic <- AIC(fitted_model())
      return(paste("AIC formula 2:", round(aic, 2)))
    }
  })

  output$aic_output_2 <- renderText({
    aic_output_2()
  })


  #print the model summary
  output$model_summary <- renderPrint({
    if (is.null(fitted_model())) {
      "Model fitting failed. Please check your formula and dataset."
    } else {
      summary(fitted_model())
    }
  })

  #download the model summary
  output$download_summary <- downloadHandler(
    filename = function() {
      paste("glm_summary.txt", sep = "")
    },
    content = function(file) {
      writeLines(capture.output(print(summary(fitted_model()))), file)
    }
  )

  # Download the GLM model object as an RDS file
  #consider adding option to remove data
  output$download_model <- downloadHandler(
    filename = function() {
      paste("glm_model.rds", sep = "")
    },
    content = function(file) {
      saveRDS(fitted_model(), file)
    }
  )

  # Download handler for the CSV file
  output$download_data_with_predictions <- downloadHandler(
    filename = function() {
      "predicted_data.csv"
    },
    content = function(file) {
      write.csv(
        cbind(
          selected_data()
          ,prediction=predict(fitted_model(), selected_data(), type="response")
        )
        , file, row.names = FALSE
      )
    }
  )

  #visualisation
  fitness_plot <- eventReactive(input$execute_visualization, {
    show_spinner()
    if (input$visualize_variable == "None") {
      hide_spinner()
      return(NULL)
    }
    #create temporary dataframe
    plot_data <- data.frame(
      Actual = selected_data()[,input$response_variable],
      Predicted = predict(object = fitted_model(), selected_data(), type="response"),
      Visualized = selected_data()[,input$visualize_variable],
      Exposure = 1
    )
    if (input$offset != "None") {
      plot_data$Exposure <- selected_data()[,input$offset]
    }
    #band the visualisation variable
    if (is.numeric(plot_data$Visualized)) {
      if (length(unique(plot_data$Visualized))>input$number_of_bands_input){
        plot_data$Visualized <- cut(plot_data$Visualized, breaks = input$number_of_bands_input)
      }
    }
    # aggregate the dataframe
    plot_data <- aggregate(plot_data[,c('Actual', 'Predicted', 'Exposure')], by=list(plot_data$Visualized), sum)
    plot_data$Actual <- plot_data$Actual/plot_data$Exposure
    plot_data$Predicted <- plot_data$Predicted/plot_data$Exposure

    p <- plot_ly(data = plot_data, x = ~Group.1)
    p <- add_lines(p, y = ~Actual, name = "Actual", type = "scatter", mode = "lines")
    p <- add_lines(p, y = ~Predicted, name = "Predicted", type = "scatter", mode = "lines")

    # Assuming you want counts on the second y-axis
    p <- layout(p, yaxis2 = list(overlaying = "y", side = "right"))
    p <- add_bars(p, y = ~Exposure, name = "Exposure", yaxis = "y2", opacity = 0.5)

    #set axis names
    p <- layout(p, xaxis = list(title = input$visualize_variable))
    p <- layout(p, yaxis = list(title = input$response_variable))
    p <- layout(p, yaxis2 = list(title = 'Expopsure'))
    p <- layout(p, title = "Comparison of Actual and Predicted Values")

    hide_spinner()
    return(p)
  })

  # Render the plot in the main panel
  output$fitness_plot <- renderPlotly({
    fitness_plot()
  })

}

#set seed and number of simulations
set.seed(1)
n=1000*1000


#set gamma parameters
cap=10000
shape=1.5
rate=0.0004


#Capped gamma mean
x<-rgamma(n, shape, rate)
hist(x)
y<-ifelse(x<cap,x,cap)
mean(y)
hist(y)
GammaCappedMean(cap,shape,rate)


#exposure curve gamma
mean(y)/mean(x)
ExposureCurveGamma(cap,shape,rate)


#ILF Gamma
z<-ifelse(y>cap/2,cap/2,y)
mean(y)/mean(z)
ILFGamma(cap/2,cap,shape, rate)
ILFGamma(cap/2,c(cap,cap*1.1),shape, rate)

####################
#LogNormal Functions
####################


#' Error function
#'
#' @param x A real number.
#' @return The value of the error function at \code{x}.
#' @export
#' @examples
#' erf(0.1)
#' erf(0.5)
erf<-function(x){2*pnorm(sqrt(2)*x)-1}



#' Lognormal capped mean
#'
#' @param cap A positive real number -  the claim severity cap.
#' @param mu A real number - the first parameter of the Claim Severity's LogNormal distribution.
#' @param sigma A positive real number - the second parameter of the Claim Severity's LogNormal distribution.
#' @return The mean of the claim severity capped at \code{cap} with a LogNormal distribution with parameters \code{mu} and \code{sigma}.
#' @export
#' @examples
#' LNormCappedMean(2000,6,1.5)
#' LNormCappedMean(1000,5,1.6)
LNormCappedMean<- function(cap,mu,sigma){
  cap-(0.5*(exp(mu+0.5*sigma*sigma)*erf((mu+sigma*sigma-log(cap))/(sigma*sqrt(2)))+cap+cap*erf((log(cap)-mu)/(sqrt(2)*sigma)))-0.5*exp(mu+0.5*sigma*sigma))
}



#' Exposure Curve from LogNormal a severity distribution
#'
#' @param x A positive real number -  the claim amount where the exposure curve will be evaluated.
#' @param mu A real number - the first parameter of the Claim Severity's LogNormal distribution.
#' @param sigma A positive real number - the second parameter of the Claim Severity's LogNormal distribution.
#' @return The value of the Exposure curve at \code{x} with Claim Severity from a LogNormal distribution with parameters \code{mu} and \code{sigma}.
#' @export
#' @examples
#' ExposureCurveLNorm(2000,6,1.5)
#' ExposureCurveLNorm(1000,5,1.6)
ExposureCurveLNorm<-function(x,mu,sigma){
  LNormCappedMean(x,mu,sigma)/(exp(mu+0.5*sigma*sigma))
}



#' Increased Limit Factor Curve from a LogNormal severity distribution
#'
#' @param xLow A positive real number -  the claim amount where the Increased Limit Factor Curve will be evaluated from.
#' @param xHigh A positive real number -  the claim amount where the Increased Limit Factor Curve will be evaluated to.
#' @param mu A real number - the first parameter of the Claim Severity's LogNormal distribution.
#' @param sigma A positive real number - the second parameter of the Claim Severity's LogNormal distribution.
#' @return The value of the Increased Limit Factor curve from \code{xLow} to \code{xHigh} with Claim Severity from a LogNormal distribution with parameters \code{mu} and \code{sigma}.
#' @export
#' @examples
#' ILFLNorm(1000,2000,6,1.5)
#' ILFLNorm(1000,1500,5,1.6)
ILFLNorm<-function(xLow,xHigh,mu,sigma){
  LNormCappedMean(xHigh,mu,sigma)/LNormCappedMean(xLow,mu,sigma)
}



#' Pure IBNR exposure from a LogNormal reporting delay distribution
#'
#' @param IncDate A date -  the inception date of the period.
#' @param ExpDate A date -  the expiry date of the period. Must be greater than inception date.
#' @param ValDate A date -  the valuation date.
#' @param mu A real number - the first parameter of the reporing delay's LogNormal distribution.
#' @param sigma A positive real number - the second parameter of the reporing delay's LogNormal distribution.
#' @return Unearned and Pure IBNR exposure in days and as a percentage of the period's duration, where the reporting delay has a LogNormal distribution with parameters \code{mu} and \code{sigma}.
#' @export
#' @examples
#' Dates = data.frame(
#'     inceptionDate = c("01/01/2006", "01/07/2006", "01/01/2007")
#'     ,expiryDate = c("31/12/2006", "30/06/2007", "31/12/2007")
#' )
#'
#' Dates$inceptionDate<-as.POSIXct(Dates$inceptionDate, format="%d/%m/%Y")
#'
#' Dates$expiryDate<-as.POSIXct(Dates$expiryDate, format="%d/%m/%Y")
#'
#' ValuationDate<-as.POSIXct("30/10/2007", format="%d/%m/%Y")
#'
#' PureIBNRLNorm(Dates$inceptionDate,Dates$expiryDate,ValuationDate,4,1.5)
PureIBNRLNorm <- function(IncDate, ExpDate, ValDate, mu, sigma){
  MinRepDelay<-ifelse(ValDate<ExpDate,0,difftime(ValDate, ExpDate, units="days"))
  MaxRepDelay<-ifelse(ValDate<IncDate,0,difftime(ValDate, IncDate, units="days"))
  Duration<-as.numeric(difftime(ExpDate,IncDate, units="days"))
  EarnedDuration<-MaxRepDelay-MinRepDelay
  UnearnedDuration<-Duration-EarnedDuration
  UnearnedDurationRatio<-ifelse(Duration==0,0,round(UnearnedDuration/Duration,5))
  PureIBNRDuration<-round(LNormCappedMean(MaxRepDelay,mu,sigma)-LNormCappedMean(MinRepDelay,mu,sigma),2)
  PureIBNRDurationRatio<-ifelse(Duration==0,0,round(PureIBNRDuration/Duration,5))
  data.frame(UnearnedDuration,PureIBNRDuration,UnearnedDurationRatio,PureIBNRDurationRatio)
}

---
title: "Capped mean, Exposure and Increased Limit Factor curves"
author: "Yiannis Parizas"
date: '`r format(Sys.Date(), "%d-%m-%Y")`'
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{CappedMean}
  %\VignetteEngine{knitr::rmarkdown}
  \usepackage[utf8]{inputenc}
---


```{r setup, include=FALSE}
knitr::opts_chunk$set(echo = TRUE)
```


## 1.Introduction


This vignette will cover capped mean functions, exposure and increased limit factor curves of the NetSimR package. It will overview the theory behind these functions, propose posible uses and illustrate examples. The distributions covered for claim severities are LogNormal, Gamma, Pareto, Sliced LogNormal-Pareto, Sliced Gamma-Pareto. An application of the capped mean is suggested in the article "Taken to Excess" accessible [here](https://www.theactuary.com/features/2019/03/2019/03/06/taken-excess).



\pagebreak
## 2.Backround theory


Let claim severity $x$ originate from a probability density function $f(x)$ and a cumulative density function $F(x)$.
Let claims be capped at amount $c$.
The capped claim severity is defined as $y=Min(x,c)$.

### 2.1 Capped mean
The expected mean can be estimated by: $$E(x)=\int_0^{\infty}x*f(x)~dx$$
The expected capped claim can be obtained by: $$E(y)=\int_0^{\infty}y*f(x)~dx$$
The above simplifies to: $$E(y)=c-\int_0^{c}F(x)~dx$$
This equation will need to be solved for every distribution.

### 2.2 Exposure curve
This function provides the percentage of total claims cost that lies below a certain claims level.
Exposure curve at a claims cap is calculated by: $$EC(c)=\frac{E(y)}{E(x)}$$

### 2.3 Increased limit factor curve
This function provides the ratio of expected capped claims to a higher cap $c_{h}$ to the expected claims of a lower cap $c_{l}$.
Let the claim capped at $c_{h}$ be $y_{h}$ and the claim capped at $c_{h}$ be $y_{h}$.
Increased limit factor curve at a claims caps $c_{h}$ and $c_{l}$ is calculated by: $$ILF(c_{h},c_{l})=\frac{E(y_{h})}{E(y_{l})}$$



\pagebreak
## 3. Functions usage


In this section we will suggest possible uses of the specified functions.


### 3.1 Capped mean
- Restricting expected claims from being modelled beyond the property's value, in property lines.
- Modelling attritional losses with a censored or truncated distribution, in order to improve our estimate. Large claims will be modelled separately. This can be extended to Generalised Linear Models.
- Estimating the cost of deductibles.
- Pricing reinsurance excess of loss layers.


### 3.2 Exposure curve
For property lines/sections:

- Pricing/allocating premium of exess of loss layers.
- Loading for large losses.

Note: for property lines, instead of severity, we may opt to model severity as a percentage of sum insured. In GLMs this is equivalent to having the sum insured as the exposure for the severity distribution.

### 3.3 Increased limit factor curve
For casualty lines/sections:

- Pricing/allocating premium of exess of loss layers.
- Loading for large losses.


\pagebreak
## 4. Examples


### 4.1 Capped mean
The below code ilustrates an alternative and more accurate approach to applying capping in regressions, for modelling attritional claims part.
The code steps are:

- Simulate 10,000 claims
- Cap the claims
- Fit a lognormal regression to the capped claims
- Fit a censored regression to the capped claims
- Compare:
  - the expected mean from the normal regression
  - the expected capped mean from the censored regression
  - the mean of the capped claims data

The example suggests that the proposed method outperforms the market approach on LogNormal distribution. The one-way benefit will not be significant on a Gamma distribution, but when we move to a GLM and the model relativities are larger, the capped mean benefit may also be significant.

```{r capped mean example, eval=requireNamespace("crch", quietly = TRUE)}
library("NetSimR")

#Set parameters
n<-10000
mu<-6
sigma<-1.7
Cap <- 10000

#Set seed to keep simulations constant
set.seed(10)

#Simulate data
x<-round(rlnorm(n,mu,sigma),0)
head(x)
summary(x)
plot(x)
hist(x, breaks = 400, xlim = c(0,10000))


#cap data
z<-ifelse(x>Cap,Cap,x)
head(z)
summary(z)
plot(z)
hist(z, breaks = 200)


#fit linear regression
lmLinear<-lm(log(z)~1)
summary(lmLinear)


#fit right censored regression
lmCensored<-crch::crch(log(z)~1, right = log(Cap), link.scale="identity", dist = "gaussian")
summary(lmCensored)


#Compare regression outputs
cbind(coefficients(lmLinear),summary(lmLinear)$sigma)
coefficients(lmCensored)


#Compare regressions' attritional capped cost to empirical
round(cbind(
  Empirical=mean(z)
  ,LinearModel=exp(coefficients(lmLinear)+0.5*summary(lmLinear)$sigma*summary(lmLinear)$sigma)
  ,CensoredModel=LNormCappedMean(Cap,coefficients(lmCensored)[1],coefficients(lmCensored)[2])
),0)
```

Modelling excess levels would be similar to the above approach. In that case the expected mean above the excess levels would be the mean less the expected mean below the excess level. This is elaborated in the article cited in the introduction section.


### 4.2 Exposure curve
For a LogNormal severity, generating the exposure curve value at a particular point is obtained by running the below code:
```{r Exposure curve LogNormal}
ExposureCurveLNorm(1000,5,1.6)
```
This would be interpreted as: 56% of the expected cost is up to 1,000 currency units.
Other distributions follow the same logic.
The challenge for this exercise would be fitting the correct severity distribution, rather than calculating the Exposure curve's value.

### 4.3 Increased limit factor curve
For a LogNormal severity, generating the increased limit factor value between two particular points is obtained by running the below code:
```{r ILF curve LogNormal}
ILFLNorm(1000,1500,5,1.6)
```
This would be interpreted as: the expected cost of the claims up to 1,500 currency units is 1.155 times the expected cost is up to 1,000 currency units.
Other distributions follow the same logic.
The challenge for this exercise would be fitting the severity distribution, rather than calculating the Increased Limit Factor curve's value.

---
title: "Simulation report"
output: html_document
params:
  numOfSimulations: NULL
  freq_params: NULL
  sev_params: NULL
  seedSetBinary: NULL
  seedValue: NULL
  freqDistr: NULL
  sevDistr: NULL
  paretoSlice: NULL
  pareto_slice_times: NULL
  slice_pareto_alphas: NULL
  slice_pareto_x_ms: NULL
  sevCapBinary: NULL
  sev_cap_amount: NULL
  reinsuranceStructureEEL: NULL
  reinsurance_structure_eel_dedctible_amount: NULL
  reinsurance_structure_eel_limit_amount: NULL
  reinsuranceStructureAL: NULL
  reinsurance_structure_al_dedctible_amount: NULL
  reinsurance_structure_al_limit_amount: NULL
  multiprocessing: NULL
  reinsuranceStructureLimitedReinstatements: NULL
  reinsuranceStructureReinstatementLimit: NULL
  total_claims_data: NULL
---
### Report produced by NetSimR

## Simulation settings
```{r, echo=FALSE, comment="", null_prefix=TRUE}
writeLines(c(
  paste('Number Of Simulations:', formatC(params$numOfSimulations, format="d", big.mark=","))
  ,paste('Multiprocessing used:', params$multiprocessing)
  ,paste('Fixed Seed:', params$seedSetBinary)
  ,if(!is.null(params$seedValue)) {paste('Fixed Seed Value:', params$seedValue)}
  ,paste('Frequency Distribution:', params$freqDistr)
  ,paste('Frequency Parameters:', paste(params$freq_params, collapse = ', '))
  ,paste('Severity Distribution:', params$sevDistr)
  ,paste('Severity Parameters:', paste(params$sev_params, collapse = ', '))
  ,paste('Pareto Slicing Applied:', params$paretoSlice)
  ,if(params$paretoSlice) {paste('Pareto Slice times:', params$pareto_slice_times)}
  ,if(params$paretoSlice) {paste('Pareto Slice alpha parameters:', params$slice_pareto_alphas)}
  ,if(params$paretoSlice) {paste('Pareto Slice x_m parameters:', params$slice_pareto_x_ms)}
  ,paste('Severity Cap Applied:', params$sevCapBinary)
  ,if(!is.null(params$sev_cap_amount)) {paste('Severity Cap Amount:', params$sev_cap_amount)}
  ,paste('Reinsurance Structure - EEL:', params$reinsuranceStructureEEL)
  ,if(!is.null(params$reinsurance_structure_eel_dedctible_amount)) {paste('Reinsurance Structure - EEL dedctible:',   params$reinsurance_structure_eel_dedctible_amount)}
  ,if(!is.null(params$reinsurance_structure_eel_limit_amount)) {paste('Reinsurance Structure - EEL limit:',   params$reinsurance_structure_eel_limit_amount)}
  ,if(!is.null(params$reinsuranceStructureLimitedReinstatements)) {paste('Reinsurance Structure - EEL Limited Reinstatements:',   params$reinsuranceStructureLimitedReinstatements)}
  ,if(!is.null(params$reinsuranceStructureReinstatementLimit)) {paste('Reinsurance Structure - EEL Number of Reinstatements Limit:',   params$reinsuranceStructureReinstatementLimit)}
  ,paste('Reinsurance Structure - AL:', params$reinsuranceStructureAL)
  ,if(!is.null(params$reinsurance_structure_al_dedctible_amount)) {paste('Reinsurance Structure - AL dedctible:',   params$reinsurance_structure_al_dedctible_amount)}
  ,if(!is.null(params$reinsurance_structure_al_limit_amount)) {paste('Reinsurance Structure - AL limit:',   params$reinsurance_structure_al_limit_amount)}
))
```


## Summary of Total Claims
```{r, echo=FALSE, comment="", null_prefix=TRUE}
noquote(formatC(summary(params$total_claims_data), format="d", big.mark=","))
```

## Table summarising percentiles of Total Claims
```{r, echo=FALSE, comment="", null_prefix=TRUE}
percentile_values <- c(seq(0.1,0.7,0.1), seq(0.75, 0.95, 0.05), 0.975, 0.99, 0.995)
print(data.frame(
  Percentile = paste0(percentile_values*100,"%")
  ,Claims = formatC(round(quantile(params$total_claims_data, probs = percentile_values),0), format="d", big.mark=",")
), row.names = F)

```

## Histogram of Total Claims

```{r, echo=FALSE}
hist(params$total_claims_data, breaks=200, main="Histogram of Total Claims", xlab="Total Claims", ylab="Simulation Count", xaxt='n')
axis(1, pretty(params$total_claims_data), formatC(pretty(params$total_claims_data), format = "f", big.mark = ",", digits = 0))
```

## A box plot plot of Total Claims

```{r, echo=FALSE}
boxplot(params$total_claims_data, main = "Box-Plot of Total Claims", ylab = "Total Claim Amount", col = "blue", yaxt = "n")
axis(2, pretty(params$total_claims_data), formatC(pretty(params$total_claims_data), format = "f", big.mark = ",", digits = 0))
```

## Cumulative distribution function (CDF) plot of Total Claims

```{r, echo=FALSE}
plot(ecdf(params$total_claims_data), xlab = "Total Claims", ylab = "Cumulative Density", main = "Cumulative Density Function of Total Claims", xaxt = "n")
axis(1, pretty(params$total_claims_data), formatC(pretty(params$total_claims_data), format = "f", big.mark = ",", digits = 0))
```

---
title: "Simulation Report"
output:
  html_document:
    toc: true
    toc_depth: 2
    theme: readable
    df_print: paged
params:
  numOfSimulations: NULL
  freq_params: NULL
  sev_params: NULL
  seedSetBinary: NULL
  seedValue: NULL
  freqDistr: NULL
  sevDistr: NULL
  paretoSlice: NULL
  pareto_slice_times: NULL
  slice_pareto_alphas: NULL
  slice_pareto_x_ms: NULL
  sevCapBinary: NULL
  sev_cap_amount: NULL
  reinsuranceStructureEEL: NULL
  reinsurance_structure_eel_dedctible_amount: NULL
  reinsurance_structure_eel_limit_amount: NULL
  reinsuranceStructureAL: NULL
  reinsurance_structure_al_dedctible_amount: NULL
  reinsurance_structure_al_limit_amount: NULL
  multiprocessing: NULL
  reinsuranceStructureLimitedReinstatements: NULL
  reinsuranceStructureReinstatementLimit: NULL
  total_claims_data: NULL
---

# Simulation Report

This report summarizes the simulation settings and the resulting distribution of total claims.

## Overview

```{r setup, include=FALSE}
knitr::opts_chunk$set(echo = FALSE, message = FALSE, warning = FALSE)

fmt_num <- function(x) formatC(x, format = "f", digits = 2, big.mark = ",")
fmt_int <- function(x) formatC(round(x, 0), format = "d", big.mark = ",")
fmt_pct <- function(x) paste0(fmt_num(100 * x), "%")
claims <- params$total_claims_data
```

```{r overview-table}
overview <- data.frame(
  Metric = c(
    "Number of simulations",
    "Multiprocessing used",
    "Fixed seed",
    "Seed value"
  ),
  Value = c(
    fmt_int(params$numOfSimulations),
    ifelse(isTRUE(params$multiprocessing), "Yes", "No"),
    ifelse(isTRUE(params$seedSetBinary), "Yes", "No"),
    ifelse(is.null(params$seedValue), "—", as.character(params$seedValue))
  )
)

knitr::kable(overview, align = c("l", "l"))
```

## Model settings

```{r settings-table}
settings <- data.frame(
  Section = c(
    rep("Frequency", 2),
    rep("Severity", 2),
    rep("Pareto slicing", 3),
    rep("Severity cap", 2),
    rep("EEL reinsurance", 3),
    rep("AL reinsurance", 3)
  ),
  Metric = c(
    "Distribution", "Parameters",
    "Distribution", "Parameters",
    "Applied", "Slice count", "Slice parameters",
    "Applied", "Cap amount",
    "Structure", "Deductible", "Limit",
    "Structure", "Deductible", "Limit"
  ),
  Value = c(
    params$freqDistr,
    paste(params$freq_params, collapse = ", "),
    params$sevDistr,
    paste(params$sev_params, collapse = ", "),
    ifelse(isTRUE(params$paretoSlice), "Yes", "No"),
    ifelse(is.null(params$pareto_slice_times), "—", as.character(params$pareto_slice_times)),
    ifelse(isTRUE(params$paretoSlice),
           paste(
             "alpha:", paste(params$slice_pareto_alphas, collapse = ", "),
             "; x_m:", paste(params$slice_pareto_x_ms, collapse = ", ")
           ),
           "—"),
    ifelse(isTRUE(params$sevCapBinary), "Yes", "No"),
    ifelse(is.null(params$sev_cap_amount), "—", fmt_num(params$sev_cap_amount)),
    params$reinsuranceStructureEEL,
    ifelse(is.null(params$reinsurance_structure_eel_dedctible_amount), "—", fmt_num(params$reinsurance_structure_eel_dedctible_amount)),
    ifelse(is.null(params$reinsurance_structure_eel_limit_amount), "—", fmt_num(params$reinsurance_structure_eel_limit_amount)),
    params$reinsuranceStructureAL,
    ifelse(is.null(params$reinsurance_structure_al_dedctible_amount), "—", fmt_num(params$reinsurance_structure_al_dedctible_amount)),
    ifelse(is.null(params$reinsurance_structure_al_limit_amount), "—", fmt_num(params$reinsurance_structure_al_limit_amount))
  )
)

knitr::kable(settings, align = c("l", "l", "l"))
```

## Summary statistics

```{r summary-stats}
summary_stats <- data.frame(
  Metric = c(
    "Count",
    "Mean",
    "Median",
    "Standard deviation",
    "Minimum",
    "Maximum",
    "Coefficient of variation"
  ),
  Value = c(
    fmt_int(length(claims)),
    fmt_num(mean(claims, na.rm = TRUE)),
    fmt_num(median(claims, na.rm = TRUE)),
    fmt_num(sd(claims, na.rm = TRUE)),
    fmt_num(min(claims, na.rm = TRUE)),
    fmt_num(max(claims, na.rm = TRUE)),
    fmt_num(sd(claims, na.rm = TRUE) / mean(claims, na.rm = TRUE))
  )
)

knitr::kable(summary_stats, align = c("l", "r"))
```

## Percentiles and tail risk

```{r tail-table}
probs <- c(0.5, 0.75, 0.9, 0.95, 0.975, 0.99, 0.995)
qv <- quantile(claims, probs = probs, na.rm = TRUE)

percentile_df <- data.frame(
  Percentile = fmt_pct(probs),
  Claims = fmt_num(as.numeric(qv))
)

knitr::kable(percentile_df, align = c("l", "r"))
```

```{r tail-risk-table}
var95 <- unname(quantile(claims, 0.95, na.rm = TRUE))
var99 <- unname(quantile(claims, 0.99, na.rm = TRUE))
var995 <- unname(quantile(claims, 0.995, na.rm = TRUE))

tail_df <- data.frame(
  Metric = c(
    paste0("VaR ", fmt_pct(0.95)),
    paste0("TVaR ", fmt_pct(0.95)),
    paste0("VaR ", fmt_pct(0.99)),
    paste0("TVaR ", fmt_pct(0.99)),
    paste0("VaR ", fmt_pct(0.995)),
    paste0("TVaR ", fmt_pct(0.995))
  ),
  Value = c(
    fmt_num(var95),
    fmt_num(mean(claims[claims >= var95], na.rm = TRUE)),
    fmt_num(var99),
    fmt_num(mean(claims[claims >= var99], na.rm = TRUE)),
    fmt_num(var995),
    fmt_num(mean(claims[claims >= var995], na.rm = TRUE))
  )
)

knitr::kable(tail_df, align = c("l", "r"))
```

## Visual diagnostics

### Histogram

```{r hist-plot}
hist(
  claims,
  breaks = 200,
  main = "Histogram of Total Claims",
  xlab = "Total Claims",
  ylab = "Frequency",
  col = "steelblue",
  border = "white"
)
```

### Box plot

```{r boxplot-plot}
boxplot(
  claims,
  main = "Box Plot of Total Claims",
  ylab = "Total Claims",
  col = "lightsteelblue",
  border = "gray40"
)
```

### Empirical CDF

```{r ecdf-plot}
plot(
  ecdf(claims),
  xlab = "Total Claims",
  ylab = "Cumulative Probability",
  main = "Empirical CDF of Total Claims",
  verticals = TRUE,
  do.points = FALSE
)
```

## Notes

- High skewness and a large gap between the median and tail percentiles usually indicate heavy-tail behavior.
- VaR and TVaR are often the most useful measures for insurance and reinsurance interpretation.
- If the histogram is very concentrated near zero with a long right tail, consider adding a log-scale histogram as an optional view.

---
title: "PureIBNR"
author: "Yiannis Parizas"
date: '`r format(Sys.Date(), "%d-%m-%Y")`'
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{PureIBNR}
  %\VignetteEngine{knitr::rmarkdown}
  \usepackage[utf8]{inputenc}
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(echo = TRUE)
```

## 1.Introduction


This vignette will cover pure IBNR functions of the NetSimR package. It will overview the theory behind these functions, propose possible uses and illustrate examples. The reporting delay distributions supported are LogNormal and Gamma. An application of the pure IBNR functions is suggested in the article "Escaping the triangle" accessible [here](https://www.theactuary.com/features/2019/06/2019/06/05/escaping-triangle).



\pagebreak
## 2.Backround theory


### 2.1 Reporting delay distribution
Reporting delay is the time difference between when the accident occured and when the insurer was notified.
Reporting delay is a positive and positively skewed variable.
It can me modelled with a LogNormal or Gamma regression.
As suggested by the article cited in the introduction, estimating reserves can be more accurate if the reserves are split to pure IBNR and IBNER.
Also note tha reporting delay is a right truncated variable. The closer the exposure period to the valuation date, the more the truncation. As such reporting delay would be better modelled by a right truncated regression.

### 2.2 From reporting delay to pure IBNR
Pure IBNR refers to claims not reported yet to the insurer, while IBNER refers to the over/under estimation in the outstanding part of reported claims. Together the two form the insurer's reserves for claims occured. Unearned premium will also be calculated by the functions provided, and this relates to future claims from business already written.
If we split the period to days, one can think of the reporting delay cumulative distribution function as the proportion of the day earned net of pure IBNR, based on the maximum reporting delay (Maximum reporting delay would be the difference between the reference day and the valuation day).
As integration is equivalent to summing the function values, when the partition size is one, one can derive that integrating from the maximum to the minimum reporting delay calculates the pure IBNR earned exposure for the period. This can be proportioned with the full duration and the earned duration, to provide different metrics.



\pagebreak
## 3. Functions usage


In this section we will suggest possible uses of the specified functions.


### 3.1 Pure IBNR
The function will input the inception, expiry, valuation dates and estimate the proportions unearned and allocated to pure IBNR.
The pure IBNR proportion multiplied by the frequency-severity from exposure rating, can provide an accurate estimate of the reserve.
Alternatively mutliplying this with the premium and a loss ratio proxy will provide an acceptable approximation of the reserve.
The proportion of the unearned exposure multiplied by the premium will provide the unearned premium reserve.
One can use the exposure net of unearned period and pure IBNR as the offset (exposure) for modelling frequency in exposure or experience rating. This will be the modellers choice rather than removing the last periods in situations where data is less.
The pure IBNR and unearned exposures can be used for simulating stochastic claim reserves.
Following the above, the analyst can incorporate the unearned and pure IBNR exposure models to dashboards, hence automate estimating the reserves.
For all the above this method is able to drill down individual segments and be insensitive to portfolio mix changes, when the reporting delay is modelled with generalised linear models.
When this method is combined with a truncated regression, the method can adjust for reporting delay development for the latest periods. The method can also reveal patterns otherwise invisible to the analyst.
Reinsurance structures are easier to be implemneted on top of this method compared to triangles.


\pagebreak
## 4. Examples


### 4.1 Pure IBNR

In the below example we will following these steps:

- Construct a policy dataset with policy inception/expiry dates
- Generate claims
- Generate accident and reporing dates, through a reporting delay disribution
- Derive reported claims up to the valuation date
- Model reporting delay
- Use the function of the NetSimR package to estimate PureIBNR duration
- Model claim frequency with adjusted claim duration and compare modelling without adjusting
- Predict claim frequency for pure IBNR and unearned duration and compare with our data
- Generate a chain ladder estimate for pure IBNR claims count and compare with the data and the GLM approach
- Simulate the pure IBNR reserve distribution

```{r pure IBNR example}
library("NetSimR")
#set the seed
set.seed(0)

#Set the reporting delay distribution parameters
mu <- 3
sigma <- 1

#Generate reporting delays and preview the distribution
x<-rlnorm(1000,mu,sigma)
summary(x)
hist(x, breaks = 100)
```

```{r pure IBNR example continued, eval=requireNamespace("crch", quietly = TRUE)}
#Generate dates data
PoliciesPerDay<-100
PeriodLength<-365*2
InceptionDateStart<-as.Date("2011/1/1")
InceptionDate<-seq(InceptionDateStart, by = "day", length.out = PeriodLength)
InceptionDate<-rep(InceptionDate,PoliciesPerDay)
#assume all policies are yearly
DayPolicyDuration<-365
ExpiryDate<-InceptionDate+DayPolicyDuration
ValuationDate<-max(InceptionDate)
Data<-data.frame(InceptionDate,ExpiryDate)
summary(Data)
ValuationDate

#Generate Claims Counts Data and reporting delays
#Assume only 1 claim per policy possible with probability of claiming 10%
ClaimFreq<-0.1
Data$ClaimCount<-ifelse(runif(nrow(Data))<ClaimFreq,1,0)
#generate accident date assuming uniformly happening throughout the year
Data$AccidentDate<-Data$InceptionDate+runif(nrow(Data))*(Data$ExpiryDate-Data$InceptionDate)
#generate reporting delays
Data$ReportingDelay<-round(Data$ClaimCount*rlnorm(nrow(Data),mu,sigma),1)
#generate reporting days
Data$NotificationDate<-Data$AccidentDate+Data$ReportingDelay
#remove claims reported beyond valuation day
Data$ClaimReportedBeforeValuation<-ifelse(Data$NotificationDate>ValuationDate,0,Data$ClaimCount)
#Maximum reporting delays
Data$MaxReportingDelay<-as.numeric(ValuationDate-Data$AccidentDate)+1

#Earned Duration
Data$EarnedDuration<-ifelse(Data$ExpiryDate>ValuationDate,as.numeric(ValuationDate-Data$InceptionDate),as.numeric(Data$ExpiryDate-Data$InceptionDate))/365

#Filter claims only data
ModelData<-Data[Data$ClaimReportedBeforeValuation==1,]

#model reporting delays with a linear model
lm<-lm(log(ModelData$ReportingDelay)~1)
#summary(lm)
cbind(coefficients(lm),summary(lm)$sigma)

#model reporting delays with a truncated model (splitting to monthly periods would be more accurate in terms of maximum reporting delay)
rTLm<-crch::crch(log(ModelData$ReportingDelay)~1, truncated = TRUE, right = log(ModelData$MaxReportingDelay), dist = "gaussian", link.scale = "identity")
#summary(rTLm)
coefficients(rTLm)

#Generate and predict pure IBNR and UPR Claims
Data$PureIBNRClaims<-ifelse(Data$AccidentDate>ValuationDate,0,Data$ClaimCount-Data$ClaimReportedBeforeValuation)
Data$UPRClaims<-Data$ClaimCount-Data$ClaimReportedBeforeValuation-Data$PureIBNR
PureIBNRPrediction<-PureIBNRLNorm(Data$InceptionDate,Data$ExpiryDate,ValuationDate,coefficients(rTLm)[1],coefficients(rTLm)[2])
Data$PredictedPureIBNRYears<-PureIBNRPrediction$PureIBNRDuration/DayPolicyDuration
Data$PredictedUPRYears<-PureIBNRPrediction$UnearnedDuration/DayPolicyDuration
Data$NetEarnedDuration<-1-Data$PredictedUPRYears-Data$PredictedPureIBNRYears+0.00001

#Predict Claim Frequency with reduced exposure to adjust for pure IBNR
AdjFreqGLM<-glm(Data$ClaimReportedBeforeValuation~1, offset = log(Data$NetEarnedDuration), family = "poisson")
#summary(AdjFreqGLM)
PredictedFrequency<-exp(AdjFreqGLM$coefficients)

#Compare as if not adjusting for pure IBNR
UnAdjFreqGLM<-glm(Data$ClaimReportedBeforeValuation~1, offset = log(Data$EarnedDuration+0.00001), family = "poisson")
#summary(UnAdjFreqGLM)

cbind(AdjustedModel=round(exp(AdjFreqGLM$coefficients),3),UnAdjustedModel=round(exp(UnAdjFreqGLM$coefficients),3),Actual=ClaimFreq)


#Compare UPR Claim predictions
cbind(Actual=sum(Data$UPRClaims),Predicted=round(sum(Data$PredictedUPRYears)*PredictedFrequency,0))

#Compare Pure IBNR Claim predictions
cbind(Actual=sum(Data$PureIBNRClaims),Predicted=round(sum(Data$PredictedPureIBNRYears)*PredictedFrequency,0), Theoretical=327)

#Theoretical comes from re-running with the theoretical parameter values

#Compare Pure IBNR predictions with Chain Ladder
ClaimsAY1RY1<-sum(Data[Data$AccidentDate<as.Date("2012/1/1") & Data$NotificationDate<as.Date("2012/1/1"), ]$ClaimReportedBeforeValuation)

ClaimsAY1RY2<-sum(Data[Data$AccidentDate<as.Date("2012/1/1") & Data$NotificationDate<as.Date("2013/1/1"), ]$ClaimReportedBeforeValuation)

ClaimsAY2RY1<-sum(Data$ClaimReportedBeforeValuation)-ClaimsAY1RY1-ClaimsAY1RY2

PredictedPureIBNRClaimsChainLadder<-(ClaimsAY1RY2/ClaimsAY1RY1-1)*ClaimsAY2RY1

cbind(Actual=sum(Data$PureIBNRClaims),PredictedGLM=round(sum(Data$PredictedPureIBNRYears)*PredictedFrequency,0),PredictedCL=round(PredictedPureIBNRClaimsChainLadder,0))

#Note: Chain ladder can predict better if periods brake down to months or quarters

#Sliced LogNormal-Pareto claim Severity assumption
mu=5.6
sigma=1.65
s=10000
alpha=1.5

#Simulate the reserve distribution
numberOfSimulations=10000
SimulatedYears <- numeric(length = numberOfSimulations)
ExpectedFrequency <- sum(Data$PredictedPureIBNRYears)*PredictedFrequency

for (i in 1 :numberOfSimulations){
  SimulatedYears[i]=round(sum(qSlicedLNormPareto(runif(rpois(1,ExpectedFrequency)),mu,sigma,s,alpha)),0)
}

#Visualising pure IBNR reserve distribution
head(SimulatedYears)
hist(SimulatedYears, breaks = 1000, xlim = c(0,1.5e6))
summary(SimulatedYears)

#Note: the strength of the method illustrated is that it can have GLM factors for reporting delay, frequency and severity, and run the simulations per risk

```

---
title: "SlicedDistribution"
author: "Yiannis Parizas"
date: '`r format(Sys.Date(), "%d-%m-%Y")`'
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{SlicedDistribution}
  %\VignetteEngine{knitr::rmarkdown}
  \usepackage[utf8]{inputenc}
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(echo = TRUE)
```

## 1.Introduction


This vignette will cover the sliced distributions available in the NetSimR package. It will overview the theory behind these functions, propose posible uses and illustrate examples. Currently, Sliced distributions available are LogNormal and Gamma with a Pareto distribution for the tail.


\pagebreak
## 2.Backround theory


### 2.1 Sliced distribution backround

At various insurance modelling exercises, it may be the case that a sliced distribution provides a significantly better fit than a non-sliced distribution.
Tail losses may be very influential on the premium, as such modelling the tail more accurately would significantly improve the accuacy of the premium. This becomes more important when modelling upper layers that will be mainly based on the severity tail.
The analyst may opt to model attritional losses using a regression and have a single dimensional Pareto covering all cases.
Theory suggests that Pareto distributions provide an appropriate fit for the tail of distributions and the mean excess function can be used to set the slice point.
The analyst may consider using a truncated or censored distribution fit for the attritional data, in order to improve the fit.

### 2.2 Sliced distribution definition

Let claim severity $x$ have a probability density function $f(x)$ and a cumulative density funtion $F(x)$. Let $s$ be the slice point.
We define $f(x)$ as:
$$f(x)= \left[\begin{array}
{rrr}
g(x) & x \le s \\
(1-G(s))*h(x) & x > s 
\end{array}\right]
$$
$g(x)$ is the attritional attritional probability density function, fitted to the full claims. As suggested this could be fitted to full claims as a censored distribution or to the claims below $s$ as a truncated distribution for a better fit. $g(x)$ in NetSimR package will either be Gamma or LogNormal.
$h(x)$ is the large probability density function. $h(x)$ in NetSimR package will be a single parameter Pareto distribution. The scale paramter $x_{m}$ will be the slice point $s$. The shape parameter will be estimated from the claims larger than the slice point.
$H(x)$ and $G(x)$ are the cumulative density functions of $h(x)$ and $g(x)$ respectively.

We define $F(x)$ as:
$$F(x)= \left[\begin{array}
{rrr}
G(x) & x \le s \\
G(s)+(1-G(s))*H(x) & x > s 
\end{array}\right]
$$
The mean $E(x)$ is defined to be:
$$E(x)=Capped Mean (g(x) @ s) + (1-G(s))*(Mean(h(x))-s)$$
Let claims be capped at amount $c$.
The capped claim severity is defined as $y=Min(x,c)$.The capped mean $E(y)$ is defined to be:
$$E(y)= \left[\begin{array}
{rrr}
Capped Mean (g(y)@c) & c \le s \\
Capped Mean (g(x) @ s)+(1-G(s))*(Capped Mean (h(y)@c)-s) & c > s 
\end{array}\right]
$$


\pagebreak
## 3. Functions usage


NetSimR package provides functions for the mean, capped mean, exposure curve and increased limit factor curves. These were discussed in the "Capped mean, Exposure and Increased Limit Factor curves" vignette and will not be repeated here. The package also provides the probability density function (pdf), the cumulative probability density function (cdf) and the inverse cumulative probability function for the sliced distributions.

The probability density function can be used for optimising parameters with the maximum likelihood method, when fitting such a distribution, in cases where fitting the distributions separately is not appropriate.

The cumulative density function provides probability of claims being less or more than a particular level. The distribution can be used alongst methods such as the Panjer recursion, in order to combine it with the frequency distribution and provide the aggrecate claims distribution.

The inverse cumulative probability distribution can be used to run simulations over the defined distribution. Reinsurance strucutures can then be applied to use the simulation exercise for reinsurance pricing.


\pagebreak
## 4. Examples


### 4.1 Mean of a sliced distribution
Here we calculate the expected severity from a sliced lognormal pareto distributions, sliced at 10,000 currency units, with a Pareto shape parameter $shape = 1.2$ and lognormal parameters $mu = 6$ and $sigma = 1.6$.
```{r sliced mean}
library("NetSimR")
round(SlicedLNormParetoMean(6,1.6,10000,1.2),0)
```
Multiplying the above with claim frequency would provide the expected claims cost. A capped mean of a sliced distribution follows the capped mean vignette examples.


### 4.2 Sliced distribution simulations
We will only demonstrate simulating from the inverse distribution, the probability density and cumulative probability functions' use is simpler.
Below we simulate from a sliced distribution, applying an each and every loss excess of loss reinsurance structure.
```{r sliced distribution simulation}
#set seed
set.seed(1)

#set parameters
numberOfSimulations=20000
freq=15
mu=6
sigma=1.6
s=1000
alpha=1.2
Deductible=10000
limit=100000
XoLCededClaims <- numeric(length = numberOfSimulations)

#loop simulates frequency then severity and applies layer deductible and limit
for (i in 1 :numberOfSimulations){
  x=qSlicedLNormPareto(runif(rpois(1,freq)),mu,sigma,s,alpha)
  x=ifelse(x<Deductible,0,ifelse(x-Deductible>limit,limit,x-Deductible))
  XoLCededClaims[i]=round(sum(x),0)
}

#Visualising Simulation results
head(XoLCededClaims)
hist(XoLCededClaims, breaks = 100, xlim = c(0,100000))
summary(XoLCededClaims)
  
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/LogNormal.R
\name{LNormCappedMean}
\alias{LNormCappedMean}
\title{Lognormal capped mean}
\usage{
LNormCappedMean(cap, mu, sigma)
}
\arguments{
\item{cap}{A positive real number -  the claim severity cap.}

\item{mu}{A real number - the first parameter of the Claim Severity's LogNormal distribution.}

\item{sigma}{A positive real number - the second parameter of the Claim Severity's LogNormal distribution.}
}
\value{
The mean of the claim severity capped at \code{cap} with a LogNormal distribution with parameters \code{mu} and \code{sigma}.
}
\description{
Lognormal capped mean
}
\examples{
LNormCappedMean(2000,6,1.5)
LNormCappedMean(1000,5,1.6)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/SlicecdLogNormalPareto.R
\name{dSlicedLNormPareto}
\alias{dSlicedLNormPareto}
\title{The probability density function (pdf) of a Sliced LogNormal Pareto severity distribution}
\usage{
dSlicedLNormPareto(x, mu, sigma, SlicePoint, shape)
}
\arguments{
\item{x}{A positive real number -  the claim amount where the probability density function (pdf) will be evaluated.}

\item{mu}{A real number -  the first parameter of the attritional Claim Severity's LogNormal distribution.}

\item{sigma}{A positive real number -  the second parameter of the attritional Claim Severity's LogNormal distribution.}

\item{SlicePoint}{A positive real number - the slice point and the scale parameter of the Claim Severity's Pareto distribution.}

\item{shape}{A positive real number - the shape parameter of the Claim Severity's Pareto distribution.}
}
\value{
The value of the probability density function (pdf) at \code{x} with an attritional claim LogNormal distribution with parameters \code{mu} and \code{sigma} and a large claim Pareto distribution with parameters \code{SlicePoint} and \code{shape}.
}
\description{
The probability density function (pdf) of a Sliced LogNormal Pareto severity distribution
}
\examples{
dSlicedLNormPareto(1200,6,1.5,1000,1.2)
dSlicedLNormPareto(4000,7,1.6,3000,1.4)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ShinySimulatorGlobal.R
\name{reinsurance_structures_options}
\alias{reinsurance_structures_options}
\title{A vector with the reinsurance structure options}
\usage{
reinsurance_structures_options
}
\value{
The reinsurance structure options
}
\description{
A vector with the reinsurance structure options
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GLMFittingToolUI.R
\name{GLMFittingToolUI}
\alias{GLMFittingToolUI}
\title{UI file for the Shiny GLM fitting tool}
\usage{
GLMFittingToolUI
}
\value{
Returns the UI code for the shiny application.
}
\description{
UI file for the Shiny GLM fitting tool
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/SlicecdGammaPareto.R
\name{SlicedGammaParetoCappedMean}
\alias{SlicedGammaParetoCappedMean}
\title{Sliced Gamma Pareto capped mean}
\usage{
SlicedGammaParetoCappedMean(cap, GShape, GRate, SlicePoint, PShape)
}
\arguments{
\item{cap}{A positive real number -  the claim severity cap.}

\item{GShape}{A positive real number -  the shape parameter of the attritional Claim Severity's Gamma distribution.}

\item{GRate}{A positive real number -  the rate parameter of the attritional Claim Severity's Gamma distribution.}

\item{SlicePoint}{A positive real number - the slice point and the scale parameter of the tail Claim Severity's Pareto distribution.}

\item{PShape}{A positive real number - the shape parameter of the tail Claim Severity's Pareto distribution.}
}
\value{
The mean of the claim severity capped at \code{cap} with an attritional claim Gamma distribution with parameters \code{GShape} and \code{GRate} and a large claim Pareto distribution with parameters \code{SlicePoint} and \code{PShape}.
}
\description{
Sliced Gamma Pareto capped mean
}
\examples{
SlicedGammaParetoCappedMean(3000,1,0.0005,1000,1.2)
SlicedGammaParetoCappedMean(1000,1.1,0.0006,2000,1.6)
SlicedGammaParetoCappedMean(2000,1.2,0.0004,3000,1.4)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Gamma.R
\name{ILFGamma}
\alias{ILFGamma}
\title{Increased Limit Factor Curve from a Gamma severity distribution}
\usage{
ILFGamma(xLow, xHigh, shape, rate)
}
\arguments{
\item{xLow}{A positive real number -  the claim amount where the Increased Limit Factor Curve will be evaluated from.}

\item{xHigh}{A positive real number -  the claim amount where the Increased Limit Factor Curve will be evaluated to.}

\item{shape}{A positive real number - the shape parameter of the Claim Severity's Gamma distribution.}

\item{rate}{A positive real number - the rate parameter of the Claim Severity's Gamma distribution.}
}
\value{
The value of the Increased Limit Factor curve from \code{xLow} to \code{xHigh} with Claim Severity from a Gamma distribution with parameters \code{shape} and \code{rate}.
}
\description{
Increased Limit Factor Curve from a Gamma severity distribution
}
\examples{
ILFGamma(1000,700,1,0.0005)
ILFGamma(1200,1000,1.5,0.0006)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/NetSimR.R
\docType{package}
\name{NetSimR-package}
\alias{NetSimR}
\alias{NetSimR-package}
\title{NetSimR: A non-life insurance package for computating various statistics.}
\description{
The NetSimR package provides three categories of functions:
\enumerate{
\item Capped means, Exposure and ILF curve from various severity distributions
\item Pure IBNR and UPR earned periods
\item Sliced distributions
}
}
\section{NetSimR mean functions}{

\code{\link{SlicedGammaParetoMean}}
\code{\link{SlicedLNormParetoMean}}
}

\section{NetSimR capped mean functions}{

\code{\link{GammaCappedMean}}
\code{\link{LNormCappedMean}}
\code{\link{ParetoCappedMean}}
\code{\link{SlicedGammaParetoCappedMean}}
\code{\link{SlicedLNormParetoCappedMean}}
}

\section{NetSimR exposure curve functions}{

\code{\link{ExposureCurveGamma}}
\code{\link{ExposureCurveLNorm}}
\code{\link{ExposureCurvePareto}}
\code{\link{ExposureCurveSlicedGammaPareto}}
\code{\link{ExposureCurveSlicedLNormPareto}}
}

\section{NetSimR ILF curve functions}{

\code{\link{ILFGamma}}
\code{\link{ILFLNorm}}
\code{\link{ILFPareto}}
\code{\link{ILFSlicedGammaPareto}}
\code{\link{ILFSlicedLNormPareto}}
}

\section{NetSimR pure IBNR functions}{

\code{\link{PureIBNRGamma}}
\code{\link{PureIBNRLNorm}}
}

\section{NetSimR Sliced distribution functions}{

\code{\link{dSlicedGammaPareto}}
\code{\link{dSlicedLNormPareto}}
\code{\link{pSlicedGammaPareto}}
\code{\link{pSlicedLNormPareto}}
\code{\link{qSlicedGammaPareto}}
\code{\link{qSlicedLNormPareto}}
}

\author{
\strong{Maintainer}: Yiannis Parizas \email{yiannis.parizas@gmail.com}

Authors:
\itemize{
  \item Yiannis Parizas \email{yiannis.parizas@gmail.com}
}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/distribution_fitting_tool_Server.R
\name{distribution_fitting_tool_Server}
\alias{distribution_fitting_tool_Server}
\title{Server function for the Distribution Fitting tool application}
\usage{
distribution_fitting_tool_Server(input, output, session)
}
\arguments{
\item{input}{Input for the server function.}

\item{output}{Output for the server function.}

\item{session}{Session for the server function.}
}
\value{
Returns server rendering for the shiny application.
}
\description{
Server function for the Distribution Fitting tool application
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ShinySimulatorGlobal.R
\name{simulate_function}
\alias{simulate_function}
\title{A function to simulate frequency - severity of insurance claims using chunked vectorisation.
The function applies severity cap, reinsurance structure for each and every loss claim,
reinsurance structure for aggregate claims, and allows for piecewise pareto slices.}
\usage{
simulate_function(
  numOfSimulations,
  freq_params,
  sev_params,
  seedSetBinary,
  seedValue,
  freqDistr,
  sevDistr,
  paretoSlice,
  pareto_slice_times,
  slice_pareto_alphas,
  slice_pareto_x_ms,
  sevCapBinary,
  sev_cap_amount,
  reinsuranceStructureEEL,
  reinsurance_structure_eel_dedctible_amount,
  reinsurance_structure_eel_limit_amount,
  reinsuranceStructureAL,
  reinsurance_structure_al_dedctible_amount,
  reinsurance_structure_al_limit_amount,
  reinsuranceStructureLimitedReinstatements,
  reinsuranceStructureReinstatementLimit,
  multiprocessing,
  chunk_size = 10000
)
}
\arguments{
\item{numOfSimulations}{The number of simulations to run.}

\item{freq_params}{A vector of the frequency distribution parameters.}

\item{sev_params}{A vector of the severity distribution parameters.}

\item{seedSetBinary}{True if there is a fixed seed, otherwise false.}

\item{seedValue}{The seed value.}

\item{freqDistr}{The frequency distribution. Options are as per the freq_dist_options.}

\item{sevDistr}{The severity distribution. Options are as per the sev_dist_options.}

\item{paretoSlice}{True if there is Pareto slicing.}

\item{pareto_slice_times}{The number of Pareto slices.}

\item{slice_pareto_alphas}{A vector of Pareto slices' alpha parameters.}

\item{slice_pareto_x_ms}{A vector of Pareto slices' x_m parameters.}

\item{sevCapBinary}{True if there is a severity cap.}

\item{sev_cap_amount}{The severity cap amount.}

\item{reinsuranceStructureEEL}{The chosen reinsurance structure for each and every loss claim.}

\item{reinsurance_structure_eel_dedctible_amount}{The deductible for each and every loss reinsurance structure.}

\item{reinsurance_structure_eel_limit_amount}{The limit for each and every loss reinsurance structure.}

\item{reinsuranceStructureAL}{The chosen reinsurance structure for aggregate claims.}

\item{reinsurance_structure_al_dedctible_amount}{The deductible for aggregate reinsurance structure.}

\item{reinsurance_structure_al_limit_amount}{The limit for aggregate reinsurance structure.}

\item{reinsuranceStructureLimitedReinstatements}{True if there is a limit in reinstatements, otherwise false.}

\item{reinsuranceStructureReinstatementLimit}{The reinstatement limit.}

\item{multiprocessing}{True if multiprocessing is used, otherwise false.}

\item{chunk_size}{The number of simulations processed per vectorised batch. Defaults to 10000.}
}
\value{
A data frame with claims counts, ceded claims and the number of reinstatements used.
}
\description{
A function to simulate frequency - severity of insurance claims using chunked vectorisation.
The function applies severity cap, reinsurance structure for each and every loss claim,
reinsurance structure for aggregate claims, and allows for piecewise pareto slices.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ShinySimulatorGlobal.R
\name{run_shiny_simulator}
\alias{run_shiny_simulator}
\title{A function to run the shiny simulator application}
\usage{
run_shiny_simulator()
}
\value{
Opens the shiny simulator application
}
\description{
A function to run the shiny simulator application
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Gamma.R
\name{GammaCappedMean}
\alias{GammaCappedMean}
\title{Gamma capped mean}
\usage{
GammaCappedMean(cap, shape, rate)
}
\arguments{
\item{cap}{A positive real number -  the claim severity cap.}

\item{shape}{A positive real number - the shape parameter of the Claim Severity's Gamma distribution.}

\item{rate}{A positive real number - the rate parameter of the Claim Severity's Gamma distribution.}
}
\value{
The mean of the claim severity capped at \code{cap} with a Gamma distribution with parameters \code{shape} and \code{rate}.
}
\description{
Gamma capped mean
}
\examples{
GammaCappedMean(700,1,0.0005)
GammaCappedMean(1000,1.5,0.0006)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Pareto.R
\name{ILFPareto}
\alias{ILFPareto}
\title{Increased Limit Factor Curve from a Pareto severity distribution}
\usage{
ILFPareto(xLow, xHigh, scale, shape)
}
\arguments{
\item{xLow}{A positive real number -  the claim amount where the Increased Limit Factor Curve will be evaluated from.}

\item{xHigh}{A positive real number -  the claim amount where the Increased Limit Factor Curve will be evaluated to.}

\item{scale}{A positive real number - the scale parameter of the Claim Severity's Pareto distribution.}

\item{shape}{A positive real number - the shape parameter of the Claim Severity's Pareto distribution.}
}
\value{
The value of the Increased Limit Factor curve from \code{xLow} to \code{xHigh} with Claim Severity from a Pareto distribution with parameters \code{scale} and \code{shape}.
}
\description{
Increased Limit Factor Curve from a Pareto severity distribution
}
\examples{
ILFPareto(700,1200,500,1.2)
ILFPareto(1200,20000,200,1.1)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GLMFittingToolGlobal.R
\name{run_shiny_glm_fitting_tool}
\alias{run_shiny_glm_fitting_tool}
\title{A function to run the glm fitting tool application}
\usage{
run_shiny_glm_fitting_tool()
}
\value{
Opens the glm fitting tool application
}
\description{
A function to run the glm fitting tool application
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Gamma.R
\name{ExposureCurveGamma}
\alias{ExposureCurveGamma}
\title{Exposure Curve from a Gamma severity distribution}
\usage{
ExposureCurveGamma(x, shape, rate)
}
\arguments{
\item{x}{A positive real number -  the claim amount where the exposure curve will be evaluated.}

\item{shape}{A positive real number - the shape parameter of the Claim Severity's Gamma distribution.}

\item{rate}{A positive real number - the rate parameter of the Claim Severity's Gamma distribution.}
}
\value{
The value of the Exposure curve at \code{x} with Claim Severity from a Gamma distribution with parameters \code{shape} and \code{rate}.
}
\description{
Exposure Curve from a Gamma severity distribution
}
\examples{
ExposureCurveGamma(700,1,0.0005)
ExposureCurveGamma(1000,1.5,0.0006)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/LogNormal.R
\name{erf}
\alias{erf}
\title{Error function}
\usage{
erf(x)
}
\arguments{
\item{x}{A real number.}
}
\value{
The value of the error function at \code{x}.
}
\description{
Error function
}
\examples{
erf(0.1)
erf(0.5)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/LogNormal.R
\name{ILFLNorm}
\alias{ILFLNorm}
\title{Increased Limit Factor Curve from a LogNormal severity distribution}
\usage{
ILFLNorm(xLow, xHigh, mu, sigma)
}
\arguments{
\item{xLow}{A positive real number -  the claim amount where the Increased Limit Factor Curve will be evaluated from.}

\item{xHigh}{A positive real number -  the claim amount where the Increased Limit Factor Curve will be evaluated to.}

\item{mu}{A real number - the first parameter of the Claim Severity's LogNormal distribution.}

\item{sigma}{A positive real number - the second parameter of the Claim Severity's LogNormal distribution.}
}
\value{
The value of the Increased Limit Factor curve from \code{xLow} to \code{xHigh} with Claim Severity from a LogNormal distribution with parameters \code{mu} and \code{sigma}.
}
\description{
Increased Limit Factor Curve from a LogNormal severity distribution
}
\examples{
ILFLNorm(1000,2000,6,1.5)
ILFLNorm(1000,1500,5,1.6)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ShinySimulatorGlobal.R
\name{max_number_of_pareto_slices}
\alias{max_number_of_pareto_slices}
\title{Parameter to set the maximum number of pareto slices}
\usage{
max_number_of_pareto_slices
}
\value{
The the maximum number of Pareto Slices.
}
\description{
Parameter to set the maximum number of pareto slices
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Pareto.R
\name{ParetoCappedMean}
\alias{ParetoCappedMean}
\title{Pareto capped mean}
\usage{
ParetoCappedMean(cap, scale, shape)
}
\arguments{
\item{cap}{A positive real number -  the claim severity cap.}

\item{scale}{A positive real number - the scale parameter of the Claim Severity's Pareto distribution.}

\item{shape}{A positive real number - the shape parameter of the Claim Severity's Pareto distribution.}
}
\value{
The mean of the claim severity capped at \code{cap} with a Pareto distribution with parameters \code{scale} and \code{shape}.
}
\description{
Pareto capped mean
}
\examples{
ParetoCappedMean(600,200,1.2)
ParetoCappedMean(800,100,1)
ParetoCappedMean(1000,500,0.8)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Pareto.R
\name{ExposureCurvePareto}
\alias{ExposureCurvePareto}
\title{Exposure Curve from a Pareto severity distribution}
\usage{
ExposureCurvePareto(x, scale, shape)
}
\arguments{
\item{x}{A positive real number -  the claim amount where the exposure curve will be evaluated.}

\item{scale}{A positive real number - the scale parameter of the Claim Severity's Pareto distribution.}

\item{shape}{A positive real number - the shape parameter of the Claim Severity's Pareto distribution.}
}
\value{
The value of the Exposure curve at \code{x} with Claim Severity from a Pareto distribution with parameters \code{scale} and \code{shape}.
}
\description{
Exposure Curve from a Pareto severity distribution
}
\examples{
ExposureCurvePareto(700,500,1.2)
ExposureCurvePareto(20000,200,1.1)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/SlicecdGammaPareto.R
\name{qSlicedGammaPareto}
\alias{qSlicedGammaPareto}
\title{The inverse cumulative density function of a Sliced Gamma Pareto severity distribution}
\usage{
qSlicedGammaPareto(q, GShape, GRate, SlicePoint, PShape)
}
\arguments{
\item{q}{A real number between 0 and 1 -  the probability where the inverse cumulative density function will be evaluated.}

\item{GShape}{A positive real number -  the shape parameter of the attritional Claim Severity's Gamma distribution.}

\item{GRate}{A positive real number -  the rate parameter of the attritional Claim Severity's Gamma distribution.}

\item{SlicePoint}{A positive real number - the slice point and the scale parameter of the tail Claim Severity's Pareto distribution.}

\item{PShape}{A positive real number - the shape parameter of the tail Claim Severity's Pareto distribution.}
}
\value{
The value of the inverse cumulative density function at \code{q} with an attritional claim Gamma distribution with parameters \code{GShape} and \code{GRate} and a large claim Pareto distribution with parameters \code{SlicePoint} and \code{PShape}.
}
\description{
The inverse cumulative density function of a Sliced Gamma Pareto severity distribution
}
\examples{
qSlicedGammaPareto(0.5,1,0.0005,1000,1.2)
qSlicedGammaPareto(0.2,1.1,0.0006,2000,1.6)
qSlicedGammaPareto(0.8,1.2,0.0004,3000,1.4)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Gamma.R
\name{IGamma}
\alias{IGamma}
\title{Lower incomplete gamma function}
\usage{
IGamma(a, x)
}
\arguments{
\item{a}{A positive real number.}

\item{x}{A positive real number.}
}
\value{
The value of the lower incomplete gamma function at \code{x} with shape parameter \code{a}.
}
\description{
Lower incomplete gamma function
}
\examples{
IGamma(1,1)
IGamma(0.1,2)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ShinySimulatorUI.R
\name{shiny_simulator_ui}
\alias{shiny_simulator_ui}
\title{UI file for the Shiny NetSimR Simulator Tool}
\usage{
shiny_simulator_ui
}
\value{
Returns the UI code for the shiny application.
}
\description{
UI file for the Shiny NetSimR Simulator Tool
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/LogNormal.R
\name{ExposureCurveLNorm}
\alias{ExposureCurveLNorm}
\title{Exposure Curve from LogNormal a severity distribution}
\usage{
ExposureCurveLNorm(x, mu, sigma)
}
\arguments{
\item{x}{A positive real number -  the claim amount where the exposure curve will be evaluated.}

\item{mu}{A real number - the first parameter of the Claim Severity's LogNormal distribution.}

\item{sigma}{A positive real number - the second parameter of the Claim Severity's LogNormal distribution.}
}
\value{
The value of the Exposure curve at \code{x} with Claim Severity from a LogNormal distribution with parameters \code{mu} and \code{sigma}.
}
\description{
Exposure Curve from LogNormal a severity distribution
}
\examples{
ExposureCurveLNorm(2000,6,1.5)
ExposureCurveLNorm(1000,5,1.6)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/SlicecdLogNormalPareto.R
\name{ILFSlicedLNormPareto}
\alias{ILFSlicedLNormPareto}
\title{Increased Limit Factor Curve from a Sliced LogNormal Pareto severity distribution}
\usage{
ILFSlicedLNormPareto(xLow, xHigh, mu, sigma, SlicePoint, shape)
}
\arguments{
\item{xLow}{A positive real number -  the claim amount where the Limit Factor Curve will be evaluated from.}

\item{xHigh}{A positive real number -  the claim amount where the Limit Factor Curve will be evaluated to.}

\item{mu}{A real number -  the first parameter of the attritional Claim Severity's LogNormal distribution.}

\item{sigma}{A positive real number -  the second parameter of the attritional Claim Severity's LogNormal distribution.}

\item{SlicePoint}{A positive real number - the slice point and the scale parameter of the tail Claim Severity's Pareto distribution.}

\item{shape}{A positive real number - the shape parameter of the tail Claim Severity's Pareto distribution.}
}
\value{
The value of the Increased Limit Factor curve from \code{xLow} to \code{xHigh} with an attritional claim LogNormal distribution with parameters \code{mu} and \code{sigma} and a large claim Pareto distribution with parameters \code{SlicePoint} and \code{shape}.
}
\description{
Increased Limit Factor Curve from a Sliced LogNormal Pareto severity distribution
}
\examples{
ILFSlicedLNormPareto(800,1200,6,1.5,1000,1.2)
ILFSlicedLNormPareto(2000,4000,7,1.6,3000,1.4)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ShinySimulatorGlobal.R
\name{freq_dist_options}
\alias{freq_dist_options}
\title{A vector with the frequency distribution objects}
\usage{
freq_dist_options
}
\value{
The frequency distribution objects.
}
\description{
A vector with the frequency distribution objects
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ShinySimulatorGlobal.R
\name{apply_deductible_limit}
\alias{apply_deductible_limit}
\title{Apply a deductible and limit to claims}
\usage{
apply_deductible_limit(
  gross_claims_data,
  reinsurance_structure,
  deductible,
  limit
)
}
\arguments{
\item{gross_claims_data}{A vector of Claims.}

\item{reinsurance_structure}{The chosen reinsurance structure. Options are: 'No Reinsurance Structure', 'Unlimited Layer', 'Limited Layer', 'Exclude Layer'.}

\item{deductible}{The deductible of the reinsurance structure.}

\item{limit}{The limit of the reinsurance structure.}
}
\value{
The ceded claims for the structure, with the chosen deductible and limit.
}
\description{
Apply a deductible and limit to claims
}
\examples{
apply_deductible_limit(c(100, 50, 20), 'Limited Layer', 40, 20)
apply_deductible_limit(c(100, 50, 20), 'Limited Layer', 10, 30)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Gamma.R
\name{PureIBNRGamma}
\alias{PureIBNRGamma}
\title{Pure IBNR exposure from a Gamma reporting delay distribution}
\usage{
PureIBNRGamma(IncDate, ExpDate, ValDate, shape, rate)
}
\arguments{
\item{IncDate}{A date -  the inception date of the period.}

\item{ExpDate}{A date -  the expiry date of the period. Must be greater than inception date.}

\item{ValDate}{A date -  the valuation date.}

\item{shape}{A positive real number - the shape parameter of the reporing delay's Gamma distribution.}

\item{rate}{A positive real number - the rate parameter of the reporing delay's Gamma distribution.}
}
\value{
Unearned and Pure IBNR exposure in days and as a percentage of the period's duration, where the reporting delay has a Gamma distribution with parameters \code{shape} and \code{rate}.
}
\description{
Pure IBNR exposure from a Gamma reporting delay distribution
}
\examples{
Dates = data.frame(
    inceptionDate = c("01/01/2006", "01/07/2006", "01/01/2007")
    ,expiryDate = c("31/12/2006", "30/06/2007", "31/12/2007")
)

Dates$inceptionDate<-as.POSIXct(Dates$inceptionDate, format="\%d/\%m/\%Y")

Dates$expiryDate<-as.POSIXct(Dates$expiryDate, format="\%d/\%m/\%Y")

ValuationDate<-as.POSIXct("30/10/2007", format="\%d/\%m/\%Y")

PureIBNRGamma(Dates$inceptionDate,Dates$expiryDate,ValuationDate,7,0.15)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/SlicecdLogNormalPareto.R
\name{SlicedLNormParetoMean}
\alias{SlicedLNormParetoMean}
\title{Sliced LogNormal Pareto mean}
\usage{
SlicedLNormParetoMean(mu, sigma, SlicePoint, shape)
}
\arguments{
\item{mu}{A real number -  the first parameter of the attritional Claim Severity's LogNormal distribution.}

\item{sigma}{A positive real number -  the second parameter of the attritional Claim Severity's LogNormal distribution.}

\item{SlicePoint}{A positive real number - the slice point and the scale parameter of the tail Claim Severity's Pareto distribution.}

\item{shape}{A positive real number - the shape parameter of the tail Claim Severity's Pareto distribution.}
}
\value{
The mean of the claim severity with an attritional claim LogNormal distribution with parameters \code{mu} and \code{sigma} and a large claim Pareto distribution with parameters \code{SlicePoint} and \code{shape}.
}
\description{
Sliced LogNormal Pareto mean
}
\examples{
SlicedLNormParetoMean(6,1.5,1000,1.2)
SlicedLNormParetoMean(6.5,1.4,2000,1.6)
SlicedLNormParetoMean(7,1.6,3000,1.4)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ShinySimulatorGlobal.R
\name{freq_dist_parameter_placeholders}
\alias{freq_dist_parameter_placeholders}
\title{A data frame with the frequency distribution parameter placeholders}
\usage{
freq_dist_parameter_placeholders
}
\value{
The frequency distribution parameter placeholders.
}
\description{
A data frame with the frequency distribution parameter placeholders
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ShinySimulatorGlobal.R
\name{sev_dist_parameter_placeholders}
\alias{sev_dist_parameter_placeholders}
\title{A data frame with the severity distribution parameter placeholders}
\usage{
sev_dist_parameter_placeholders
}
\value{
The severity distribution parameter placeholders.
}
\description{
A data frame with the severity distribution parameter placeholders
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ShinySimulatorGlobal.R
\docType{class}
\name{distributionClass-class}
\alias{distributionClass-class}
\alias{distributionClass}
\title{The class of the distribution objects}
\description{
The class of the distribution objects
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/LogNormal.R
\name{PureIBNRLNorm}
\alias{PureIBNRLNorm}
\title{Pure IBNR exposure from a LogNormal reporting delay distribution}
\usage{
PureIBNRLNorm(IncDate, ExpDate, ValDate, mu, sigma)
}
\arguments{
\item{IncDate}{A date -  the inception date of the period.}

\item{ExpDate}{A date -  the expiry date of the period. Must be greater than inception date.}

\item{ValDate}{A date -  the valuation date.}

\item{mu}{A real number - the first parameter of the reporing delay's LogNormal distribution.}

\item{sigma}{A positive real number - the second parameter of the reporing delay's LogNormal distribution.}
}
\value{
Unearned and Pure IBNR exposure in days and as a percentage of the period's duration, where the reporting delay has a LogNormal distribution with parameters \code{mu} and \code{sigma}.
}
\description{
Pure IBNR exposure from a LogNormal reporting delay distribution
}
\examples{
Dates = data.frame(
    inceptionDate = c("01/01/2006", "01/07/2006", "01/01/2007")
    ,expiryDate = c("31/12/2006", "30/06/2007", "31/12/2007")
)

Dates$inceptionDate<-as.POSIXct(Dates$inceptionDate, format="\%d/\%m/\%Y")

Dates$expiryDate<-as.POSIXct(Dates$expiryDate, format="\%d/\%m/\%Y")

ValuationDate<-as.POSIXct("30/10/2007", format="\%d/\%m/\%Y")

PureIBNRLNorm(Dates$inceptionDate,Dates$expiryDate,ValuationDate,4,1.5)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ShinySimulatorGlobal.R
\name{rpareto}
\alias{rpareto}
\title{Random Pareto generator}
\usage{
rpareto(n, alpha, x_m)
}
\arguments{
\item{n}{Number of values to generate.}

\item{alpha}{A positive real number. Alpha parameter of the Pareto distribution.}

\item{x_m}{A positive real number. The minimum value for the Pareto distribution.}
}
\value{
A vector of \code{n} random Pareto variables with parameters \code{alpha} and \code{x_m}.
}
\description{
Random Pareto generator
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ShinySimulatorServer.R
\name{shiny_simulator_server}
\alias{shiny_simulator_server}
\title{Server function for the Shiny Simulator application}
\usage{
shiny_simulator_server(input, output, session)
}
\arguments{
\item{input}{Input for the server function.}

\item{output}{Output for the server function.}

\item{session}{Session for the server function.}
}
\value{
Returns server rendering for the shiny application.
}
\description{
Server function for the Shiny Simulator application
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/distribution_fitting_tool_UI.R
\name{distribution_fitting_tool_UI}
\alias{distribution_fitting_tool_UI}
\title{UI file for the Shiny glm fitting tool}
\usage{
distribution_fitting_tool_UI
}
\value{
Returns the UI code for the shiny application.
}
\description{
UI file for the Shiny glm fitting tool
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Pareto.R
\name{ParetoCappedMeanCalc}
\alias{ParetoCappedMeanCalc}
\title{Pareto capped mean intermediary calculation}
\usage{
ParetoCappedMeanCalc(cap, scale, shape)
}
\arguments{
\item{cap}{A positive real number -  the claim severity cap.}

\item{scale}{A positive real number - the scale parameter of the Claim Severity's Pareto distribution.}

\item{shape}{A positive real number - the shape parameter of the Claim Severity's Pareto distribution.}
}
\value{
An interim calculation for the mean of the claim severity capped at \code{cap} with a Pareto distribution with parameters \code{scale} and \code{shape}.
}
\description{
Pareto capped mean intermediary calculation
}
\examples{
ParetoCappedMeanCalc(800,100,1.1)
ParetoCappedMeanCalc(1000,500,0.9)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GLMFittingToolServer.R
\name{GLMFittingToolServer}
\alias{GLMFittingToolServer}
\title{Server function for the GLM Fitting tool application}
\usage{
GLMFittingToolServer(input, output, session)
}
\arguments{
\item{input}{Input for the server function.}

\item{output}{Output for the server function.}

\item{session}{Session for the server function.}
}
\value{
Returns server rendering for the shiny application.
}
\description{
Server function for the GLM Fitting tool application
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ShinySimulatorGlobal.R
\name{sev_dist_options}
\alias{sev_dist_options}
\title{A vector with the severity distribution objects}
\usage{
sev_dist_options
}
\value{
The severity distribution objects.
}
\description{
A vector with the severity distribution objects
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ShinySimulatorGlobal.R
\name{apply_severity_cap}
\alias{apply_severity_cap}
\title{Apply severity cap function}
\usage{
apply_severity_cap(claims, severity_cap_boolean, severity_cap_amount)
}
\arguments{
\item{claims}{A vector of Claims.}

\item{severity_cap_boolean}{A variable that if true, the function will cap the claims, otherwise will just return them.}

\item{severity_cap_amount}{The claim cap value.}
}
\value{
If \code{severity_cap_boolean} is true, then will return the minimum of \code{severity_cap_amount} or \code{claims} otherwise will return \code{claims}. The operation is vectorised.
}
\description{
Apply severity cap function
}
